--
-- Author: KK
-- Date: 2014-09-22 14:47:03
--
--任务类

--[[
TaskType
1、建造X个X炮塔
2、使用X炮塔消灭X个怪物
3、清除X个X道具
4、X秒内获得X金钱
5、X秒内清除X个X道具
6、关卡中X炮塔的数量不超过X个
7、使用大技能消灭X个敌人
8、关卡中炮塔的总数不超过X个
9、X秒内清除所有道具
10、造X个满级的X炮塔
11、使用X炮塔消灭X个X怪物
12、使用X炮塔清除X个X道具
13、在X秒内使用X炮塔清除X个X道具
14、用英雄杀死X只怪
15、清除所有道具（固定任务）
--]]

local MapInfo = import("..views.MapInfo")

local Task = class("Task")
function Task:ctor(playScene, levelTasks)
	self.playScene = playScene
	self.level = playScene.level
	--显示的层
	self:initShowLayer()

	self.tasks = {}
	for i = 1, #levelTasks do
		self.tasks[i] = {
			idx = i,
			complete = GameData.levelTask[""..self.level][""..i]
		}
		for k,v in pairs(levelTasks[i]) do
			--包括类型和数值参数
			self.tasks[i][k] = v
			if k == "thingCount"
				or k == "towerCount"
				or k == "timeCount"
				or k == "enemyCount" 
				or k == "money" then
				--数值变量
				self.tasks[i]["my"..k] = 0
			end
		end
		--所有道具，要先取总数
		if self.tasks[i].type == 9 then
			--print("normalThingCount:",MapInfo.things.normalThingCount)
			self.tasks[i].thingCount = MapInfo.things.normalThingCount
			self.tasks[i].mythingCount = 0
		end
	end
	--固定的第四个任务清除所有道具
	local idx = #levelTasks + 1
	self.tasks[#self.tasks+1] = {
		idx = idx,
		complete = GameData.levelTask[""..self.level][""..idx],
		type = 15,
		thingCount = MapInfo.things.normalThingCount,
		mythingCount = 0,
	}
	
	--哪些需要检查
	self:checkFlag()

	--计时
    self.lastCheckTime = 0
	--dump(self.tasks)
end

function Task:clear()
	--self.taskLayer:removeFromParent() --先添加后删除。
	self.tasks = nil
	self.taskLayer = nil
end

function Task:checkFlag()
	self.flag = {}
	--清除道具都需要
	self.flag.checkThing = true
	for i = 1, #self.tasks do
		local type = self.tasks[i].type
		--是否需要各项检查
		if type == 1 or type == 6 or type == 8 then
			self.flag.checkCreateFort = true
		end
		if type == 2 or type == 7 or type == 11 or type == 14 then
			self.flag.checkEnemy = true
		end
		if type == 4 then
			self.flag.checkTime = true
			self.flag.checkMoney = true
		end
		if type == 5 or type == 9 or type == 13 then
			self.flag.checkTime = true
		end
		if type == 10 then
			self.flag.checkTowerUpgrade = true
		end
	end
end

function Task:initShowLayer()
	self.taskLayer = display.newLayer()
	local grayBg = display.newColorLayer(cc.c4b(0,0,0,128))
	self.taskLayer:addChild(grayBg)
	self.showLayer = display.newSprite("#taskBg.png", display.cx, display.cy)
	self.taskLayer:addChild(self.showLayer)	
	self.showNode = display.newNode()
	self.showLayer:addChild(self.showNode)
	self.playScene:addChild(self.taskLayer,5000)
	self.taskLayer:setVisible(false)
	self.taskLayer:setTouchEnabled(true)
	self.taskLayer:setTouchSwallowEnabled(true)
	local bx
	local by
	if VERSION_MONSTER then
		bx = 743
		by = 413
	else
		bx = 690
		by = 420
	end	
    self.closeButton = cc.ui.UIPushButton.new({
        normal = "#taskClose.png",
        })
        :onButtonPressed(function()
            self.closeButton:setScale(1.05)
            --音效
            SoundManager.playSound(GAME_SOUND.Button2)
        end)
        :onButtonRelease(function()
            self.closeButton:setScale(1)
        end)
        :onButtonClicked(function(event)
            self:hide()
        end)
        :pos(bx, by)
        :addTo(self.showLayer)
end

function Task:show()
	self.paused = false
	if not self.playScene.pause then 
		self.paused = true
		self.playScene:suspend(true)
	end

	self.taskLayer:setVisible(true)

	self.showNode:removeAllChildren()
	for i = 1, #self.tasks-1 do
		local layer = self:getSingleTaskShow(self.tasks[i], i)
		local pos
		if VERSION_MONSTER then
			pos = cc.p(420 + (i-2)*250, 200)
		else
			pos = cc.p(355 + (i-2)*215, 200)
		end
		layer:setPosition(pos)
		self.showNode:addChild(layer)
	end
	--dump(self.tasks)
end

function Task:hide()
	if self.paused then
		self.playScene:suspend(true)
	end
	self.taskLayer:setVisible(false)
end

function Task:getTaskText(task)
	if task.type == 1 then
		text = "建造"..task.towerCount.."个"..TowersData[task.towerIdx].name
		--[[local name = TowersData[task.towerIdx].nameEnglish
		if task.towerCount > 1 then
			name = TowersData[task.towerIdx].nameEnglish.."s"
		end]]
		textEnglish = "Build "..task.towerCount.." "..TowersData[task.towerIdx].nameEnglish
	elseif task.type == 2 then
		text = "使用"..TowersData[task.towerIdx].name.."消灭"..task.enemyCount.."个怪物"
		textEnglish = "Kill "..task.enemyCount.." monsters by using "..TowersData[task.towerIdx].nameEnglish
	elseif task.type == 3 then
		text = "清除"..task.thingCount.."个"..ThingData[task.thingIdx].name
		textEnglish = "Destroy "..task.thingCount.." "..ThingData[task.thingIdx].nameEnglish
	elseif task.type == 4 then
		text = task.timeCount.."秒内获得"..task.money.."圣水"
		textEnglish = "Earn "..task.money.." elixir in "..task.timeCount.."s seconds"
	elseif task.type == 5 then
		text = task.timeCount.."秒内清除"..task.thingCount.."个"..ThingData[task.thingIdx].name
		textEnglish = "Destroy "..task.thingCount.." "..ThingData[task.thingIdx].nameEnglish.." in "..task.timeCount.."s seconds"
	elseif task.type == 6 then
		text = "关卡中"..TowersData[task.towerIdx].name.."的数量不超过"..task.towerCount.."个"
		textEnglish = TowersData[task.towerIdx].nameEnglish.." can’t be built more than "..task.towerCount.." in this level"
	elseif task.type == 7 then
		text = "使用大技能消灭"..task.enemyCount.."个怪物"
		textEnglish = "Kill "..task.enemyCount.." monsters by using the skills of hero"
	elseif task.type == 8 then
		text = "关卡中炮塔的总数不超过"..task.towerCount.."个"
		textEnglish = "The total of the towers no more than "..task.towerCount
	elseif task.type == 9 then
		text = task.timeCount.."秒内清除所有道具"
		textEnglish = "Destroy all theobstacles in "..task.timeCount.."s seconds"
	elseif task.type == 10 then
		text = "建造"..task.towerCount.."个满级的"..TowersData[task.towerIdx].name
		textEnglish = "Build "..task.towerCount.." "..TowersData[task.towerIdx].nameEnglish.." to its level 3"
	elseif task.type == 11 then
		text = "使用"..TowersData[task.towerIdx].name.."消灭"..task.enemyCount.."个"..EnemyData[task.enemyIdx].name
		textEnglish = "Kill "..task.enemyCount.." "..EnemyData[task.enemyIdx].name.." by using "..TowersData[task.towerIdx].nameEnglish
	elseif task.type == 12 then
		text = "使用"..TowersData[task.towerIdx].name.."清除"..task.thingCount.."个"..ThingData[task.thingIdx].name
		textEnglish = "Destroy "..task.thingCount.." "..ThingData[task.thingIdx].nameEnglish.." by using "..TowersData[task.towerIdx].nameEnglish
	elseif task.type == 13 then
		text = task.timeCount.."秒内使用"..TowersData[task.towerIdx].name.."清除"..task.thingCount.."个"..ThingData[task.thingIdx].name
		textEnglish = "Destroy "..task.thingCount.." "..ThingData[task.thingIdx].nameEnglish.." by using "..TowersData[task.towerIdx].nameEnglish.." in "..task.timeCount.."s seconds"
	elseif task.type == 14 then
		text = "用英雄消灭"..task.enemyCount.."个怪物"
		textEnglish = "Kill "..task.enemyCount.." monsters by using hero"
	end
	if VERSION_REDUCE then
		return textEnglish
	else
		return text
	end
end

function Task:getSingleTaskShow(task, idx)
	--local layer = display.newSprite("#taskSubBg.png")
	local layer = display.newNode()
	--local midX = 105
	local midX,mesX
	local countTitleX = -35
	local countNumberX = 50
	local labelY = -120
	local label1Y = -100
	local label2Y = -130
	local msgY = 33
	local PicY = 91
	local fontcolor
	local fontcolor2
	local fontType
	if VERSION_MONSTER then
		midX = 0
		mesX = 0
		msgY = 33
		PicY = 91
		fontcolor = cc.c3b(255,216,95)
		fontcolor2 = cc.c3b(255,240,0)
		fontType = "fontCommon.TTF"
	else
		midX = 0
		mesX = -80
		msgY = 35
		PicY = 115
		fontcolor = cc.c3b(104,61,0)
		fontcolor2 = cc.c3b(138,89,20)
		fontType = "fontCommon.TTF"
	end
	if VERSION_REDUCE then
		midX = 0
		mesX = -80
		msgY = 73
		PicY = 115
		fontcolor = cc.c3b(104,61,0)
		fontcolor2 = cc.c3b(138,89,20)
		fontType = "fontEasy.ttf"
	end
	local taskIcon
	if task.type == 1 or task.type == 2 or task.type == 6 or task.type == 10 or task.type == 11 then--炮塔
		taskIcon = display.newSprite("#taskTowerIcon"..task.towerIdx..".png", midX, PicY):addTo(layer)
	elseif task.type == 8 then--所有炮塔
		taskIcon = display.newSprite("#taskTowerIcon1.png", midX, PicY):addTo(layer)
	elseif task.type == 7 or task.type == 14 then--英雄
		taskIcon = display.newSprite("#taskHeroIcon"..g_playScene.hero.idx..".png", midX, PicY):addTo(layer)
	elseif task.type == 3 or task.type == 5 or task.type == 9 or task.type ==  12 or task.type ==  13 then--道具
		taskIcon = display.newSprite("#taskThingIcon.png", midX, PicY):addTo(layer)
	elseif task.type == 4 then--金钱
		taskIcon = display.newSprite("#taskMoneyIcon.png", midX, PicY):addTo(layer)
	end	
	--小图标
	--local littlePic = display.newSprite("#"..TaskTypeData[task.type].littlePic, 80, 20):addTo(taskIcon)
	--coinPic
	if VERSION_MONSTER then
		local coinPic = display.newSprite("#taskPlaySceneCoin.png", midX-58, -37):addTo(layer)
	end

	if ExcelTaskReward[LevelsData.getRealLevel(self.level)] then
		local ny 
		if VERSION_MONSTER then
			ny = -40
		else
			ny = -47
		end
		local number = cc.ui.UILabel.new({
	        UILabelType = 1,
	        font = "fontTitleNumber.fnt",
			text = ""..ExcelTaskReward[LevelsData.getRealLevel(self.level)][idx],
			x = midX+15,
			y = ny,
		}):align(display.CENTER):addTo(layer)
	end
	if task.type == 1 then
		--CCLabelAtlas:create
		local l1
		if VERSION_REDUCE then
			l1 = cc.ui.UILabel.new({
				UILabelType = 2,
				font = fontType,	
				text = self:getTaskText(task),
				size = 17,
				dimensions = cc.size(168, 150),
				color = fontcolor,
				x = mesX,
				y = msgY,
			}):align(display.LEFT_TOP)
		else
			l1 = cc.ui.UILabel.new({
				UILabelType = 2,
				font = fontType,	
				text = self:getTaskText(task),
				size = 22,
				dimensions = cc.size(168, 85),
				align = cc.ui.TEXT_ALIGN_CENTER,
				valign = cc.ui.TEXT_VALIGN_CENTER,
				color = fontcolor,
				x = mesX,
				y = msgY,
			})
		end
		layer:addChild(l1)
		if not task.complete then
			--cc.ui.UILabel.new({text = "数量:",size = 22,color = cc.c3b(255,109,38),x = countTitleX,y = labelY})
				--:align(display.CENTER):addTo(layer)
			if VERSION_MONSTER then
				local spriteBg = display.newSprite("#taskLayerBar1.png",countTitleX+39,labelY):addTo(layer,-1)
			end
			local spriteNum = display.newSprite("#taskLayerNumber.png",countTitleX,labelY):addTo(layer)
			local l2 = cc.ui.UILabel.new({
				UILabelType = 2,
				font = fontType,
				text = ""..task.mytowerCount.."/"..task.towerCount,
				size = 22,
				color = fontcolor2,
				x = countNumberX,
				y = labelY,
			}):align(display.CENTER)
			layer:addChild(l2)
		end
	elseif task.type == 2 then
		local l1
		if VERSION_REDUCE then
			l1 = cc.ui.UILabel.new({
				UILabelType = 2,
				font = fontType,	
				text = self:getTaskText(task),
				size = 17,
				dimensions = cc.size(168, 150),
				color = fontcolor,
				x = mesX,
				y = msgY,
			}):align(display.LEFT_TOP)
		else
			l1 = cc.ui.UILabel.new({
				UILabelType = 2,
				font = fontType,	
				text = self:getTaskText(task),
				size = 22,
				dimensions = cc.size(168, 85),
				valign = cc.ui.TEXT_VALIGN_CENTER,
				color = fontcolor,
				x = mesX,
				y = msgY,
			})
		end
		layer:addChild(l1)
		if not task.complete then
			--cc.ui.UILabel.new({text = "数量:",size = 22,color = cc.c3b(255,109,38),x = countTitleX,y = labelY})
				--:align(display.CENTER):addTo(layer)
			if VERSION_MONSTER then
				local spriteBg = display.newSprite("#taskLayerBar1.png",countTitleX+39,labelY):addTo(layer,-1)
			end
			local spriteNum = display.newSprite("#taskLayerNumber.png",countTitleX,labelY):addTo(layer)	
			local l2 = cc.ui.UILabel.new({
				UILabelType = 2,
				font = fontType,
				text = task.myenemyCount.."/"..task.enemyCount,
				size = 22,
				color = fontcolor2,
				x = countNumberX,
				y = labelY,
			}):align(display.CENTER)
			layer:addChild(l2)
		end
	elseif task.type == 3 then
		local l1
		if VERSION_REDUCE then
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 17,
			dimensions = cc.size(168, 150),
			color = fontcolor,
			x = mesX,
			y = msgY,
		}):align(display.LEFT_TOP)
		else
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 22,
			color = fontcolor,
			dimensions = cc.size(168, 85),
			valign = cc.ui.TEXT_VALIGN_CENTER,
			x = mesX,
			y = msgY,
		})
		end
		-- local l1 = cc.ui.UILabel.new({
		-- 	UILabelType = 2,
		-- 	font = fontType,
		-- 	text = self:getTaskText(task),
		-- 	size = 22,
		-- 	color = fontcolor,
		-- 	x = mesX,
		-- 	y = msgY,
		-- }):align(display.CENTER)
		layer:addChild(l1)
		if not task.complete then
			--cc.ui.UILabel.new({text = "数量:",size = 22,color = cc.c3b(255,109,38),x = countTitleX,y = labelY})
				--:align(display.CENTER):addTo(layer)
			if VERSION_MONSTER then
				local spriteBg = display.newSprite("#taskLayerBar1.png",countTitleX+39,labelY):addTo(layer,-1)
			end
			local spriteNum = display.newSprite("#taskLayerNumber.png",countTitleX,labelY):addTo(layer)
			local l2 = cc.ui.UILabel.new({
				UILabelType = 2,
				font = fontType,
				text = task.mythingCount.."/"..task.thingCount,
				size = 22,
				color = fontcolor2,
				x = countNumberX,
				y = labelY,
			}):align(display.CENTER)
			layer:addChild(l2)
		end
	elseif task.type == 4 then
		local l1
		if VERSION_REDUCE then
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 17,
			dimensions = cc.size(168, 150),
			color = fontcolor,
			x = mesX,
			y = msgY,
		}):align(display.LEFT_TOP)
		else
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 22,
			color = fontcolor,
			dimensions = cc.size(168, 85),
			valign = cc.ui.TEXT_VALIGN_CENTER,
			x = mesX,
			y = msgY,
		})
		end
		-- local l1 = cc.ui.UILabel.new({
		-- 	UILabelType = 2,
		-- 	font = fontType,
		-- 	text = self:getTaskText(task),
		-- 	size = 22,
		-- 	color = fontcolor,
		-- 	x = mesX,
		-- 	y = msgY,
		-- }):align(display.CENTER)
		layer:addChild(l1)
		if not task.complete and not task.fail then
			--cc.ui.UILabel.new({text = "时间:",size = 22,color = cc.c3b(255,109,38),x = countTitleX,y = label1Y})
				--:align(display.CENTER):addTo(layer)
			if VERSION_MONSTER then
				local spriteBg = display.newSprite("#taskLayerBar2.png",countTitleX+43,labelY+5):addTo(layer,-1)
			end
			local spriteTime = display.newSprite("#taskLayerTime.png",countTitleX,label1Y):addTo(layer)
			local l2 = cc.ui.UILabel.new({
				UILabelType = 2,
				font = fontType,
				text = kkFunc.getSecondText(task.timeCount - task.mytimeCount),
				size = 22,
				color = fontcolor2,
				x = countNumberX,
				y = label1Y,
			}):align(display.CENTER)
			layer:addChild(l2)
			--cc.ui.UILabel.new({text = "金币:",size = 22,color = cc.c3b(255,109,38),x = countTitleX,y = label2Y})
				--:align(display.CENTER):addTo(layer)
			local spriteMoney = display.newSprite("#taskLayerMoney.png",countTitleX,label2Y):addTo(layer)
			local l3 = cc.ui.UILabel.new({
				UILabelType = 2,
				font = fontType,
				text = task.mymoney.."/"..task.money,
				size = 22,
				color = fontcolor2,
				x = countNumberX,
				y = label2Y,
			}):align(display.CENTER)
			layer:addChild(l3)
		end
	elseif task.type == 5 then
		local l1
		if VERSION_REDUCE then
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 17,
			dimensions = cc.size(168, 150),
			color = fontcolor,
			x = mesX,
			y = msgY,
		}):align(display.LEFT_TOP)
		else
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 22,
			color = fontcolor,
			dimensions = cc.size(168, 85),
			valign = cc.ui.TEXT_VALIGN_CENTER,
			x = mesX,
			y = msgY,
		})
		end
		-- local l1 = cc.ui.UILabel.new({
		-- 	UILabelType = 2,
		-- 	font = fontType,
		-- 	text = self:getTaskText(task),
		-- 	size = 22,
		-- 	color = fontcolor,
		-- 	x = mesX,
		-- 	y = msgY,
		-- }):align(display.CENTER)
		layer:addChild(l1)
		if not task.complete and not task.fail then
			--cc.ui.UILabel.new({text = "时间:",size = 22,color = cc.c3b(255,109,38),x = countTitleX,y = label1Y})
				--:align(display.CENTER):addTo(layer)
			if VERSION_MONSTER then
				local spriteBg = display.newSprite("#taskLayerBar2.png",countTitleX+43,labelY+5):addTo(layer,-1)
			end
			local spriteTime = display.newSprite("#taskLayerTime.png",countTitleX,label1Y):addTo(layer)
			local l2 = cc.ui.UILabel.new({
				UILabelType = 2,
				font = fontType,
				text = kkFunc.getSecondText(task.timeCount - task.mytimeCount),
				size = 22,
				color = fontcolor2,
				x = countNumberX,
				y = label1Y,
			}):align(display.CENTER)
			layer:addChild(l2)
			--cc.ui.UILabel.new({text = "数量:",size = 22,color = cc.c3b(255,109,38),x = countTitleX,y = label2Y})
				--:align(display.CENTER):addTo(layer)
			local spriteNum = display.newSprite("#taskLayerNumber.png",countTitleX,label2Y):addTo(layer)
			local l3 = cc.ui.UILabel.new({
				UILabelType = 2,
				font = fontType,
				text = ""..task.mythingCount.."/"..task.thingCount,
				size = 22,
				color = fontcolor2,
				x = countNumberX,
				y = label2Y,
			}):align(display.CENTER)
			layer:addChild(l3)
		end
	elseif task.type == 6 then
		local l1
		if VERSION_REDUCE then
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 17,
			dimensions = cc.size(168, 150),
			color = fontcolor,
			x = mesX,
			y = msgY,
		}):align(display.LEFT_TOP)
		else
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 22,
			color = fontcolor,
			dimensions = cc.size(168, 85),
			valign = cc.ui.TEXT_VALIGN_CENTER,
			x = mesX,
			y = msgY,
		})
		end
		-- local l1 = cc.ui.UILabel.new({
		-- 	UILabelType = 2,
		-- 	font = fontType,
		-- 	text = self:getTaskText(task),
		-- 	size = 22,
		-- 	color = fontcolor,
		-- 	x = mesX,
		-- 	y = msgY,
		-- }):align(display.CENTER)
		layer:addChild(l1)
		if not task.complete and not task.fail then
			--cc.ui.UILabel.new({text = "数量:",size = 22,color = cc.c3b(255,109,38),x = countTitleX,y = labelY})
				--:align(display.CENTER):addTo(layer)
			if VERSION_MONSTER then
				local spriteBg = display.newSprite("#taskLayerBar1.png",countTitleX+39,labelY):addTo(layer,-1)
			end
			local spriteNum = display.newSprite("#taskLayerNumber.png",countTitleX,labelY):addTo(layer)
			local l2 = cc.ui.UILabel.new({
				UILabelType = 2,
				font = fontType,
				text = task.mytowerCount.."/"..task.towerCount,
				size = 22,
				color = fontcolor2,
				x = countNumberX,
				y = labelY,
			}):align(display.CENTER)
			layer:addChild(l2)
		end
	elseif task.type == 7 then
		local l1
		if VERSION_REDUCE then
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 17,
			dimensions = cc.size(168, 150),
			color = fontcolor,
			x = mesX,
			y = msgY,
		}):align(display.LEFT_TOP)
		else
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 22,
			color = fontcolor,
			dimensions = cc.size(168, 85),
			valign = cc.ui.TEXT_VALIGN_CENTER,
			x = mesX,
			y = msgY,
		})
		end
		-- local l1 = cc.ui.UILabel.new({
		-- 	UILabelType = 2,
		-- 	font = fontType,
		-- 	text = self:getTaskText(task),
		-- 	size = 22,
		-- 	color = fontcolor,
		-- 	x = mesX,
		-- 	y = msgY,
		-- }):align(display.CENTER)
		layer:addChild(l1)
		if not task.complete then
			--cc.ui.UILabel.new({text = "数量:",size = 22,color = cc.c3b(255,109,38),x = countTitleX,y = labelY})
				--:align(display.CENTER):addTo(layer)
			if VERSION_MONSTER then
				local spriteBg = display.newSprite("#taskLayerBar1.png",countTitleX+39,labelY):addTo(layer,-1)
			end
			local spriteNum = display.newSprite("#taskLayerNumber.png",countTitleX,labelY):addTo(layer)
			local l2 = cc.ui.UILabel.new({
				UILabelType = 2,
				font = fontType,
				text = task.myenemyCount.."/"..task.enemyCount,
				size = 22,
				color = fontcolor2,
				x = countNumberX,
				y = labelY,
			}):align(display.CENTER)
			layer:addChild(l2)
		end
	elseif task.type == 8 then
		local l1
		if VERSION_REDUCE then
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 17,
			dimensions = cc.size(168, 150),
			color = fontcolor,
			x = mesX,
			y = msgY,
		}):align(display.LEFT_TOP)
		else
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 22,
			color = fontcolor,
			dimensions = cc.size(168, 85),
			valign = cc.ui.TEXT_VALIGN_CENTER,
			x = mesX,
			y = msgY,
		})
		end
		-- local l1 = cc.ui.UILabel.new({
		-- 	UILabelType = 2,
		-- 	font = fontType,
		-- 	text = self:getTaskText(task),
		-- 	size = 22,
		-- 	color = fontcolor,
		-- 	x = mesX,
		-- 	y = msgY,
		-- }):align(display.CENTER)
		layer:addChild(l1)
		if not task.complete and not task.fail then
			--cc.ui.UILabel.new({text = "数量:",size = 22,color = cc.c3b(255,109,38),x = countTitleX,y = labelY})
				--:align(display.CENTER):addTo(layer)
			if VERSION_MONSTER then
				local spriteBg = display.newSprite("#taskLayerBar1.png",countTitleX+39,labelY):addTo(layer,-1)
			end
			local spriteNum = display.newSprite("#taskLayerNumber.png",countTitleX,labelY):addTo(layer)
			local l2 = cc.ui.UILabel.new({
				UILabelType = 2,
				font = fontType,
				text = task.mytowerCount.."/"..task.towerCount,
				size = 22,
				color = fontcolor2,
				x = countNumberX,
				y = labelY,
			}):align(display.CENTER)
			layer:addChild(l2)
		end
	elseif task.type == 9 then
		local l1
		if VERSION_REDUCE then
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 17,
			dimensions = cc.size(168, 150),
			color = fontcolor,
			x = mesX,
			y = msgY,
		}):align(display.LEFT_TOP)
		else
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 22,
			color = fontcolor,
			dimensions = cc.size(168, 85),
			valign = cc.ui.TEXT_VALIGN_CENTER,
			x = mesX,
			y = msgY,
		})
		end
		-- local l1 = cc.ui.UILabel.new({
		-- 	UILabelType = 2,
		-- 	font = fontType,
		-- 	text = self:getTaskText(task),
		-- 	size = 22,
		-- 	color = fontcolor,
		-- 	x = mesX,
		-- 	y = msgY,
		-- }):align(display.CENTER)
		layer:addChild(l1)
		if not task.complete and not task.fail then
			--cc.ui.UILabel.new({text = "时间:",size = 22,color = cc.c3b(255,109,38),x = countTitleX,y = labelY})
				--:align(display.CENTER):addTo(layer)
			if VERSION_MONSTER then
				local spriteBg = display.newSprite("#taskLayerBar1.png",countTitleX+39,labelY):addTo(layer,-1)
			end
			local spriteTime = display.newSprite("#taskLayerTime.png",countTitleX,labelY):addTo(layer)
			local l2 = cc.ui.UILabel.new({
				UILabelType = 2,
				font = fontType,
				text = kkFunc.getSecondText(task.timeCount - task.mytimeCount),
				size = 22,
				color = fontcolor2,
				x = countNumberX,
				y = labelY,
			}):align(display.CENTER)
			layer:addChild(l2)
		end
	elseif task.type == 10 then
		local l1
		if VERSION_REDUCE then
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 17,
			dimensions = cc.size(168, 150),
			color = fontcolor,
			x = mesX,
			y = msgY,
		}):align(display.LEFT_TOP)
		else
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 22,
			color = fontcolor,
			dimensions = cc.size(168, 85),
			valign = cc.ui.TEXT_VALIGN_CENTER,
			x = mesX,
			y = msgY,
		})
		end
		-- local l1 = cc.ui.UILabel.new({
		-- 	UILabelType = 2,
		-- 	font = fontType,
		-- 	text = self:getTaskText(task),
		-- 	size = 22,
		-- 	color = fontcolor,
		-- 	x = mesX,
		-- 	y = msgY,
		-- }):align(display.CENTER)
		layer:addChild(l1)
		if not task.complete then
			--cc.ui.UILabel.new({text = "数量:",size = 22,color = cc.c3b(255,109,38),x = countTitleX,y = labelY})
				--:align(display.CENTER):addTo(layer)
			if VERSION_MONSTER then
				local spriteBg = display.newSprite("#taskLayerBar1.png",countTitleX+39,labelY):addTo(layer,-1)
			end
			local spriteNum = display.newSprite("#taskLayerNumber.png",countTitleX,labelY):addTo(layer)	
			local l2 = cc.ui.UILabel.new({
				UILabelType = 2,
				font = fontType,
				text = ""..task.mytowerCount.."/"..task.towerCount,
				size = 22,
				color = fontcolor2,
				x = countNumberX,
				y = labelY,
			}):align(display.CENTER)
			layer:addChild(l2)
		end
	elseif task.type == 11 then
		local l1
		if VERSION_REDUCE then
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 17,
			dimensions = cc.size(168, 150),
			color = fontcolor,
			x = mesX,
			y = msgY,
		}):align(display.LEFT_TOP)
		else
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 22,
			color = fontcolor,
			dimensions = cc.size(168, 85),
			valign = cc.ui.TEXT_VALIGN_CENTER,
			x = mesX,
			y = msgY,
		})
		end
		-- local l1 = cc.ui.UILabel.new({
		-- 	UILabelType = 2,
		-- 	font = fontType,
		-- 	text = self:getTaskText(task),
		-- 	align = ui.TEXT_ALIGN_CENTER,
		-- 	size = 22,
		-- 	color = fontcolor,
		-- 	x = mesX,
		-- 	y = msgY,
		-- }):align(display.CENTER)
		layer:addChild(l1)
		if not task.complete then
			--cc.ui.UILabel.new({text = "数量:",size = 22,color = cc.c3b(255,109,38),x = countTitleX,y = labelY})
				--:align(display.CENTER):addTo(layer)
			if VERSION_MONSTER then
				local spriteBg = display.newSprite("#taskLayerBar1.png",countTitleX+39,labelY):addTo(layer,-1)
			end
			local spriteNum = display.newSprite("#taskLayerNumber.png",countTitleX,labelY):addTo(layer)
			local l2 = cc.ui.UILabel.new({
				UILabelType = 2,
				font = fontType,
				text = task.myenemyCount.."/"..task.enemyCount,
				size = 22,
				color = fontcolor2,
				x = countNumberX,
				y = labelY,
			}):align(display.CENTER)
			layer:addChild(l2)
		end
	elseif task.type == 12 then
		local l1
		if VERSION_REDUCE then
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 17,
			dimensions = cc.size(168, 150),
			color = fontcolor,
			x = mesX,
			y = msgY,
		}):align(display.LEFT_TOP)
		else
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 22,
			color = fontcolor,
			dimensions = cc.size(168, 85),
			valign = cc.ui.TEXT_VALIGN_CENTER,
			x = mesX,
			y = msgY,
		})
		end
		-- local l1 = cc.ui.UILabel.new({
		-- 	UILabelType = 2,
		-- 	font = fontType,
		-- 	text = self:getTaskText(task),
		-- 	size = 22,
		-- 	color = fontcolor,
		-- 	x = mesX,
		-- 	y = msgY,
		-- }):align(display.CENTER)
		layer:addChild(l1)
		if not task.complete then
			--cc.ui.UILabel.new({text = "数量:",size = 22,color = cc.c3b(255,109,38),x = countTitleX,y = labelY})
				--:align(display.CENTER):addTo(layer)
			if VERSION_MONSTER then
				local spriteBg = display.newSprite("#taskLayerBar1.png",countTitleX+39,labelY):addTo(layer,-1)
			end
			local spriteNum = display.newSprite("#taskLayerNumber.png",countTitleX,labelY):addTo(layer)
			local l2 = cc.ui.UILabel.new({
				UILabelType = 2,
				font = fontType,
				text = ""..task.mythingCount.."/"..task.thingCount,
				size = 22,
				color = fontcolor2,
				x = countNumberX,
				y = labelY,
			}):align(display.CENTER)
			layer:addChild(l2)
		end
	elseif task.type == 13 then
		local l1
		if VERSION_REDUCE then
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 17,
			dimensions = cc.size(168, 150),
			color = fontcolor,
			x = mesX,
			y = msgY,
		}):align(display.LEFT_TOP)
		else
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 22,
			color = fontcolor,
			dimensions = cc.size(168, 85),
			valign = cc.ui.TEXT_VALIGN_CENTER,
			x = mesX,
			y = msgY,
		})
		end
		-- local l1 = cc.ui.UILabel.new({
		-- 	UILabelType = 2,
		-- 	font = fontType,
		-- 	text = self:getTaskText(task),
		-- 	size = 22,
		-- 	color = fontcolor,
		-- 	x = mesX,
		-- 	y = msgY,
		-- }):align(display.CENTER)
		layer:addChild(l1)
		if not task.complete and not task.fail then
			--cc.ui.UILabel.new({text = "时间:",size = 22,color = cc.c3b(255,109,38),x = countTitleX,y = label1Y})
				--:align(display.CENTER):addTo(layer)
			if VERSION_MONSTER then
				local spriteBg = display.newSprite("#taskLayerBar2.png",countTitleX+43,labelY+5):addTo(layer,-1)
			end
			local spriteTime = display.newSprite("#taskLayerTime.png",countTitleX,label1Y):addTo(layer)	
			local l2 = cc.ui.UILabel.new({
				UILabelType = 2,
				font = fontType,
				text = kkFunc.getSecondText(task.timeCount - task.mytimeCount),
				size = 22,
				color = fontcolor2,
				x = countNumberX,
				y = label1Y,
			}):align(display.CENTER)
			layer:addChild(l2)
			--cc.ui.UILabel.new({text = "数量:",size = 22,color = cc.c3b(255,109,38),x = countTitleX,y = label2Y})
				--:align(display.CENTER):addTo(layer)
			local spriteNum = display.newSprite("#taskLayerNumber.png",countTitleX,label2Y):addTo(layer)
			local l3 = cc.ui.UILabel.new({
				UILabelType = 2,
				font = fontType,
				text = task.mythingCount.."/"..task.thingCount,
				size = 22,
				color = fontcolor2,
				x = countNumberX,
				y = label2Y,
			}):align(display.CENTER)
			layer:addChild(l3)
		end
	elseif task.type == 14 then
		local l1
		if VERSION_REDUCE then
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 17,
			dimensions = cc.size(168, 150),
			color = fontcolor,
			x = mesX,
			y = msgY,
		}):align(display.LEFT_TOP)
		else
			l1 = cc.ui.UILabel.new({
			UILabelType = 2,
			font = fontType,	
			text = self:getTaskText(task),
			size = 22,
			color = fontcolor,
			dimensions = cc.size(168, 85),
			valign = cc.ui.TEXT_VALIGN_CENTER,
			x = mesX,
			y = msgY,
		})
		end
		-- local l1 = cc.ui.UILabel.new({
		-- 	UILabelType = 2,
		-- 	font = fontType,
		-- 	text = self:getTaskText(task),
		-- 	size = 22,
		-- 	color = fontcolor,
		-- 	x = mesX,
		-- 	y = msgY,
		-- }):align(display.CENTER)
		layer:addChild(l1)
		if not task.complete then
			--cc.ui.UILabel.new({text = "数量:",size = 22,color = cc.c3b(255,109,38),x = countTitleX,y = labelY})
				--:align(display.CENTER):addTo(layer)
			if VERSION_MONSTER then
				local spriteBg = display.newSprite("#taskLayerBar1.png",countTitleX+39,labelY):addTo(layer,-1)
			end
			local spriteNum = display.newSprite("#taskLayerNumber.png",countTitleX,labelY):addTo(layer)	
			local l2 = cc.ui.UILabel.new({
				UILabelType = 2,
				font = fontType,
				text = task.myenemyCount.."/"..task.enemyCount,
				size = 22,
				color = fontcolor2,
				x = countNumberX,
				y = labelY,
			}):align(display.CENTER)
			layer:addChild(l2)
		end
	end
	local picY
	if VERSION_MONSTER then
		picY = -110
	else
		picY = -120
	end
	if task.complete then
		local complete = display.newSprite("#taskComplete.png", midX, picY):addTo(layer)
		if VERSION_REDUCE then
			complete:setScale(0.8)
		end
	elseif task.fail then
		local complete = display.newSprite("#taskFail.png", midX, picY):addTo(layer)
		complete:setScale(0.8)
	end
	return layer
end

--单个任务检查是否完成
function Task:checkSingleTask(task)
	if task.complete or task.fail then return end
	if task.type == 1 then
		if task.mytowerCount >= task.towerCount then
			task.complete = true		
		end
	elseif task.type == 2 then
		if task.myenemyCount >= task.enemyCount then
			task.complete = true	
		end	
	elseif task.type == 3 then
		if task.mythingCount >= task.thingCount then
			task.complete = true		
		end		
	elseif task.type == 4 then
		if task.mytimeCount > task.timeCount then
			task.fail = true
		elseif task.mymoney >= task.money then
			task.complete = true
		end
	elseif task.type == 5 then
		if task.mytimeCount > task.timeCount then
			task.fail = true
		elseif task.mythingCount >= task.thingCount then
			task.complete = true
		end
	elseif task.type == 6 then
		if task.mytowerCount > task.towerCount then
			task.fail = true
		end
	elseif task.type == 8 then
		if task.mytowerCount > task.towerCount then
			task.fail = true
		end
	elseif task.type == 9 then
		if task.mytimeCount > task.timeCount then
			task.fail = true
		elseif task.mythingCount >= task.thingCount then
			task.complete = true
		end
	elseif task.type == 10 then
		if task.mytowerCount >= task.towerCount then
			task.complete = true	
		end
	elseif task.type == 11 then
		if task.myenemyCount >= task.enemyCount then
			task.complete = true	
		end
	elseif task.type == 12 then
		if task.mythingCount >= task.thingCount then
			task.complete = true	
		end
	elseif task.type == 13 then
		if task.mytimeCount > task.timeCount then
			task.fail = true
		elseif task.mythingCount >= task.thingCount then
			task.complete = true
		end
	elseif task.type == 14 then
		if task.myenemyCount >= task.enemyCount then
			task.complete = true		
		end
	elseif task.type == 15 then
		if task.mythingCount >= task.thingCount then
			task.complete = true	
		end			
	else				
	end
	if task.complete then
		print(task.idx.."号任务完成！")
	elseif task.fail then
		print(task.idx.."号任务失败！")
	end
	if task.complete or task.fail then
		self.playScene.skillLayer:setTask(task.idx, task.complete, task.fail)
	end
end

function Task:checkTaskAfterWin()
	for i = 1, #self.tasks do
		local task = self.tasks[i]
		if task.type == 6 and not task.fail then
			if task.mytowerCount <= task.towerCount then
				task.complete = true
				print(task.idx.."号任务完成！")
			end
		elseif task.type == 8 and not task.fail then
			if task.mytowerCount <= task.towerCount then
				task.complete = true
				print(task.idx.."号任务完成！")
			end
		end
	end
end

--时间记录
function Task:timePassed()
    self.lastCheckTime = self.lastCheckTime + 1   
	for i = 1, #self.tasks do
		if self.tasks[i].type == 4 or self.tasks[i].type == 5 or self.tasks[i].type == 9 or self.tasks[i].type == 13 then
			self.tasks[i].mytimeCount = self.lastCheckTime
			self:checkSingleTask(self.tasks[i])
		end
	end
end

--获得钱时记录
function Task:addMoney(add)
	if not self.flag.checkMoney then return end
	for i = 1, #self.tasks do
		if self.tasks[i].type == 4 then
			self.tasks[i].mymoney = self.tasks[i].mymoney + add
			self:checkSingleTask(self.tasks[i])
		end
	end
end

--预设塔和道具打开的塔
function Task:getPreFort(towerIdx)
	if not self.flag.checkCreateFort then return end
	local function doAddRecord(task)
		task.mytowerCount = task.mytowerCount + 1
		self:checkSingleTask(task)
	end
	for i = 1, #self.tasks do
		if self.tasks[i].type == 6 then
			if self.tasks[i].towerIdx == towerIdx then
				doAddRecord(self.tasks[i])
			end
		end
		if self.tasks[i].type == 8 then
			doAddRecord(self.tasks[i])
		end
	end
end

--建造炮塔时记录
function Task:createFort(towerIdx)
	if not self.flag.checkCreateFort then return end
	local function doAddRecord(task)
		task.mytowerCount = task.mytowerCount + 1
		self:checkSingleTask(task)
	end
	for i = 1, #self.tasks do
		if self.tasks[i].type == 1 then
			if self.tasks[i].towerIdx == towerIdx then
				doAddRecord(self.tasks[i])
			end
		end
		if self.tasks[i].type == 6 then
			if self.tasks[i].towerIdx == towerIdx then
				doAddRecord(self.tasks[i])
			end
		end
		if self.tasks[i].type == 8 then
			doAddRecord(self.tasks[i])
		end
	end
end

function Task:removeFort(towerIdx)
	local function doAddRecord(task)
		task.mytowerCount = task.mytowerCount - 1
		self:checkSingleTask(task)
	end
	for i = 1, #self.tasks do
		if self.tasks[i].type == 6 then
			if self.tasks[i].towerIdx == towerIdx then
				doAddRecord(self.tasks[i])
			end
		end
		if self.tasks[i].type == 8 then
			doAddRecord(self.tasks[i])
		end
		--[[if self.tasks[i].type == 10 then
			if self.tasks[i].towerIdx == towerIdx then
				doAddRecord(self.tasks[i])
			end
		end]]
	end
end

function Task:createSuperFort(towerIdx)
	if not self.flag.checkTowerUpgrade then return end
	local function doAddRecord(task)
		task.mytowerCount = task.mytowerCount + 1
		self:checkSingleTask(task)
	end
	for i = 1, #self.tasks do
		if self.tasks[i].type == 10 then
			if self.tasks[i].towerIdx == towerIdx then
				doAddRecord(self.tasks[i])
			end
		end
	end
end

function Task:removeThing(thingIdx,towerIdx)
	if not self.flag.checkThing then return end
	local function doAddRecord(task)
		task.mythingCount = task.mythingCount + 1
		self:checkSingleTask(task)
	end
	for i = 1, #self.tasks do
		local task = self.tasks[i]
		if task.type == 3 or task.type == 5 then
			if task.thingIdx == thingIdx then
				doAddRecord(task)
			end
		elseif task.type == 12 or task.type == 13 then
			if task.thingIdx == thingIdx and task.towerIdx == towerIdx then
				doAddRecord(task)
			end
		elseif task.type == 9 or  task.type == 15 then
			doAddRecord(task)
		end
	end
end

function Task:killEnemy(towerIdx,enemyIdx,isHero)
	if not self.flag.checkEnemy then return end
	local function doAddRecord(task)
		task.myenemyCount = task.myenemyCount + 1
		self:checkSingleTask(task)
	end
	for i = 1, #self.tasks do
		local task = self.tasks[i]
		if task.type == 2 then
			if task.towerIdx == towerIdx then
				doAddRecord(task)
			end
		elseif task.type == 11 then
			if task.towerIdx == towerIdx and task.enemyIdx == enemyIdx then
				doAddRecord(task)
			end
		elseif task.type == 14 then
			if isHero == true then
				doAddRecord(task)
			end
		end
	end
end

--保存用户记录。
function Task:saveTaskRecord()
	local change = false
	for i = 1, #self.tasks do
		if self.tasks[i].complete then
			GameData.levelTask[""..self.level][""..i] = true
			change = true
		end
	end
	if change then
		GameState.save(GameData)
	end
end

function Task:saveDataEyeTask()
	local level = LevelsData.getRealLevel(self.level)
    if level < 10 then
        level = "0"..level
    end
    	--厦门
	-- for i = 1, #self.tasks do
	-- 	if self.tasks[i].complete and GameData.levelTask[""..self.level][""..i] == false then
	-- 		--DataEye
	-- 		if device.platform == "android" or device.platform == "ios" then	
	-- 			DCTask.begin(level.."0"..i, DC_BranchLine)
	-- 			DCTask.complete(level.."0"..i)
	-- 	    end
	-- 	elseif self.tasks[i].fail and GameData.levelTask[""..self.level][""..i] == false then
	-- 		if device.platform == "android" or device.platform == "ios" then
	-- 			DCTask.begin(level.."0"..i, DC_BranchLine)
	-- 			local reason
	-- 	        local task = self.tasks[i]
	-- 			if task.type == 4 or task.type == 5 or task.type == 9 or task.type == 13 then
	-- 				reason = "任务超时"
	-- 			elseif task.type == 6 or task.type == 8 then
	-- 				reason = "建造塔超标"
	-- 			end
	-- 			DCTask.fail(level.."0"..i,reason)
	-- 		end
	-- 	end
	-- end
	    --厦门

end

return Task