
local MapInfo = import("..views.MapInfo")

local Bullet = class("Bullet", function(params)
    return display.newSprite("#"..BulletData[params.idx].level[params.level].pic)
end)
function Bullet:ctor(params)
	self.towerIdx = params.towerIdx
	self:setPosition(params.fortPos)
	self.target = params.target
	self.thingTarget = params.thingTarget
	self.fortTarget = params.fortTarget
	self.cityTarget = params.cityTarget--攻城模式
	self.passTime = 0
	self.needUpdate = true

	if self.target ~= nil then
		self.targetType = 1
	elseif self.thingTarget ~= nil then
		self.targetType = 2
	elseif self.fortTarget ~= nil then
		self.targetType = 3
	elseif self.cityTarget ~= nil then
		self.targetType = 4
	end

	self.idx = params.idx
	self.level = params.level
	self.data = BulletData[params.idx]
	self.levelData = self.data.level[params.level]
	self.isEnemyBullet = params.idx > 100
	if not self.isEnemyBullet then--兔子子弹从excel数据中读
		local speed = nil
		if params.level < 4 then
			self.excelLevelData = ExcelTowersData[params.idx].level[params.level]
			self.attack = self.excelLevelData.attack
			speed = self.excelLevelData.bulletSpeed
		else --4级大技能子弹取3级数值
			self.attack = ExcelTowersData[params.idx].level[3].attack
			speed = ExcelTowersData[params.idx].level[3].bulletSpeed
		end
		if speed ~= nil then --有些不是发射子弹，如吉它光圈弹。
			self.speed = speed * MapInfo.tileSize
			self:initSpeed()
		end
	else--怪子弹攻击从怪定义里取。
		self.attack = ExcelEnemyData[params.fromEnemy].attack
		--self.attack = self.levelData.attack
		if self.levelData.speed ~= nil then --有些不是发射子弹，如吉它光圈弹。
			self.speed = self.levelData.speed * MapInfo.tileSize
			self:initSpeed()
		end
	end
	self.pic = self.levelData.pic
	self.bombAniName = self.levelData.actBomb
end

function Bullet:initSpeed()
	local px, py = self:getPosition()
	local dest
	if self.targetType == 2 then
		--self:setRotation(self:getRotation()-90)
		self.thingPos = MapInfo:getCenterOfThing(self.thingTarget)
		dest = cc.p(self.thingPos.x - px,
				 	self.thingPos.y - py)
		local z = math.sqrt(dest.x*dest.x +dest.y*dest.y)
		self.xSpeed = self.speed * dest.x / z
		self.ySpeed = self.speed * dest.y / z
		self.time = z / self.speed
	elseif self.targetType == 1 then
		self:setRotation(self:getRotation()-90)
		self.targetPos = self.target:getPositionCenter()
		dest = cc.p(self.targetPos.x - px,
				 	self.targetPos.y - py)
		local z = math.sqrt(dest.x*dest.x +dest.y*dest.y)
		self.xSpeed = self.speed * dest.x / z
		self.ySpeed = self.speed * dest.y / z
		self.time = z / self.speed
	elseif self.targetType == 4 then
		self:setRotation(self:getRotation()-90)
		self.cityTargetPos = self.cityTarget:getPositionCenter()
		dest = cc.p(self.cityTargetPos.x - px,
				 	self.cityTargetPos.y - py)
		local z = math.sqrt(dest.x*dest.x +dest.y*dest.y)
		self.xSpeed = self.speed * dest.x / z
		self.ySpeed = self.speed * dest.y / z
		self.time = z / self.speed
	elseif self.targetType == 3 then
		--self:setRotation(self:getRotation()-90)
		self.fortPos = self.fortTarget:getPositionCenter()
		dest = cc.p(self.fortPos.x - px,
				 	self.fortPos.y - py)
		local z = math.sqrt(dest.x*dest.x +dest.y*dest.y)
		self.xSpeed = self.speed * dest.x / z
		self.ySpeed = self.speed * dest.y / z
		self.time = z / self.speed
	end
	if self.data.rotate then
		local rt = kkFunc.getLineRotation(cc.p(0,0), dest)
		self:setRotation(rt)
	end
	--出塔一小段距离开始放子弹
	local startDist = self.data.startDist or 20
	local t = startDist/self.speed
	self:setPosition(px + t*self.xSpeed, py + t*self.ySpeed)
	self.time = self.time - t
end

--跟踪弹要变向
function Bullet:changeSpeed(enPt)
	local px, py = self:getPosition()
	local dest = cc.p(enPt.x - px,
			 	enPt.y - py)
	local z = math.sqrt(dest.x*dest.x +dest.y*dest.y)
	self.xSpeed = self.speed * dest.x / z
	self.ySpeed = self.speed * dest.y / z
	if self.data.rotate then
		local rt = kkFunc.getLineRotation(cc.p(px,py), enPt)
		self:setRotation(rt)
	end
end

function Bullet:checkOutOfScreen(pos)
	local r = cc.rect(0,0,CONFIG_SCREEN_WIDTH,CONFIG_SCREEN_HEIGHT)
	return cc.rectContainsPoint(r, pos) == false
end

function Bullet.getObjRect(p, s)
	return cc.rect(p.x - s.width*0.5, p.y - s.height*0.5,  s.width, s.height)
end

function Bullet:checkCrash(bulletPos, targetPos)
	local r1 = Bullet.getObjRect(targetPos, cc.size(20,20))
	local r2 = Bullet.getObjRect(bulletPos, cc.size(20,20))--self:getContentSize())
	if cc.rectIntersectsRect(r1, r2) then
		return true
	else
		return false
	end
end

function Bullet:updateForFort(dt)
	if self.passTime > self.time then
		if self.fortTarget ~= nil then
			self.fortTarget:updateHp(-self.attack)
		end
		self:bomb()
	end
end

function Bullet:updateForThing(dt)
	if self.passTime > self.time then
		if self.thingTarget then
			g_fightLayer:hitThing(self.thingTarget, self.attack,self.towerIdx)
		end
		self:bomb()
	end
end

function Bullet:updateForTarget(dt)
	if self.passTime > self.time then
		if self.target ~= nil then
			self.target:updateHp(-self.attack, self.idx)
		end
		self:bomb()
	end
end

function Bullet:updateForCityTarget(dt)
	if self.passTime > self.time then
		if self.cityTarget ~= nil then
			self.cityTarget:updateHp(-self.attack, self.idx)
		end
		self:bomb()
	end
end

function Bullet:bomb()
	--没有爆炸动画
	if self.bombAniName == nil then
		g_fightLayer:removeBullet(self)
		return
	end
	local pos
	if self.targetType == 1 then
		pos = self.targetPos
	elseif self.targetType == 2 then 
		pos = self.thingPos
	elseif self.targetType == 3 then 
		pos = self.fortPos
	elseif self.targetType == 4 then
		pos = self.cityTargetPos
	else return end
	
	local bomb = display.newSprite("#"..self.pic,pos.x,pos.y)
	g_fightLayer:addChild(bomb, BaseZOrder.bullet)
	local bombAni = display.getAnimationCache(self.bombAniName)
	bomb:playAnimationOnce(bombAni, true)
	g_fightLayer:removeBullet(self)
end

function Bullet:updatePos(dt)
	local x,y = self:getPosition()
	local pos = cc.p(x + dt*self.xSpeed, y + dt*self.ySpeed)
	self:setPosition(pos)
	if self:checkOutOfScreen(pos) then
		g_fightLayer:removeBullet(self)
		return false
	end
	return true
end

function Bullet:update(dt)
	self.passTime = self.passTime + dt
	if not self:updatePos(dt) then return end
	if self.targetType == 2 then --self.thingTarget ~= nil then
		self:updateForThing(dt)
	elseif self.targetType == 1 then --self.target ~= nil then
		self:updateForTarget(dt)
	elseif self.targetType == 3 then
		self:updateForFort(dt)
	elseif self.targetType == 4 then
		self:updateForCityTarget(dt)		
	end
end

function Bullet:PointAtLineLeftRight(ptStart,ptEnd,ptTest)
	self.xVarStart = ptStart.x - ptTest.x
	self.yVarStart = ptStart.y - ptTest.y
	self.xVarEnd = ptEnd.x - ptTest.x
	self.yVarEnd = ptEnd.y - ptTest.y

	self.nRet = self.xVarStart * self.yVarEnd - self.yVarStart * self.xVarEnd
	
	if self.nRet == 0 then
		return 0
	elseif self.nRet > 0 then
		return 1
	elseif self.nRet < 0 then
		return -1
	else
		return 0
	end

end

function Bullet:IsTwoLineIntersect(ptLine1Start,ptLine1End,ptLine2Start,ptLine2End)
	local nLine1Start = self:PointAtLineLeftRight(ptLine2Start, ptLine2End, ptLine1Start)
	local nLine1End = self:PointAtLineLeftRight(ptLine2Start, ptLine2End, ptLine1End)
	if nLine1Start * nLine1End > 0 then
		return false
	end

	local nLine2Start = self:PointAtLineLeftRight(ptLine1Start, ptLine1End, ptLine2Start)
	local nLine2End = self:PointAtLineLeftRight(ptLine1Start, ptLine1End, ptLine2End)

	if nLine2Start * nLine2End > 0 then
		return false
	else
		return true
	end
end

function Bullet:IsLineIntersectRect(ptStart,ptEnd,rect)
	if cc.rectContainsPoint(rect,ptStart) or cc.rectContainsPoint(rect,ptEnd) then
		return true
	elseif self:IsTwoLineIntersect(ptStart, ptEnd, cc.p(rect.x, rect.y), cc.p(rect.x+rect.width, rect.y+rect.height)) then
		return true
	elseif self:IsTwoLineIntersect(ptStart, ptEnd, cc.p(rect.x, rect.y+rect.height), cc.p(rect.x+rect.width, rect.y)) then
		return true
	end

	return false
end

function Bullet:GetMapThingsInPath()--得到子弹路径上的物体

	self.thingsInPath = {}

	local xLimit = self:getPositionX()+self.xSpeed*1000
	local yLimit = self:getPositionY()+self.ySpeed*1000

	for i=1,#MapInfo.things do
		local thing = MapInfo.things[i]
		if thing ~= nil and thing.part == false then
			local w = MapInfo.things[i].width*MapInfo.tileSize
			local h = MapInfo.things[i].height*MapInfo.tileSize
			local r1 = Bullet.getObjRect(MapInfo.things[i].pos, cc.size(w, h))
			if self:IsLineIntersectRect(cc.p(xLimit,yLimit),cc.p(self:getPosition()),r1) then
				self.thingsInPath[#self.thingsInPath+1] = MapInfo.things[i]
			end
		end
	end
end

return Bullet