
local RANDOM_ENEMY_DIST = false

local MapInfo = import("..views.MapInfo")
local HpBar = import("..sprites.HpBar")

local Enemy = class("Enemy", function(params)
    return display.newNode()
end)

EnemyBuffer = {
	BUFFER_OPPOSITE = 1,
	BUFFER_STOP = 2,
	BUFFER_DIZZY = 3,
	BUFFER_FIRE = 4,
	BUFFER_SLOW = 5,
	BUFFER_BUBBLE = 6,
	BUFFER_WEAK = 7,
}

function Enemy:ctor(params)
	self.enemyPic = display.newSprite("#"..EnemyData[params.enemyIdx].pic):addTo(self)
	self.enemySize = self.enemyPic:getContentSize()
	if DEBUG_INFO and not USE_BATCHNODE then
		local idLabel = cc.ui.UILabel.new({
			text = "E"..params.enemyIdx,
			size = 15,
			color = display.COLOR_BLACK,
			x = 0,
			y = 0,
		})
		self:addChild(idLabel)
	end
	--预设血量
	self.setHp = params.enHp

	self.wayIdx = params.wayIdx
	self.wayPoint = 1
	self.moveDist = 0
	self.hpTime = 0

	self.buffers = {
		dizzy = false, --晕
		dizzyPic = nil,
		dizzyTime = 0,
		slow = false, --减速
		slowPic = nil,
		slowTime = 0,
		slowSpeed = 0,
		stop = false, --拦路
		stopTime = 0,
		opposite = false, --兔女郎
		oppositeTime = 0,
		oppositePic = nil,
		fire = false, --辣椒怪加速
		firePic = nil,
		fireTime = 0,
		fireSpeed = 0,
		bubble = false, --泡泡兔怒气
		bubblePic = nil,
		bubbleTime = 0,
		bubbleSpeed = 0,
		weak = false, --吉它兔怒气
		weakPic = nil,
		weakTime = 0,
		weakSpeed = 0,
		hero = nil, --被英雄打
		skill = false, --正在放技能
	}
	self.pause = false --怪物暂停移动
	self.needUpdate = true
	self.canHit = true --特殊怪在某些情况下不能被打。
	self.bossSkillReady = true --Boss怪在一开始就准备好技能。被攻击时释放。

	if params.pos ~= nil then
		self:setPosition(params.pos)
	end

	--减少update消耗
	self.newPos = cc.p(0,0)
	self.interval = 0
	--按enemyIdx初始化，变身需要，所以单独提出来。
	self:initWithEnemyIdx(params.enemyIdx)
	--id和赚的钱不变。
	self.idx = params.enemyIdx
	self.earn = self.data.earn
	self.pic = self.localData.pic --头一帧图片。

	--朝向守护点
	self.orient = "left"
	if params.pos.x < MapInfo.endPos.x then 
		self.orient = "right"
		self:setFlip(true)
	end
end
function Enemy:attackCityEnemyData()
	--重置怪物的攻击力，攻速等
    --[[for i = 1 ,#g_playScene.waves do
        local smallwave = g_playScene.waves[i]
        for j = 1,#smallwave do
            local enemysIdx = smallwave[j].enemy
            addEnemy
        end
    end]]
    --怪物数据的增长方式按照循环波数出现的次数以百分比增加，循环次数越多，怪物越强
    local speedUpRate = 0.1 --速度按照循环波数的增加而减小，暂定每循环一次减少0.1速度
    local attackUpRate = 5 --攻击按照循环波数的增加而增加，暂定每循环一次增加5攻击
    local hpUpRate = 20 --血量按照循环波数的增加而增加，暂定每循环一次增加20血量
	self.speed = (self.data.speed + g_playScene.loopNum*speedUpRate)* MapInfo.tileSize
	self.attack = self.data.attack + g_playScene.loopNum*attackUpRate
	self.hpMax = self.setHp or self.data.hp
	self.hpMax = self.hpMax + g_playScene.loopNum*hpUpRate
end
function Enemy:initWithEnemyIdx(enemyIdx)
	self.localData = EnemyData[enemyIdx]
	--受击动画取消，这一帧也不用了。
	--self.hurtPic = self.localData.hurtFrame

	self.data = ExcelEnemyData[enemyIdx]
	--[[if g_playScene.mode == "attackCityMode" then
		self:attackCityEnemyData()
	else
		self.speed = self.data.speed * MapInfo.tileSize
		self.attack = self.data.attack --攻击英雄时用
		self.hpMax = self.setHp or self.data.hp --有预设，则变身前后血量一样。
	end]]
	self.speed = self.data.speed * MapInfo.tileSize
	self.attack = self.data.attack --攻击英雄时用
	self.hpMax = self.setHp or self.data.hp --有预设，则变身前后血量一样。
	if not VERSION_REDUCE then
		if g_playScene.mode == "attackCityMode" then
	        if GameData.failNumber["level"..g_playScene.level] >= 2 then
	        	self.attack = self.data.attack*0.7
	        end
	    end
	end
	self.hp = self.hpMax
	--如果有攻击会有cd，第一次攻击时间通常比cd短，所以首次cd会先加上一段时间。
	self.cd = 0
	self.skillCD = 0 --技能的计时
	if self.data.attackCD ~= nil and self.data.attackFirstTime ~= nil then
		self.cd = self.data.attackCD - self.data.attackFirstTime
	end
	--攻击范围。
	if self.data.range ~= nil then
    	self.rangeSq = math.pow(self.data.range*MapInfo.tileSize,2)
    end

    --怪的锚点。
	if self.localData.anchor ~= nil then
		local x, y = unpack(self.localData.anchor)
		self.anchorPt = cc.p(x,y)
		self.enemyPic:setAnchorPoint(self.anchorPt)
		self.footHigh = y*self.enemySize.height
	else
		self.footHigh = 10
		self.anchorPt = cc.p(0.5, self.footHigh/self.enemySize.height)
    	self.enemyPic:setAnchorPoint(self.anchorPt)
	end

	--打完变形
	self.changeTo = self.localData.changeTo
	--血条
	if self.hpBar == nil then
		if g_playScene.mode == "bossMode" and self.idx == 64 then
		else
			self.hpBar = HpBar.new(HPBAR_TYPE_ENEMY, cc.size(0, self.enemySize.height-10), self.hpMax)
			self:addChild(self.hpBar)
			self.hpBar:setVisible(false)
		end	
	else
		self.hpBar:updateHp(self.hpMax)
	end
	self.enemyPic:stopAllActions()
	self.moveAct = display.getAnimationCache(self.localData.actMove)
	self.enemyPic:playAnimationForever(self.moveAct)

	--攻击动画。有些没有
	if self.localData.attackAct ~= nil then
		self.attackAct = display.getAnimationCache(self.localData.attackAct)
	end
	if self.localData.magicAct ~= nil then
		self.magicAct = display.getAnimationCache(self.localData.magicAct)
	end
end

function Enemy:setFlip(b)
	self.enemyPic:setFlippedX(b)
	if self.anchorPt.x ~= 0.5 then
		if b then
			self.enemyPic:setAnchorPoint(1-self.anchorPt.x, self.anchorPt.y)
		else
			self.enemyPic:setAnchorPoint(self.anchorPt.x, self.anchorPt.y)
		end
	end
end

function Enemy:getRect()
	local enPt = cc.p(self:getPosition())
	return cc.rect(enPt.x - self.enemySize.width*0.5,enPt.y - self.footHigh,self.enemySize.width,self.enemySize.height)
end

function Enemy:getPositionCenterDy()
	local dy = self.enemySize.height*0.5 - self.footHigh
	if dy > 70 then
		dy = 70
	end
	return dy
end

function Enemy:getPositionCenter()
	local enPt = cc.p(self:getPosition())
	enPt.y = enPt.y + self:getPositionCenterDy()
	return enPt
end

function Enemy:getPositionCenterToBomb()
	local enPt = cc.p(self:getPosition())
	enPt.y = enPt.y - self.anchorPt.y*self.enemySize.height + 10
	return enPt
end
function Enemy:lockByHero(hero)
	--正在打的小怪前置显示
	self:setLocalZOrder(BaseZOrder.enemy+1)
	self.buffers.hero = hero
	local heroX = hero:getPositionX()
	local selfX = self:getPositionX()
	self.buffers.heroFlippedX = self.enemyPic:isFlippedX()
	if self.enemyPic:isFlippedX() and heroX < selfX then
		self:setFlip(false)
	elseif not self.enemyPic:isFlippedX() and heroX > selfX then
		self:setFlip(true)
	end
end

function Enemy:unlockByHero()
	if self.buffers.hero == nil then return end
	self:setLocalZOrder(BaseZOrder.enemy)
	self.buffers.hero = nil
	self:setFlip(self.buffers.heroFlippedX)
end

function Enemy:pokeBubble()
	--数值定的1格半径内爆炸伤害160
	--print("戳气泡")
	self.buffers.bubble = false
	self.buffers.bubblePic:removeFromParent()

	local enemies = g_fightLayer:getEnimiesAroundPos(cc.p(self:getPosition()), 
		math.pow(MapInfo.tileSize*1,2))
	for i = 1, #enemies do
		enemies[i]:updateHp(-160)
	end
end

function Enemy:addBuffer(params)
	if self.idx == 44 and self.filled then
		--填坑的黄瓜只受伤害不加状态
		if params.attack ~= nil then
			self:updateHp(-params.attack)
		end
		return
	end
	if params.bufferType == EnemyBuffer.BUFFER_OPPOSITE then
		self:addOppositeBuffer(params)
	elseif params.bufferType == EnemyBuffer.BUFFER_STOP then
		self:addStopBuffer(params)
	elseif params.bufferType == EnemyBuffer.BUFFER_DIZZY then
		self:addDizzyBuffer(params)
	elseif params.bufferType == EnemyBuffer.BUFFER_FIRE then
		self:addFireBuffer(params)
	elseif params.bufferType == EnemyBuffer.BUFFER_SLOW then
		self:addSlowBuffer(params)
	elseif params.bufferType == EnemyBuffer.BUFFER_BUBBLE then
		self:addBubbleBuffer(params)
	elseif params.bufferType == EnemyBuffer.BUFFER_WEAK then
		self:addWeakBuffer(params)
	end
end

function Enemy:addOppositeBuffer(params)
	if self.tornado then return end
	if self.buffers.opposite then
		self.buffers.oppositeTime = params.time
		return
	end
	self.buffers.opposite = true
	self.buffers.oppositeTime = params.time

	self.buffers.oppositePic = display.newSprite("#youhuo.png")
	self.buffers.oppositePic:setPosition(0,self.enemySize.height)
	self:addChild(self.buffers.oppositePic)
	local action = cc.RepeatForever:create(transition.sequence({
		cc.MoveBy:create(0.5, cc.p(0,15)),
		cc.MoveBy:create(0.5, cc.p(0,-15)),
	}))
	self.buffers.oppositePic:runAction(action)
end

function Enemy:addStopBuffer(params)
	if self.tornado then return end
	if self.buffers.stop then
		self.buffers.stopTime = params.time
		return
	end
	self.buffers.stop = true
	self.buffers.stopTime = params.time
end

function Enemy:addDizzyBuffer(params)
	if self.tornado then 
		self:updateHp(-params.attack)
		return 
	end
	local success = math.random(1,100) <= params.chance*100
	if not success then
		self:updateHp(-params.attack)
		return
	end
	if self.buffers.dizzy then
		self.buffers.dizzyTime = params.time
		self:updateHp(-params.attack)
		return
	end
	self.buffers.dizzyPic = params.pic
	self.buffers.dizzyPic:setPosition(0,self.enemySize.height-30)
	self:addChild(self.buffers.dizzyPic)
	self.buffers.dizzy = true
	self.buffers.dizzyTime = params.time
	self:updateHp(-params.attack)
end

function Enemy:addFireBuffer(params)
	if self.tornado then 
		return 
	end
	--冰火泡不能共存
	self:clearSlow()
	self:clearBubble()

	self.buffers.firePic = display.newSprite("#"..EnemyData.common.firePic,0,0)
	self:addChild(self.buffers.firePic,-1)
	local ani = display.getAnimationCache(EnemyData.common.fireAni)
	self.buffers.firePic:playAnimationForever(ani)
	self.buffers.fire = true
	self.buffers.fireTime = params.time or 2
	self.buffers.fireSpeed = self.speed*1.5
end

function Enemy:addSlowBuffer(params) -- pic, attack, effect, time)
	if self.tornado then 
		self:updateHp(-params.attack)
		return 
	end
	--冰火泡不能共存
	self:clearFire()
	self:clearBubble()

	--if self == nil then return end
	if self.buffers.slow then
		self.buffers.slowTime = params.time
		self:updateHp(-params.attack,params.towerIdx)
		return
	end
	self.buffers.slowPic = params.pic
	self:addChild(self.buffers.slowPic)
	self.buffers.slow = true
	self.buffers.slowSpeed = self.speed*params.effect
	self.buffers.slowTime = params.time

	self:updateHp(-params.attack,params.towerIdx)
end

function Enemy:addBubbleBuffer(params)
	if self.tornado then 
		self:updateHp(-params.attack)
		return 
	end
	--冰火泡不能共存
	self:clearFire()
	self:clearSlow()

	--if self == nil then return end
	if self.buffers.bubble then
		self.buffers.bubbleTime = params.time
		self:updateHp(-params.attack)
		return
	end
	self.buffers.bubblePic = params.pic
	self.buffers.bubblePic:setPosition(0,self.enemySize.height*0.5)
	self:addChild(self.buffers.bubblePic)
	self.buffers.bubble = true
	self.buffers.bubbleSpeed = self.speed*params.effect
	self.buffers.bubbleTime = params.time

	self:updateHp(-params.attack)
end

function Enemy:addWeakBuffer(params)
	if self.tornado then 
		self:updateHp(-params.attack)
		return 
	end

	--if self == nil then return end
	if self.buffers.weak then
		self.buffers.weakTime = params.time
		self:updateHp(-params.attack)
		return
	end
	self.buffers.weakPic = params.pic
	self.buffers.weakPic:setPosition(0,self.enemySize.height*0.5)
	self:addChild(self.buffers.weakPic)
	self.buffers.weak = true
	self.buffers.weakSpeed = self.speed*params.slowEffect
	self.buffers.weakEffect = params.hurtEffect
	self.buffers.weakTime = params.time

	self:updateHp(-params.attack)
end

function Enemy:clearDizzy()
	if not self.buffers.dizzy then return end
	self.buffers.dizzy = false
	self.buffers.dizzyPic:removeFromParent()
end

function Enemy:clearStop()
	if not self.buffers.stop then return end
	self.buffers.stop = false
end

function Enemy:clearOpposite()
	if not self.buffers.opposite then return end
	self.buffers.opposite = false
	self.buffers.oppositePic:removeFromParent()
end

function Enemy:clearSlow()
	if not self.buffers.slow then return end
	self.buffers.slow = false
	self.buffers.slowPic:removeFromParent()
end

function Enemy:clearFire()
	if not self.buffers.fire then return end
	self.buffers.fire = false
	self.buffers.firePic:removeFromParent()
end

function Enemy:clearBubble()
	if not self.buffers.bubble then return end
	self.buffers.bubble = false
	self.buffers.bubblePic:removeFromParent()
end

function Enemy:clearWeak()
	if not self.buffers.weak then return end
	self.buffers.weak = false
	self.buffers.weakPic:removeFromParent()
end

function Enemy:clearAllState()
	self:clearDizzy()
	self:clearStop()
	self:clearSlow()
	self:clearFire()
	self:clearBubble()
	self:clearWeak()
	self:unlockByHero()
end

function Enemy:getSpeed()
	local speed = self.speed
	if self.buffers.skill or self.buffers.hero or 
			self.buffers.dizzy or self.buffers.stop then
		speed = 0
	elseif self.buffers.weak then
		speed = self.buffers.weakSpeed
	elseif self.buffers.slow then
		speed = self.buffers.slowSpeed
	elseif self.buffers.bubble then
		speed = self.buffers.bubbleSpeed
	elseif self.buffers.fire then
		speed = self.buffers.fireSpeed
	end
	if self.buffers.opposite then
		speed = -speed
	end
	return speed
end

function Enemy:updateForState(dt)
	if self.buffers.dizzy then
		self.buffers.dizzyTime = self.buffers.dizzyTime - dt
		if self.buffers.dizzyTime < 0 then
			self:clearDizzy()
		end
	end
	if self.buffers.slow then
		self.buffers.slowTime = self.buffers.slowTime - dt
		if self.buffers.slowTime < 0 then
			self:clearSlow()
		end
	end
	if self.buffers.stop then
		self.buffers.stopTime = self.buffers.stopTime - dt
		if self.buffers.stopTime < 0 then
			self:clearStop()
		end
	end
	if self.buffers.opposite then
		self.buffers.oppositeTime = self.buffers.oppositeTime - dt
		if self.buffers.oppositeTime < 0 then
			self:clearOpposite()
		end
	end
	if self.buffers.fire then
		self.buffers.fireTime = self.buffers.fireTime - dt
		if self.buffers.fireTime < 0 then
			self:clearFire()
		end
	end
	if self.buffers.bubble then
		self.buffers.bubbleTime = self.buffers.bubbleTime - dt
		if self.buffers.bubbleTime < 0 then
			self:clearBubble()
		end
	end
	if self.buffers.weak then
		self.buffers.weakTime = self.buffers.weakTime - dt
		if self.buffers.weakTime < 0 then
			self:clearWeak()
		end
	end
end

function Enemy:hurt()
	--[[取消受击动画
	self:stopAllActions()
	local frame = display.newSpriteFrame(self.hurtPic)
	self:setSpriteFrame(frame)
	self:runAction(transition.sequence({
		cc.DelayTime:create(0.3),
		cc.CallFunc:create(function()
			self:playAnimationForever(self.moveAct)
		end)
	}))
	--]]
end

function Enemy:secKill()
	self.hpBar:updateHp(0)
	local pic = display.newSprite("#secKill.png")
	local px,py = self:getPosition()
	pic:setPosition(cc.p(px,py+50))
	g_fightLayer:addChild(pic,300)
	pic:runAction(transition.sequence({
		cc.MoveBy:create(0.5, cc.p(0,20)),
		cc.CallFunc:create(function()
			pic:removeFromParent()
		end)
	}))
	--7号弹弓兔才能秒杀
	self:die(7)
end

--死于什么炮塔，如果不是空。
function Enemy:die(towerIdx,isHero)
	if g_playScene ~= nil then
		if g_playScene.task then
			g_playScene.task:killEnemy(towerIdx,self.idx,isHero)
		end
	end
	--音效
	SoundManager.playEnemyDieSound(self.idx)

	g_fightLayer:removeEnemy(self)
	--成就累计
	AchievementData.checkEnemies()
	--每日任务
	if isHero then
		AchievementData.checkHeroKillEnemy()
	end
end

function Enemy:updateHp(hpAdd, towerIdx, isHero)
	if not self.filled --黄瓜怪
			and not self.needUpdate then return end
	if hpAdd ~= nil then
		self.hp = self.hp + hpAdd
		if self.buffers.weak then
			--虚弱时多受伤害
			self.hp = self.hp + self.buffers.weakEffect*hpAdd
		end
	end
	if g_playScene.mode == "bossMode" and self.idx == 64 then
		g_playScene.bossHpBar:updateHp(self.hp)
	else
		self.hpBar:updateHp(self.hp)
	end	
	
	if self.hp<=0 then
		--变身还是变钱
		if self.changeTo ~= nil then
			self:takeOffEquipment()
		else
			self:playAnimationAfterDead()
			if g_playScene ~= nil then
				g_playScene:changeMoney(self.earn)
				g_fightLayer:moneyShow(self.earn, cc.p(self:getPosition()))
			end
			self:die(towerIdx,isHero)
		end
	else
		--Boss怪准备好技能时受到炮塔攻击就释放技能
		if self.idx > 40 and not isHero and self.bossSkillReady then
			self.bossSkillReady = false
			self:doSomething()
		end
		self:hurt()
	end
end

function Enemy:playAnimationAfterDead()
	--怪死后动画
	local px,py = self:getPosition()
	local dead = display.newSprite(EnemyData.common.deadPic,px,py)
	g_fightLayer:addChild(dead,300)
	local deadAni = display.getAnimationCache(EnemyData.common.deadAni)
	dead:playAnimationOnce(deadAni, true)
end

function Enemy:takeOffEquipment(param)
	local oldPic = self.localData.pic
	self:playAnimationAfterDead()
	self:initWithEnemyIdx(self.changeTo)
	local frame = display.newSpriteFrame(self.localData.pic)
	self.enemyPic:setSpriteFrame(frame)

	if param ~= nil and param.fortPos ~= nil then
		self.needUpdate = false
		local equipment = display.newSprite("#"..oldPic)
		g_fightLayer:addChild(equipment,BaseZOrder.enemy)
		equipment:setPosition(self:getPosition())
		equipment:runAction(transition.sequence({
			cc.MoveTo:create(0.5, param.fortPos),
			cc.CallFunc:create(function()
				self.needUpdate = true
				equipment:removeFromParent()
			end)
		}))
	end
end

function Enemy:updateForTornado(dt)
	self.tornadoTime = self.tornadoTime + dt
	local x = MapInfo.tileSize*0.8*math.cos(300*self.tornadoTime / 180 * math.pi)
	local y = MapInfo.tileSize*0.5*math.sin(300*self.tornadoTime / 180 * math.pi)
	local px,py = self.tornado:getPosition()
	self:setPosition(cc.p(px+x,py+MapInfo.tileSize*0.2+y))
end

--竹笋怪定时长出，幽灵怪隐身现形
function Enemy:updateForSpecial(dt)
end

function Enemy:update(dt)
	--子类处理
	self:updateForSpecial(dt)

	if not self.needUpdate then return end

	--龙卷风高级啊
	if self.tornado ~= nil then
		self:updateForTornado(dt)
		return
	end

	--[[
	if self.hpBar:isVisible() then
		self.hpTime = self.hpTime + dt
		if self.hpTime > 3 then
			self.hpBar:setVisible(false)
		end
	end
	]]
	self:updateForState(dt)
	
	--走到终点就返回false结束了。。
	if not self.pause then--移动控制
		if not self:updateForMove(dt) then return end
	end

	--CD在这里进行，子类只做具体的事不计时。

	--只有boss怪的cd是用来放技能的
	if self.idx > 40 and not self.bossSkillReady then
		if self.data.attackCD ~= nil then
			self.skillCD = self.skillCD+dt
			if self.skillCD >= 10 then --self.data.attackCD then
				self.bossSkillReady = true
				--self:doSomething()
				self.skillCD = 0
			end
		end
	end
	--跟英雄PK
	if self.data.attackCD ~= nil and self.buffers.hero ~= nil 
			and not self.buffers.hero.skilling then --英雄放技能时不攻击。
		self.cd = self.cd+dt
		--小怪就用excel表的cd，boss用2秒做cd
		if (self.idx <= 40 and self.cd >= self.data.attackCD)
				or (self.idx > 40 and self.cd >= 2) then
			if self.attackAct ~= nil then
				--放技能的状态要清掉，否则英雄死后怪就不动了。
				self.buffers.skill = false
				self.enemyPic:stopAllActions()
				self.enemyPic:playAnimationOnce(self.attackAct, false, function()
					self:attackHero()
					self.enemyPic:playAnimationForever(self.moveAct)
				end)
			else
				self:attackHero()
			end
			self.cd = 0
		end
	end
end

function Enemy:updateForMove(dt)
	--根据不同buff取一个速度
	self.currentSpeed = self:getSpeed()

	if self.buffers.opposite then
		if not self:setMovePositionOpposite(dt) then
			return true
		end
	else
		if not self:setMovePosition(dt) then
			return false 
		end
	end

	self:getMoveDist(dt)

	self.interval = self.interval + dt
	if self.interval > 0.1 then
		self.interval = self.interval - 0.1
		--self:resetOrder()
		self:gofloatWay(dt)
	end
	return true
end

function Enemy:setMovePosition(dt)
	local x,y = self:getPosition()
	if NEW_GUIDE and g_playScene.newbieguidelayer ~= nil and g_playScene.level == 102 and g_playScene.waveNow == 2 then
		local pt = MapInfo:getCenterOfTile(cc.p(4,1))
		if x > pt.x and g_playScene.newbieguidelayer.heroSkillControl == false then
			g_playScene.newbieguidelayer:goLevel2(3)
		end
	end
	--兔女郎退回到起点时。
	if self.wayPoint == 0 then
		self.wayPoint = 1
	end
	local orient = MapInfo.ways[self.wayIdx].wayOrient[self.wayPoint]
	--local targetPt = MapInfo:getCenterOfTile(MapInfo.ways[self.wayIdx].way[self.wayPoint])
	local targetPt = MapInfo.wayPoints[self.wayIdx][self.wayPoint]
	self.newPos.x = x
	self.newPos.y = y
	if not RANDOM_ENEMY_DIST then
		self.randmDist = 0
	else
		self.randmDist = self.randmDist or math.random(-MapInfo.tileSize*0.25,MapInfo.tileSize*0.25)
	end
	if orient == "right" then
		if x > targetPt.x+self.randmDist then
			self.newPos.x = targetPt.x+self.randmDist
			self.randmDist = nil
			self.wayPoint = self.wayPoint + 1
			--改为上下时朝守护点
			if self.orient == "left" and self.newPos.x < MapInfo.endPos.x then 
				self.orient = "right"
				self:setFlip(true)
			elseif self.orient == "right" and self.newPos.x > MapInfo.endPos.x then
				self.orient = "left"
				self:setFlip(false)
			end
		else
			self.newPos.x = self.newPos.x + self.currentSpeed*dt
			if self.orient == "left" then
				self.orient = "right"
				self:setFlip(true)
			end
		end
	elseif orient == "left" then
		if x < targetPt.x+self.randmDist then
			self.newPos.x = targetPt.x+self.randmDist
			self.randmDist = nil
			self.wayPoint = self.wayPoint + 1
			--改为上下时朝守护点
			if self.orient == "left" and self.newPos.x < MapInfo.endPos.x then 
				self.orient = "right"
				self:setFlip(true)
			elseif self.orient == "right" and self.newPos.x > MapInfo.endPos.x then
				self.orient = "left"
				self:setFlip(false)
			end
		else
			self.newPos.x = self.newPos.x - self.currentSpeed*dt
			if self.orient == "right" then
				self.orient = "left"
				self:setFlip(false)
			end
		end
	elseif orient == "up" then
		if y > targetPt.y+self.randmDist then
			self.newPos.y = targetPt.y+self.randmDist
			self.randmDist = nil
			self.wayPoint = self.wayPoint + 1
		else
			self.newPos.y = self.newPos.y + self.currentSpeed*dt
		end
	elseif orient == "down" then
		if y < targetPt.y+self.randmDist then
			self.newPos.y = targetPt.y+self.randmDist
			self.randmDist = nil
			self.wayPoint = self.wayPoint + 1
		else
			self.newPos.y = self.newPos.y - self.currentSpeed*dt
		end
	end
	if self.wayPoint > #MapInfo.ways[self.wayIdx].wayOrient then
		if g_playScene.mode == "bossMode" then
			self.wayPoint = 1
		else
			if g_playScene ~= nil then
				if self.idx <= 40 then --小怪
					g_playScene:setHeart(g_playScene.heartNum - 1)
				elseif self.idx <= 60 then --中型怪
					g_playScene:setHeart(g_playScene.heartNum - 2)
				else --Boss
					g_playScene:setHeart(g_playScene.heartNum - 5)
				end
				--音效
				if not g_playScene.soundHandler then
					g_playScene.soundHandler = SoundManager.playSound(GAME_SOUND.BabyRabbit[math.random(1,#GAME_SOUND.BabyRabbit)],false)
					g_playScene:performWithDelay(function ()
						g_playScene.soundHandler = nil
					end, 3.0)
				end
			end
			g_fightLayer:removeEnemy(self)
		end
		
		return false
	end
	
	self:setPosition(self.newPos)

	return true
end

function Enemy:setMovePositionOpposite(dt)
	if self.wayPoint == 0 then
		return false
	end
	local x,y = self:getPosition()
	local orient = MapInfo.ways[self.wayIdx].wayOrient[self.wayPoint]
	--local targetPt = MapInfo:getCenterOfTile(MapInfo.ways[self.wayIdx].way[self.wayPoint])
	local targetPt
	if self.wayPoint == 1 then
		targetPt = MapInfo.wayPoints[self.wayIdx].startPos
	else
		targetPt = MapInfo.wayPoints[self.wayIdx][self.wayPoint-1]
	end
	self.newPos.x = x
	self.newPos.y = y
	if not RANDOM_ENEMY_DIST then
		self.randmDist = 0
	else
		self.randmDist = self.randmDist or math.random(-MapInfo.tileSize*0.25,MapInfo.tileSize*0.25)
	end
	if orient == "right" then
		if x < targetPt.x+self.randmDist then
			self.newPos.x = targetPt.x+self.randmDist
			self.randmDist = nil
			self.wayPoint = self.wayPoint - 1
		else
			self.newPos.x = self.newPos.x + self.currentSpeed*dt
		end
		if self.orient == "right" then
			self.orient = "left"
			self:setFlip(false)
		end
	elseif orient == "left" then
		if x > targetPt.x+self.randmDist then
			self.newPos.x = targetPt.x+self.randmDist
			self.randmDist = nil
			self.wayPoint = self.wayPoint - 1
		else
			self.newPos.x = self.newPos.x - self.currentSpeed*dt
		end
		if self.orient == "left" then
			self.orient = "right"
			self:setFlip(true)
		end
	elseif orient == "up" then
		if y < targetPt.y+self.randmDist then
			self.newPos.y = targetPt.y+self.randmDist
			self.randmDist = nil
			self.wayPoint = self.wayPoint - 1
		else
			self.newPos.y = self.newPos.y + self.currentSpeed*dt
		end
	elseif orient == "down" then
		if y > targetPt.y+self.randmDist then
			self.newPos.y = targetPt.y+self.randmDist
			self.randmDist = nil
			self.wayPoint = self.wayPoint - 1
		else
			self.newPos.y = self.newPos.y - self.currentSpeed*dt
		end
	end

	if self.wayPoint == 0 then
		return false
	end
	
	self:setPosition(self.newPos)
	return true

end

function Enemy:resetOrder()
	local pos = cc.p(self:getPosition())
	local x,y = MapInfo:getTilePosition(pos)
	local order = BaseZOrder.enemy+y
	---[[
	local z = self:getLocalZOrder()
	if z ~= order then
		self:setLocalZOrder(order)
	end
	--]]

	--[[
	local pos2 = MapInfo:getCenterOfTile(tilePt)
	local distY = pos2.y+MapInfo.map:getTileSize().height*0.5-pos.y
	local z = self:getLocalZOrder()
	if z ~= tilePt.y*100+distY then
		self:setLocalZOrder(tilePt.y*100+distY)
	end
	--]]

end

function Enemy:gofloatWay(dt)
	--检查有没有掉坑里
	local tilePt = MapInfo:getTilePositionInCCP(cc.p(self:getPosition()))
	--local typeName = MapInfo.tileType[MapInfo:getTileIdx(tilePt)].name 
	if MapInfo:checkFloatWayBroken(tilePt) then
		self:dropPit()
	elseif self.idx == 44 and MapInfo:checkFloatWayGood(tilePt) then
		self:fillPit()
	end
end

--掉坑里
function Enemy:dropPit()
	--音效
	SoundManager.playSound(GAME_SOUND.Drop)
	
	self.needUpdate = false
	local tilePt = MapInfo:getTilePositionInCCP(cc.p(self:getPosition()))
	local pos = MapInfo:getCenterOfTile(tilePt)
	local orient = MapInfo.ways[self.wayIdx].wayOrient[self.wayPoint]
	---[[
	local dx
	if orient == "left" then
		dx = -35
	else
		dx = 35
	end
	self:runAction(transition.sequence({
		cc.JumpTo:create(0.3, cc.p(pos.x,pos.y-30), 30, 1),
		--[[
		cc.RotateBy:create(0.02,10),
		cc.RotateBy:create(0.02,-20),
		cc.RotateBy:create(0.02,20),
		cc.RotateBy:create(0.02,-10),
		--cc.Spawn:create(
			--cc.FadeOut:create(1),
			cc.EaseInOut:create(
				cc.MoveBy:create(0.2,cc.p(0,-30))
			,0.65),
		--),
		--]]
		cc.CallFunc:create(function()
			if g_playScene ~= nil then
				g_playScene:changeMoney(self.earn)
				g_fightLayer:moneyShow(self.earn, cc.p(self:getPosition()))
			end
			self:die()
		end)
	}))
	--]]
	--[[
	if orient == "left" then
		self:runAction(transition.sequence({
			cc.Spawn:create(
				cc.MoveBy:create(0.5, cc.p(-MapInfo.tileSize/2,0)),
				cc.RotateBy:create(0.5,-90)),
			cc.CallFunc:create(function()
				if g_playScene ~= nil then
					g_playScene:changeMoney(self.earn)
					g_fightLayer:moneyShow(self.earn, cc.p(self:getPosition()))
				end
				self:die()
			end)
		}))
	else
		self:runAction(transition.sequence({
			cc.Spawn:create(
				cc.MoveBy:create(0.5, cc.p(MapInfo.tileSize/2,0)),
				cc.RotateBy:create(0.5,90)),
			cc.CallFunc:create(function()
				if g_playScene ~= nil then
					g_playScene:changeMoney(self.earn)
					g_fightLayer:moneyShow(self.earn, cc.p(self:getPosition()))
				end
				self:die()
			end)
		}))
	end
	--]]
end

--被井盖兔拉进去
function Enemy:pullByPit(pos)
	self.needUpdate = false
	self:runAction(transition.sequence({
		cc.JumpTo:create(0.3, pos, MapInfo.tileSize, 1),
		cc.CallFunc:create(function()
			self:die()
		end)
	}))
end

function Enemy:attackHero()
	local reduceHp 
	if self.buffers.hero.angerTime ~= nil then
		reduceHp = self.attack/2
	else
		reduceHp = self.attack
	end
	--有可能攻击动作做完后英雄已经解除关联了。
	if self.buffers.hero ~= nil then
		self.buffers.hero:updateHp(-reduceHp)
	end
end

function Enemy:doSomething()
end

function Enemy:getNearFort(distSq)
	local minDist = distSq
	local fort = nil
	local en = nil
	if #g_fightLayer.forts == 0 then return end
	local pos = cc.p(self:getPosition())
	for i = 1, #g_fightLayer.forts do
		local fortPos = cc.p(g_fightLayer.forts[i]:getPosition())
		local disSq = cc.pDistanceSQ(pos, fortPos)
		if disSq < minDist then
			--print(disSq,"<",minDist)
			fort = g_fightLayer.forts[i];
			minDist = disSq
		end
	end
	return fort
end

function Enemy:getMoveDist(dt)
	if dt ~= nil then
		self.moveDist = self.moveDist + self.currentSpeed*dt
	else
		---消耗太大 可以偶尔用。
		--要先算出处在哪一段路上
		self.wayPoint = MapInfo:getWayPointByPos(self.wayIdx, cc.p(self:getPosition()))
		local parent = g_fightLayer
		local idx = 1
		self.moveDist = 0
		while idx < self.wayPoint do
			self.moveDist = self.moveDist +
			 	cc.pGetDistance(MapInfo.ways[self.wayIdx].startPt, MapInfo.wayPoints[self.wayIdx][idx])
			idx = idx + 1
		end
		if idx == 1 then
			self.moveDist = self.moveDist 
				+ cc.pGetDistance(cc.p(self:getPosition()), MapInfo:getCenterOfTile(MapInfo.ways[self.wayIdx].startPt))
		else
			self.moveDist = self.moveDist
				+ cc.pGetDistance(cc.p(self:getPosition()), MapInfo:getCenterOfTile(MapInfo.wayPoints[self.wayIdx][idx-1]))
		end
	end
end

return Enemy
