

local Bullets = import(".Bullets")
local MapInfo = import("..views.MapInfo")
local HpBar = import("..sprites.HpBar")
--local GuideLayer = import("..views.GuideLayer")
local Fort = class("Fort", function(params)
    return display.newSprite("#"..TowersData[params.towerIdx].pic)
end)

function Fort:ctor(params)
	self.idx = params.towerIdx
	self.data = TowersData[params.towerIdx]
	--[[塔的类型
	fortType = 1, --只有正面动画
	fortType = 2, --可360旋转
	fortType = 3, --有左右正反翻面
	--]]
	self.hasAttackAnimation = not self.data.noAttackAnimation --有些没有攻击动画。
	self.fortType = self.data.fortType or 3
	self.excelData = ExcelTowersData[params.towerIdx]
	--位置偏移
	if self.data.towerAnchor ~= nil then
		self:setAnchorPoint(cc.p(unpack(self.data.towerAnchor)))
	else
		self:setAnchorPoint(cc.p(0.5,0.4))
	end
	self.bulletIdx = self.data.bulletIdx
	--怒气
	self.angerInfo = {}
	if self.data.level.anger ~= nil then
		self.angerInfo = {
			hasAnger = true,
			angerState = 0, --0没开始，1,开始攒怒气，2,集满可释放,3,正在释放
			angerReadyTime = self.data.level.anger.angerReadyTime, --攒怒气时间
			angerConsumeTime = self.data.level.anger.angerTime, --消耗怒气时间
			readyTime = 0,
			consumeTime = 0,
		}
		if self.angerInfo.angerConsumeTime == 0 or self.angerInfo.angerConsumeTime == nil then
			self.angerInfo.oneTime = true
		else
			self.angerInfo.oneTime = false
		end
	else
		self.angerInfo.hasAnger = false
	end
	--现在不用兔子了所以全都不用怒气技能
	self.angerInfo.hasAnger = false

	self.pos = MapInfo:getCenterOfTile(params.tilePt)
    self:setPosition(self.pos)
    self.tilePt = params.tilePt
	self.wayPoint = 1
	self.passTime = 0
	self.target = nil
	self.thingTarget = nil
	self.cityTarget = nil
	self.selectNode = nil

	self.orient = "frontLeft"
	
    --刚建出来是1级，也可能是别的级别。
    self.level = params.level or 1
    --self.topLevel = GameData.myRabbit["r"..self.idx].level
    self.topLevel = 3 --没有兔子家养成了，普通炮塔都是3级。
	--自动延时三帧的时间
	--第一级要先算出来，不能放在getLevelAnimation中，因为可能造的不是1级。
	--拦路兔没有
	if self.hasAttackAnimation then
		if self.fortType == 3 and self.data.level[1].front ~= nil then
			local frontAttack = display.getAnimationCache(self.data.level[1].front.attack)
			self.baseFrameTime = frontAttack:getDelayPerUnit()
		elseif self.fortType == 1 and self.data.level[1].animationAttack ~= nil then
			local ani = display.getAnimationCache(self.data.level[1].animationAttack.animation)
			self.baseFrameTime = ani:getDelayPerUnit()
		elseif self.fortType == 2 and self.data.level[1].animationAttack ~= nil then
			local ani = display.getAnimationCache(self.data.level[1].animationAttack.animation)
			self.baseFrameTime = ani:getDelayPerUnit()
		else
			self.baseFrameTime = 0.1 --随便给个默认值。
		end
	end
	--自旋方式的塔
	if self.fortType == 2 then
		self.rotate = {
			speed = 0,
			rt = 0,
			time = 0,
		}
	end
	--升级提示
	self.upgradeTip = display.newSprite("#upgradeTip.png")
	g_fightLayer:addFightContent(self.upgradeTip)
	self.upgradeTip:setVisible(false)
	--兔子也要血条，血条要在LevelInfo前创建，升级时要设maxHp
	self.hpBar = HpBar.new(HPBAR_TYPE_FORT, self:getContentSize(), nil, cc.p(self:getPosition()))
	g_fightLayer:addFightContent(self.hpBar)
	self.hpBar:setVisible(false)
	--还要怒气条直接加先隐藏
	if self.angerInfo.hasAnger then
		self.angerBar = HpBar.new(HPBAR_TYPE_FORT_ANGER, self:getContentSize(), nil, cc.p(self:getPosition()))
		g_fightLayer:addFightContent(self.angerBar)
		self.angerBar:setVisible(false)
	end
	--等级信息获取
    self:getLevelInfo() --要在position之后。

	self.needUpdate = true
	--禁固
	self.isLocked = false
	self.needClick = 0
	--没动作三秒钟要给正面。
	self.backTime = 0

	--减少update消耗
	self.interval = 0
	self.intervalOrient = 0
	--攻击动画时不改变目标。
	self.isShooting = false
	--有的不打thing
	self.canHitThing = true
	--一开始就能攻击。
	self.passTime = self.speed
end

function Fort:init()
end

function Fort:getLevelInfo()
	--升级时取消状态
	self:clearBuffer()
	
	self.localLevelData = self.data.level[self.level]
	self.levelData = self.excelData.level[self.level]
	self.speed = self.levelData.speed or 1 --防止异常
	self.hpMax = self.levelData.hp
	self.hpBar:setMaxHP(self.hpMax)
	self.hp = self.levelData.hp
	self.removeMoney = self.levelData.removeMoney
	self.rotateSpeed = 180
	if self.levelData.range ~= nil then
		self.range = self.levelData.range * MapInfo.tileSize
		self.rangeSq = math.pow(self.range,2)
		self.rangeThingSq = math.pow(self.range+MapInfo.tileSize*0.5,2)
		self.rangeRect = cc.rect(self.pos.x-self.range,self.pos.y-self.range,self.range*2,self.range*2)
	end

	if self.hasAttackAnimation then
		if self.level ~= 1 then
			--升级后换图，预设的塔换图。
			if self.fortType == 1 or self.fortType == 2 then
				self:setSpriteFrame(display.newSpriteFrame(self.localLevelData.animationAttack.frame))
			elseif self.fortType == 3 then
				if self.orient == "frontLeft" or self.orient == "frontRight" then
					self:setSpriteFrame(display.newSpriteFrame(self.localLevelData.front.frame))
				else
					self:setSpriteFrame(display.newSpriteFrame(self.localLevelData.back.frame))
				end
			end
		end
		self:getLevelAnimation()
	else
		--没攻击动画就没攻击延时。
		self.attackDelay = 0
		--没动画但升级有图还得换。
		if self.localLevelData.frame ~= nil then
			self:setSpriteFrame(display.newSpriteFrame(self.localLevelData.frame))
		end
	end

	--旋转炮塔的底座
	if self.fortType == 2 and self.localLevelData.seat ~= nil then
		if self.seat ~= nil then
			self.seat:removeFromParent()
			self.seat = nil
		end
		self.seat = display.newSprite("#"..self.localLevelData.seat)
		self.seat:setPosition(self:getPosition())
		if self.data.seatAnchor ~= nil then
			self.seat:setAnchorPoint(cc.p(unpack(self.data.seatAnchor)))
		end
		g_fightLayer:addChild(self.seat, BaseZOrder.fort - 1)
	end
	--预览里面可以直接造三级的塔。到三级开启怒气条
	if self.level == #self.excelData.level then
		if self.angerInfo.hasAnger then
			self:showAnger()
		end
	end
end

function Fort:getLevelAnimation()
	if self.fortType == 1 or self.fortType == 2 then --原地动的塔
		if self.localLevelData.animationAttack ~= nil then
			self.animation = display.getAnimationCache(self.localLevelData.animationAttack.animation)
		end
		--自定义攻击延时，否则取三帧的时间。
		if self.localLevelData.attackDelay ~= nil then
			self.attackDelay  = self.localLevelData.attackDelay
		elseif self.level == 1 then
			self.attackDelay = self.baseFrameTime*3
		else
			--升级后延时按比例减少
			local dpu = self.baseFrameTime*self.speed/self.excelData.level[1].speed
			--不用调动画应该。
			--self.animation:setDelayPerUnit(dpu)
			self.attackDelay = dpu*3
		end
	elseif self.fortType == 3 then --翻面的塔
		--前后面攻击动画，有的只有前面动画。
		if self.localLevelData.front ~= nil then
			self.frontAttack = display.getAnimationCache(self.localLevelData.front.attack)
			if self.localLevelData.back ~= nil then
				self.backAttack = display.getAnimationCache(self.localLevelData.back.attack)
			end
			if self.level == 1 then
				self.attackDelay = self.baseFrameTime*3
			else
				--升级后延时按比例减少
				local dpu = self.baseFrameTime*self.speed/self.excelData.level[1].speed
				self.frontAttack:setDelayPerUnit(dpu)
				if self.localLevelData.back ~= nil then
					self.backAttack:setDelayPerUnit(dpu)
				end
				self.attackDelay = dpu*3
			end
		end
	end
end

function Fort:showAnger()
	--显示怒气条。
	self.angerBar:setVisible(true)
	self.angerInfo.angerState = 1
end

function Fort:updateAngerState(dt)
	if self.angerInfo.angerState == 1 then
		--print(self.angerInfo.readyTime)
		self.angerInfo.readyTime = self.angerInfo.readyTime + dt
		self.angerBar:updateHpCent(self.angerInfo.readyTime/self.angerInfo.angerReadyTime)
		if self.angerInfo.readyTime >= self.angerInfo.angerReadyTime then
			--变身了
			self:angerReady()
		end		
	elseif self.angerInfo.angerState == 3 and not self.angerInfo.oneTime then
		self.angerInfo.consumeTime = self.angerInfo.consumeTime + dt
		self.angerBar:updateHpCent((self.angerInfo.angerConsumeTime-self.angerInfo.consumeTime)/self.angerInfo.angerConsumeTime)
		if self.angerInfo.consumeTime >= self.angerInfo.angerConsumeTime and not self.isShooting then
			self:angerEnd()
		end		
	end
end

--攒完怒气变身
function Fort:angerReady()
	self.localLevelData = self.data.level.anger
	self:getLevelAnimation()
	self.angerInfo.angerState = 2
	--如果已经选中炮塔。显示释放技能按钮
	--[[
	if self.selectNode ~= nil then
		self.angerButton = display.newSprite("#towerUpgrade.png", 0, 60)
		self.selectNode:addChild(self.angerButton)
	end
	]]
	self.angerInfo.readyTime = 0 --重置为0
	self:setColor(cc.c3b(150,150,70))
end

function Fort:angerStart()
	if self.angerInfo.start then return end --防止连续按。
	self.angerInfo.start = true
	--[[
	local sprite = display.newSprite("#skillEffect001.png")
	sprite:setPosition(self:getPosition())
	g_fightLayer:addFightContent(sprite)
	sprite:setScale(3)

    local frames = display.newFrames("skillEffect%03d.png", 1, 18)
    local animation = display.newAnimation(frames, 0.1)
    sprite:playAnimationOnce(animation, true, function()
		self.angerInfo.angerTime = self.data.level.anger.angerTime
		self.angerInfo.angerState = 3
		self.angerInfo.start = false
		--这里不知是不是应该取anger级别的数值。
		self:setColor(cc.c3b(255,255,255))
    end)
    --]]
	self.angerInfo.angerState = 3
	self.angerInfo.start = false
	--变身，暂时用颜色表示
	self:setColor(cc.c3b(255,255,255))
	--anger的一些数值，如果有。
end

function Fort:angerEnd()
	--print("anger skill...")
	self:getLevelInfo()

	if self.orient == "frontLeft" or self.orient == "frontRight" then
		local frame = display.newSpriteFrame(self.localLevelData.front.frame)
		self:setSpriteFrame(frame)
	else
		local frame = display.newSpriteFrame(self.localLevelData.back.frame)
		self:setSpriteFrame(frame)
	end

	self.angerInfo.angerState = 1
	self.angerInfo.consumeTime = 0 --重置为0
end

function Fort:upgrade()
	--第3级是最高级，怒气是单独的key:anger。
	if self.level == #self.data.level then return end
	
	SoundManager.playSound(GAME_SOUND.TowerUpgrade)
	self.level = self.level+1
	self:getLevelInfo()
	g_playScene:changeMoney(-self.excelData.level[self.level].money)

	--播升级动画
	local px,py = self:getPosition()
	local upgrade = display.newSprite("#towerUpgrade.png",px,py)
	g_fightLayer:addFightContent(upgrade)
	local upgradeAni = display.getAnimationCache("upgradeAni")
	upgrade:playAnimationOnce(upgradeAni, true)

	--到三级开启怒气条
	if self.level == #self.excelData.level then
		if self.angerInfo.hasAnger then
			self:showAnger()
		end
	end
	--
	if self.level == 3 then
		if g_playScene.task then
	        g_playScene.task:createSuperFort(self.idx)
	    end
	end
end

function Fort:isUpgradeMoneyEnough()
	return self.excelData.level[self.level+1].money <= g_playScene.money
end

function Fort:getUpgradeMoney()
	if self.level >= self.topLevel then
		return false
	end
	return true, self.excelData.level[self.level+1].money
end

FortBuffer = {
	BUFFER_COOP = 1,
	BUFFER_STINK = 2,
	BUFFER_WRAP = 3,
	BUFFER_SNOW = 4,
	BUFFER_SWIRL = 5,
	BUFFER_SPEEDUP = 6,
}
function Fort:addFortBuffer(params)
	if self.coopInfo ~= nil
		or self.stinkInfo ~= nil
		or self.wrapInfo ~= nil
		or self.snowInfo ~= nil
		or self.swirlInfo ~= nil then return end
	if self.speedInfo ~= nil then
		self:cancelFortSpeed()		
	end

	if params.bufferType == FortBuffer.BUFFER_COOP then
		self:startCoop()
	elseif params.bufferType == FortBuffer.BUFFER_STINK then
		self:stink()
	elseif params.bufferType == FortBuffer.BUFFER_WRAP then
		self:startWrap()
	elseif params.bufferType == FortBuffer.BUFFER_SNOW then
		self:snow()
	elseif params.bufferType == FortBuffer.BUFFER_SWIRL then
		self:startSwirl()
	elseif params.bufferType == FortBuffer.BUFFER_SPEEDUP then
		self:fortSpeedUp()
	end
end

function Fort:clearBuffer()
	if self.coopInfo ~= nil then
		self:cancelCoop()
	elseif self.stinkInfo ~= nil then
		self:cancelStink()
	elseif self.wrapInfo ~= nil then
		self:cancelWrap()
	elseif self.snowInfo ~= nil then
		self:cancelSnow()
	elseif self.swirlInfo ~= nil then
		self:cancelSwirl()
	end
	if self.speedInfo ~= nil then
		self:cancelFortSpeed()	
	end
end

--被吸血鬼怪禁锢
function Fort:startCoop()
	local sprite = display.newSprite("#coop_01.png")
	sprite:setPosition(self.pos.x,self.pos.y+10)
	--g_fightLayer:addFightContent(sprite)
	g_fightLayer:addChild(sprite,BaseZOrder.fort)

	--手指提示
	self.hand = display.newSprite("#hand.png")
	self.hand:setPosition(self.pos.x+10,self.pos.y-5)
	self.hand:setScale(0.8)
	g_fightLayer:addChild(self.hand,BaseZOrder.fort)
	self.hand:runAction(cc.RepeatForever:create(transition.sequence({
			cc.ScaleTo:create(0.5,1),
			cc.ScaleTo:create(0.5,0.8),
	})))

    local frames = display.newFrames("coop_%02d.png", 1, 23)
    local animation = display.newAnimation(frames, 0.13)
    sprite:playAnimationOnce(animation, false, function()
    	self.hand:setVisible(false)
    end)

    --[[
	local sprite = display.newSprite("#jingu.png")
	sprite:setPosition(self:getPosition())
	g_fightLayer:addFightContent(sprite)
	sprite:setOpacity(0)
	sprite:runAction(cc.RepeatForever:create(transition.sequence({
		cc.FadeIn:create(0.5),
		cc.FadeOut:create(0.5),
	})))
	--]]
	self.coopInfo = {
		state = 1, --1准备禁锢，2已经禁锢
		sprite = sprite,
		count = 2,
		readyTime = 3,
		coopTime = 5,
	}
end

function Fort:coop()
	--self.coopInfo.sprite:stopAllActions()
	--self.coopInfo.sprite:
	self.coopInfo.state = 2
	self.isLocked = true
	if self.__cname == "Fort11" then
		self:stopAttack()
	end
end

function Fort:cancelCoop()
	self.coopInfo.sprite:removeFromParent()
	self.coopInfo = nil
	self.isLocked = false
	self.hand:removeFromParent()
end

--被洋葱怪熏
function Fort:stink()
	--[[local sprite = display.newSprite("#stink_1.png")
	sprite:setPosition(self:getPosition())
	g_fightLayer:addChild(sprite,BaseZOrder.enemy)
	local frames = display.newFrames("stink_%d.png", 1, 4)
    local animation = display.newAnimation(frames, 0.1)
    sprite:playAnimationForever(animation)

	self.stinkInfo = {
		sprite = sprite,
		stinkTime = 5,
	}
	self.speed = self.speed*2]]
end

function Fort:cancelStink()
	self.stinkInfo.sprite:removeFromParent()
	self.stinkInfo = nil
	self.speed = self.speed/2
end

--被法老缠绕
function Fort:startWrap()
	local sprite = display.newSprite("#wrap_1.png")
	sprite:setPosition(self.pos.x,self.pos.y+10)
	--g_fightLayer:addFightContent(sprite)
	g_fightLayer:addChild(sprite,BaseZOrder.fort)
    local frames = display.newFrames("wrap_%d.png", 1, 13)
    local animation = display.newAnimation(frames, 0.1)
    sprite:playAnimationOnce(animation, false, function()
		self:wrap()
    	local frames = display.newFrames("wrap_%d.png", 13, 5)
    	local aniLoop = display.newAnimation(frames, 0.1)
    	sprite:runAction(cc.RepeatForever:create(
    		cc.Animate:create(aniLoop)
		))
    	--self.hand:setVisible(false)
    end)

	--手指提示
	self.hand = display.newSprite("#hand.png")
	self.hand:setPosition(self.pos.x+10,self.pos.y-5)
	self.hand:setScale(0.8)
	g_fightLayer:addChild(self.hand,BaseZOrder.fort)
	self.hand:runAction(cc.RepeatForever:create(transition.sequence({
			cc.ScaleTo:create(0.5,1),
			cc.ScaleTo:create(0.5,0.8),
	})))

	self.isLocked = true
	self.wrapInfo = {
		state = 1, --1准备禁锢，2已经禁锢
		sprite = sprite,
		wrapTime = 8, --禁锢时间
		needClick = 1,
	}
end

function Fort:wrap()
	--self.coopInfo.sprite:stopAllActions()
	--self.coopInfo.sprite:
	self.wrapInfo.state = 2
	if self.__cname == "Fort11" then
		self:stopAttack()
	end
end

function Fort:cancelWrap()
	self.isLocked = false
	--蛇褪下
    local frames = display.newFrames("wrap_%d.png", 1, 13, true)
    local animation = display.newAnimation(frames, 0.1)
    self.wrapInfo.sprite:stopAllActions()
    self.wrapInfo.sprite:setSpriteFrame(display.newSpriteFrame("wrap_13.png"))
    self.wrapInfo.sprite:playAnimationOnce(animation, true)
	--self.wrapInfo.sprite:removeFromParent()
	self.wrapInfo = nil
	self.hand:removeFromParent()
end

--被雪人怪雪球砸中
function Fort:snow()
	local sprite = display.newSprite("#E62_bullet_1.png")
	sprite:setPosition(self.pos.x,self.pos.y+10)
	--g_fightLayer:addFightContent(sprite)
	g_fightLayer:addChild(sprite,BaseZOrder.fort)
    local frames = display.newFrames("E62_bullet_%d.png", 1, 6)
    local animation = display.newAnimation(frames, 0.05)
    sprite:playAnimationOnce(animation, false)

	--手指提示
	self.hand = display.newSprite("#hand.png")
	self.hand:setPosition(self.pos.x+10,self.pos.y-5)
	self.hand:setScale(0.8)
	g_fightLayer:addChild(self.hand,BaseZOrder.fort)
	self.hand:runAction(cc.RepeatForever:create(transition.sequence({
			cc.ScaleTo:create(0.5,1),
			cc.ScaleTo:create(0.5,0.8),
	})))

	self.isLocked = true
	self.snowInfo = {
		sprite = sprite,
		needClick = 2,
	}

	if self.__cname == "Fort11" then
		self:stopAttack()
	end
end

function Fort:cancelSnow()
	self.isLocked = false
	self.snowInfo.sprite:removeFromParent()
	self.snowInfo = nil
	self.hand:removeFromParent()
end

--波塞冬旋涡
function Fort:startSwirl()
	if self.selectNode ~= nil then
		self.selectNode:removeFromParent()
		self.selectNode = nil
	end
	local sprite = display.newSprite("#swirl1.png")
	local x,y = self:getPosition()
	sprite:setPosition(cc.p(x, y-10))
	g_fightLayer:addChild(sprite,BaseZOrder.fort-1)
    local frames = display.newFrames("swirl%d.png", 1, 8)
    local animation = display.newAnimation(frames, 0.1)
    sprite:playAnimationForever(animation)
    self:runAction(cc.FadeTo:create(3,0))
    if self.seat ~= nil then
	    self.seat:runAction(cc.FadeTo:create(3,0))
	end
	self.swirlInfo = {
		needClick = 2,
		sprite = sprite,
		time = 3,
	}
	self.isLocked = true

	--[[local rect = self:getTextureRect()	
	self.swirlInfo.width = rect.width
	self.swirlInfo.height = rect.height
	if self:isTextureRectRotated() then
		self.swirlInfo.speed = self.swirlInfo.height / self.swirlInfo.time
	else
		self.swirlInfo.speed = self.swirlInfo.width / self.swirlInfo.time
	end]]
end

function Fort:swirl()
	self.swirlInfo.sprite:removeFromParent()
	g_fightLayer:removeFort(self)
end

function Fort:cancelSwirl()
	self.isLocked = false
	self.swirlInfo.sprite:removeFromParent()
	self.swirlInfo = nil
end

--skill10
function Fort:fortSpeedUp()
	self.speedInfo = {
		time = 5,
	}
	self.speed = self.speed/2
	if self.hasAttackAnimation then
		if self.fortType == 1 or self.fortType == 2 then
			local dpu = self.animation:getDelayPerUnit()	
			self.animation:setDelayPerUnit(dpu/2)
		end
	end
	local animation = display.getAnimationCache("TowerEnhance")
	self.light = display.newSprite("#enhance_flag.png")
	self.light:setPosition(self:getPositionX(),self:getPositionY()+50)
	self.light:setVisible(false)
	self:getParent():addChild(self.light,self:getLocalZOrder()+1)
	self.light:runAction(cc.RepeatForever:create(transition.sequence({
			cc.ScaleTo:create(0.5,1.2),
			cc.ScaleTo:create(0.5,0.8)
		})))
	local sprite = display.newSprite()
	sprite:setAnchorPoint(cc.p(0.5,0.1))
	sprite:setPosition(self:getPositionX(),self:getPositionY()-30)
	self:getParent():addChild(sprite,self:getLocalZOrder()+1)
	sprite:runAction(transition.sequence({
		cc.Animate:create(animation),
		cc.CallFunc:create(function()
			self.light:setVisible(true)
			sprite:removeFromParent()
		end)
	}))
end
function Fort:cancelFortSpeed()
	self.speed = self.speed*2
	self.speedInfo = nil
	if self.hasAttackAnimation then
		if self.fortType == 1 or self.fortType == 2 then
			local dpu = self.animation:getDelayPerUnit()
			self.animation:setDelayPerUnit(dpu*2)
		end
	end
	if self.light then
		self.light:removeFromParent()
		self.light = nil
	end
end

--状态的更新
function Fort:updateState(dt)
	if self.speedInfo ~= nil then
		self.speedInfo.time = self.speedInfo.time - dt
		if self.speedInfo.time <= 0 then
			self:cancelFortSpeed()
		end
	end
	if self.coopInfo ~= nil then
		if self.coopInfo.state == 1 then
			self.coopInfo.readyTime = self.coopInfo.readyTime - dt
			if self.coopInfo.readyTime <= 0 then
				self:coop()
			end
		elseif self.coopInfo.state == 2 then
			self.coopInfo.coopTime = self.coopInfo.coopTime - dt
			if self.coopInfo.coopTime <= 0 then
				self:cancelCoop()
			end
		end
	end
	--法老缠绕,点击一次就消失,或8秒后消失。
	if self.wrapInfo ~= nil then
		if self.wrapInfo.state == 2 then
			self.wrapInfo.wrapTime = self.wrapInfo.wrapTime - dt
			if self.wrapInfo.wrapTime <= 0 then
				self:cancelWrap()
			end
		end
	end
	if self.stinkInfo ~= nil then
		self.stinkInfo.stinkTime = self.stinkInfo.stinkTime - dt
		if self.stinkInfo.stinkTime <= 0 then
			self:cancelStink()
		end
	end
	if self.swirlInfo ~= nil then
		--[[local rect = self:getTextureRect()	
		local width = rect.width
		local height = rect.height
		local dx = self.swirlInfo.speed * dt

		rect.height = height - dx
		if not self:isTextureRectRotated() then
			rect.y = rect.y - dx
		else
			rect.x = rect.x + dx
		end
		self:setTextureRect(rect,self:isTextureRectRotated(),cc.size(rect.width, rect.height))
		self:setPosition(cc.p(self:getPositionX(),self:getPositionY() - dx*0.2))

		--print(dx,rect.x,rect.y,rect:getMaxX(),rect:getMaxY())]]
		self.swirlInfo.time = self.swirlInfo.time - dt
		if self.swirlInfo.time <= 0 then
			self:swirl()
			return false
		end
	end
	return true
end

--点击去除炮塔状态
function Fort:clickState()
	--暂停时不能点状态。
	if g_playScene.pause and 
		(self.coopInfo or self.wrapInfo or self.swirlInfo or self.snowInfo) then 
		return true
	end
	if self.coopInfo ~= nil then
		if self.coopInfo.state == 1 then
			self.coopInfo.count = self.coopInfo.count - 1
			if self.coopInfo.count == 0 then
				self:cancelCoop()
				self.unlockTime = g_playScene.gamePassTime
			end
		end
	elseif self.wrapInfo ~= nil and self.wrapInfo.state == 2 then
		self.wrapInfo.needClick = self.wrapInfo.needClick - 1
		if self.wrapInfo.needClick <= 0 then
			self:cancelWrap()
			self.unlockTime = g_playScene.gamePassTime
		end
	elseif self.swirlInfo ~= nil then
		--[[self.swirlInfo.needClick = self.swirlInfo.needClick - 1
		if self.swirlInfo.needClick <= 0 then
			self:cancelSwirl()
			self.unlockTime = g_playScene.gamePassTime
		else
			if self.swirlInfo.needClick == 2 then
				self.swirlInfo.sprite:setOpacity(165)
			elseif self.swirlInfo.needClick == 1 then 
				self.swirlInfo.sprite:setOpacity(80)
			end
		end]]
	elseif self.snowInfo ~= nil then
		self.snowInfo.needClick = self.snowInfo.needClick - 1
		if self.snowInfo.needClick == 1 then
			self.snowInfo.sprite:setOpacity(128)
		elseif self.snowInfo.needClick == 0 then
			self:cancelSnow()
			self.unlockTime = g_playScene.gamePassTime
		end
	else
		return false
	end
	return true
end

function Fort:getPositionCenter()
	--取中心点，就直接坐标好了。
	local pt = cc.p(self:getPosition())
	return pt
end

function Fort:updateHp(hpAdd)
	if self.hp == nil then return end
	if hpAdd ~= nil then
		self.hp = self.hp + hpAdd
	end
	if self.hp > self.hpMax then
		self.hp = self.hpMax
	end
	self.hpBar:updateHp(self.hp)
	if self.hp <= 0 then
		g_fightLayer:removeFort(self)
	else
		--self:hurt()
	end
end

function Fort:onTouch(x,y)
	if self.isLocked then return end
	--拆塔
	local rect = self.removeButton:getCascadeBoundingBox()
	if cc.rectContainsPoint(rect, cc.p(x,y)) then
		--英雄技能给炮塔施加的状态的移除
		if self.speedInfo ~= nil then
			self:cancelFortSpeed()	
		end
		if g_playScene.task ~= nil then
			g_playScene.task:removeFort(self.idx)
		end
		self.selectNode:removeFromParent()
		self.selectNode = nil
		g_playScene:removeFort(self)
		--音效
		SoundManager.playSound(GAME_SOUND.TowerSell)
		return true
	end

	--升级
	if self.upgradeButton ~= nil then
		local rect = self.upgradeButton:getCascadeBoundingBox()
		if cc.rectContainsPoint(rect, cc.p(x,y)) then
			if self:isUpgradeMoneyEnough() then
				--[[if NEW_GUIDE and GameData.guide.step < 48 then
					if GameData.guide.step < 40 then
					elseif GameData.guide.step == 40 and g_playScene.level == 102 then
						if g_guidelayer.step == 43 then
							g_guidelayer:ShowFort(44)
						elseif g_guidelayer.step == 44 then
							g_playScene.pause = true
							g_playScene:suspend()
							g_guidelayer:ShowFort(45)
						end
					else
						
					end
				end]]
				--新手引导
				if (NEW_GUIDE and g_playScene.level == 100)
    				or (g_playScene.newbieguidelayer and g_playScene.newbieguidelayer.isTeach) then
			    	g_playScene.newbieguidelayer:createTower(6)
			    end
				--print("upgrade...")
				self.selectNode:removeFromParent()
				self.selectNode = nil
				self:upgrade()
				return true
			else
				--print("no money")
				return false
			end
		end
	end
	--未解锁
	if self.towerUpgradeLock ~= nil then
		local rect = self.towerUpgradeLock:getCascadeBoundingBox()
		if cc.rectContainsPoint(rect, cc.p(x,y)) then
			return false
		end
	end

	--放技能，改为直接点塔施放。
	--[[
	if self.angerButton ~= nil then
		local rect = self.angerButton:getCascadeBoundingBox()
		if cc.rectContainsPoint(rect, cc.p(x,y)) then
			if NEW_GUIDE and GameData.guide.step < 48 then
				g_guidelayer.next:setVisible(false)
            	g_guidelayer.clickMes:setVisible(false)
            end
			self:angerStart()
		end
	end
	--]]

	self.selectNode:setScale(1)
	self.selectNode:runAction(transition.sequence({
		cc.ScaleTo:create(0.1, 0),
		cc.CallFunc:create(function()
			self.selectNode:removeFromParent()
			self.selectNode = nil
		end)
	}))
	return true
end

function Fort:select()
	if self:clickState() then return end

	if self.angerInfo.angerState == 2 then
		self:angerStart()
		return
	end

	--被技能缠住后就不能升级删除了。
	if self.isLocked then return end

    g_fightLayer.selectedFort = self
    self:showSelectNode()
end

function Fort:getUpgradeButtonPosition()
	local x,y = MapInfo:getTilePosition(self:getPositionCenter())
	if y == 0 then
		if x < 6 then
			return cc.p(MapInfo.tileSize,10)
		else
			return cc.p(-MapInfo.tileSize,10)
		end
	elseif y ==1 and x >= 9 then
		return cc.p(-MapInfo.tileSize,10)
	else
		return cc.p(0,MapInfo.tileSize*1.4)
	end
end
function Fort:getRemoveButtonPosition()
	local x,y = MapInfo:getTilePosition(self:getPositionCenter())
	if y < 6 then
		return 0, -MapInfo.tileSize*1.1
	elseif x == 11 then
		return -MapInfo.tileSize,0
	else
		return MapInfo.tileSize,0
	end
end

function Fort:showSelectNode()
	--[[if NEW_GUIDE and GameData.guide.step < 48 then
		if GameData.guide.step > 21 and GameData.guide.step < 40 then
		elseif g_guidelayer and g_guidelayer.step == 5 then
		    g_guidelayer:GetMessage(8)			
		elseif GameData.guide.step == 40 then
			if g_playScene.level == 102 and g_guidelayer.step == 42 then
		    	g_guidelayer:ShowFort(43)
		    end		   
		end
    end]]
    --新手引导
    if (NEW_GUIDE and g_playScene.level == 100) or (g_playScene.newbieguidelayer ~= nil and g_playScene.newbieguidelayer.isTeach) then
    	g_playScene.newbieguidelayer:createTower(5)
    end
	if self.selectNode ~= nil then
		self.selectNode:removeFromParent()
	end
	local size = self:getContentSize()
	self.selectNode = display.newNode()
	self.selectNode:setAnchorPoint(cc.p(0.5,0.5))
	self.selectNode:setPosition(self:getPosition())
	g_fightLayer:addChild(self.selectNode, BaseZOrder.towerSelectNode)
	self.selectNode:setScale(0)

	--范围圈
	self.selectCircle = display.newSprite("#towerRange.png")
	self.selectCircle:setAnchorPoint(cc.p(0.5,0.5))
	size = self.selectCircle:getContentSize()
	local scaleTo = self.range*2/size.width
	self.selectCircle:setScale(scaleTo)
	self.selectNode:addChild(self.selectCircle)

	self.upgradeButton = nil
	self.topGradePic = nil
	self.towerUpgradeLock = nil
	--升级按钮
	local canUpgrade, upgradeMoney = self:getUpgradeMoney()
	if canUpgrade then
		if self:isUpgradeMoneyEnough() then
			self.upgradeButton = display.newSprite("#towerUpgrade.png")
		else
			self.upgradeButton = display.newSprite("#towerUpgradeGray.png")
		end	
		self.upgradeButton:setPosition(self:getUpgradeButtonPosition())
		--self.upgradeButton:setAnchorPoint(cc.p(0.5,0.5))
		local moneyLabel = cc.ui.UILabel.new({
            UILabelType = 1,
			text = ""..upgradeMoney,
            font = "fontFortMoney.fnt",
            x = 56,
            y = 15,
        }):align(display.CENTER)
        moneyLabel:setColor(cc.c3b(255,255,255))
		self.upgradeButton:addChild(moneyLabel)
		self.selectNode:addChild(self.upgradeButton)
	else
		if self.level == 3 then
			self.topGradePic = display.newSprite("#towerTopGrade.png")
			self.topGradePic:setPosition(self:getUpgradeButtonPosition())
			self.selectNode:addChild(self.topGradePic)
		else --未解锁
			self.towerUpgradeLock = display.newSprite("#towerUpgradeLock.png")
			self.towerUpgradeLock:setPosition(self:getUpgradeButtonPosition())
			self.selectNode:addChild(self.towerUpgradeLock)	
		end
	end

	--技能按钮，取消，直接点塔施放
	--[[
	if self.angerInfo.angerState == 2 then
		self.angerButton = display.newSprite("#towerUpgrade.png", 0, MapInfo.tileSize*1.1)
		self.selectNode:addChild(self.angerButton)
	else
		self.angerButton = nil
	end
	--]]

	--删除按钮
	self.removeButton = display.newSprite("#towerRemove.png")
	self.removeButton:setPosition(self:getRemoveButtonPosition())
	--self.removeButton:setAnchorPoint(cc.p(0.5,0.5))
	local removeMoneyLabel = cc.ui.UILabel.new({
        UILabelType = 1,
		text = ""..self.removeMoney,
        font = "fontFortMoney.fnt",
		x = 53,
		y = 15,
	}):align(display.CENTER)
	self.removeButton:addChild(removeMoneyLabel)
	self.selectNode:addChild(self.removeButton)
	self.selectNode:runAction(cc.ScaleTo:create(0.1, 1))
end

function Fort:getNearEnemy(distSq)
	local d2 = distSq or self.rangeSq
	local en = nil
	--取怪的座标
	local pos = cc.p(self:getPosition())
	for i = 1, #g_fightLayer.enemies do
		local enPos = cc.p(g_fightLayer.enemies[i]:getPosition())
		local disSq = cc.pDistanceSQ(pos, enPos)
		if disSq < d2 then
			en = g_fightLayer.enemies[i];
			break
		end
	end
	return en
end

function Fort:isCityInRange(city)
	--使用点距离判断
	local disSq = cc.pDistanceSQ(cc.p(self:getPosition()), cc.p(city:getPosition()))
	if disSq < self.rangeThingSq then
		return true
	else
		return false
	end
	--]]
end

function Fort:isEnemyInRange(en)
	--[[
	--使用矩形碰撞检测
	return cc.rectIntersectsRect(self.rangeRect, en:getRect())
	--]]
	---[[
	--使用点距离判断
	local disSq = cc.pDistanceSQ(cc.p(self:getPosition()), cc.p(en:getPosition()))
	if disSq < self.rangeThingSq then
		return true
	else
		return false
	end
	--]]
end

--不用了，用fightLayer的函数。
function Fort:isThingInRange(thing)
	--[[太粗了，特别是斜线上的道具。
	--使用矩形碰撞检测
	if cc.rectIntersectsRect(self.rangeRect, thing) then
		return true
	else
		return false
	end
	--]]
	---[[
	--使用点距离判断，加上半格
	--大道具每块判断
	for i = 1, thing.width do
		for j = 1, thing.height do
			local thingPt =	MapInfo:getCenterOfTile(cc.p(thing.tilePt.x-1+i,
				thing.tilePt.y+1-j))
			local disSq = cc.pDistanceSQ(self.pos, thingPt)
			if disSq < self.rangeThingSq then
				return true
			end
		end
	end
	return false
	--]]
end

function Fort:checkTargetEnemy(en)
	--水萝卜不能打
	if en.underWater then return false end
	return true
end

function Fort:getTargetCity()
	if g_fightLayer.selectedCity ~= nil then
		if self:isCityInRange(g_fightLayer.selectedCity) then
			return g_fightLayer.selectedCity
		end
	end
end

function Fort:getTargetEnemy()
	--先判断选中的怪
	if g_fightLayer.selectedEnemy ~= nil then
		--子类自己写判断条件。
		if self:checkTargetEnemy(g_fightLayer.selectedEnemy) then
			if self:isEnemyInRange(g_fightLayer.selectedEnemy) then
				if self.target ~= g_fightLayer.selectedEnemy then
					self.changeTarget = true
				end
				return g_fightLayer.selectedEnemy
			end
		end
	end
	local en = nil
	--再按条件找所有怪
	for i = 1, #g_fightLayer.enemies do
		--子类自己写判断条件。
		if self:checkTargetEnemy(g_fightLayer.enemies[i]) then
			if self:isEnemyInRange(g_fightLayer.enemies[i]) then
				local underE45 = g_fightLayer.enemies[i].E45 and not g_fightLayer.enemies[i].hasChanged--地下的竹笋怪不自动攻击
				if g_fightLayer.enemies[i].canHit and not underE45 then 
					if en == nil then
						en = g_fightLayer.enemies[i]
					elseif g_fightLayer.enemies[i].moveDist > en.moveDist then
						en = g_fightLayer.enemies[i]
					end
				end
			end
		end
	end
	if self.target ~= en then
		self.changeTarget = true
	end
	return en
end

function Fort:getTargetThing()
	--不能打道具的直接跳过。
	if not self.canHitThing then
		self.thingTarget = nil
	elseif g_fightLayer.selectedTarget ~= nil 
			and self.thingTarget ~= g_fightLayer.selectedTarget then
		--判断道具是否在攻击范围内
		if g_fightLayer:isThingInPosRange(g_fightLayer.selectedTarget, self.pos, self.rangeThingSq) then
			self.thingTarget = g_fightLayer.selectedTarget
			self.target = nil
			self.passTime = 0
			self.changeTarget = true
		else
			self.thingTarget = nil
		end
	elseif g_fightLayer.selectedTarget == nil then
		self.thingTarget = nil
	end
end

function Fort:setFortRotateByEnemy(enPt)
	--只有不移动的道具用计时方式旋转，减少计算。怪物还是实时旋转。
	--改变目标才做旋转动作，否则直接在update设置旋转值。
	if self.changeTarget then
		local rt = kkFunc.getLineRotation(cc.p(self:getPosition()), enPt)
		self.rotate.rt = rt
		local rtDelta = (rt - self:getRotation())%360
		if math.abs(rtDelta) > 180 then
			self.rotate.time = (360 - math.abs(rtDelta))/self.rotateSpeed
		else
			self.rotate.time = math.abs(rtDelta)/self.rotateSpeed
		end
		if rtDelta > 180 or (rtDelta < 0 and rtDela > -180) then
			self.rotate.speed = -self.rotateSpeed
		else
			self.rotate.speed = self.rotateSpeed
		end
	end
end

--fortType == 3 翻面的方法
function Fort:setOrient3(enPt)
	local px,py = self:getPosition()
	local orient
	if enPt.y > py then
		if enPt.x > px then
			orient = "backRight"
		else
			orient = "backLeft"
		end
	else
		if enPt.x > px then
			orient = "frontRight"
		else
			orient = "frontLeft"
		end
	end
	if orient ~= self.orient then
		if orient == "frontLeft" or orient == "frontRight" then
			local frame = display.newSpriteFrame(self.localLevelData.front.frame)
			self:setSpriteFrame(frame)
			if orient == "frontRight" then
				self:setFlippedX(true)
			else
				self:setFlippedX(false)
			end
		else
			local frame = display.newSpriteFrame(self.localLevelData.back.frame)
			self:setSpriteFrame(frame)
			if orient == "backRight" then
				self:setFlippedX(true)
			else
				self:setFlippedX(false)
			end
		end
	end
	self.orient = orient
end

function Fort:setOrientByTargetPos(enPt)
	if self.fortType == 1 then
	elseif self.fortType == 2 then
		--只对道具做旋转预设。
		if self.thingTarget ~= nil then
			self:setFortRotateByEnemy(enPt)
		end
	elseif self.fortType == 3 then
		self:setOrient3(enPt)
	end
end

function Fort:updateInterval()
	self.changeTarget = false
	if self.isShooting == false then
		--优先打城堡
		self.cityTarget = self:getTargetCity()
		--先找有可有可打道具
		self:getTargetThing()
		if self.cityTarget ~= nil then
			self:setOrientByTargetPos(cc.p(self.cityTarget:getPositionCenter()))
		elseif self.thingTarget ~= nil then
			--塔的朝向
			self:setOrientByTargetPos(MapInfo:getCenterOfThing(self.thingTarget))
		else --不打道具才找怪
			--找怪目标。子类可能重写。
			self.target = self:getTargetEnemy()
			if self.target ~= nil then
				--塔的朝向
				self:setOrientByTargetPos(cc.p(self.target:getPositionCenter()))
			elseif self.fortType == 3 then
				--3秒转正面，在这里判断正好没有目标。
				self.backTime = self.backTime + self.interval
				if self.backTime >= 3 then
					if self.orient == "backLeft" or self.orient == "backRight" then
						local frame = display.newSpriteFrame(self.localLevelData.front.frame)
						self:setSpriteFrame(frame)
						if self.orient == "backLeft" then
							self:setFlippedX(false)
							self.orient = "frontLeft"
						else
							self:setFlippedX(true)
							self.orient = "frontRight"
						end
					end
				end
			end
		end
	end
end

function Fort:updateUpgradeState()
	if self.selectNode ~= nil then
		if self.upgradeButton ~= nil then
			if self:isUpgradeMoneyEnough() then
				local frame = display.newSpriteFrame("towerUpgrade.png")
				self.upgradeButton:setSpriteFrame(frame)
			else
				local frame = display.newSpriteFrame("towerUpgradeGray.png")
				self.upgradeButton:setSpriteFrame(frame)
			end
		end
	end
end

function Fort:showUpgradeTip(show)
	local b = self.upgradeTip:isVisible()
	if show and not b then
		self.upgradeTip:setVisible(true)
		local x, y = self:getPosition()
		self.upgradeTip:setPosition(x+23, y+30)
		self.upgradeTip:runAction(cc.RepeatForever:create(
			transition.sequence({
				cc.MoveBy:create(0.4, cc.p(0,5)),
				cc.MoveBy:create(0.4, cc.p(0,-5)),
			})
		))
	elseif not show and b then
		self.upgradeTip:stopAllActions()
		self.upgradeTip:setVisible(false)
	end
end

function Fort:updateForChangeMoney(money)
	if self.selectNode ~= nil
			and self.upgradeButton ~= nil then
		if money >= self.excelData.level[self.level+1].money then
			local frame = display.newSpriteFrame("towerUpgrade.png")
			self.upgradeButton:setSpriteFrame(frame)
		else
			local frame = display.newSpriteFrame("towerUpgradeGray.png")
			self.upgradeButton:setSpriteFrame(frame)
		end
	else
		if self.level < self.topLevel and money >= self.excelData.level[self.level+1].money then
			self:showUpgradeTip(true)
		else
			self:showUpgradeTip(false)
		end
	end
end

function Fort:update(dt)
	if not self.needUpdate then return end
	--各种状态更新，漩涡之后就死了。
	if not self:updateState(dt) then return end

	if self.isLocked then return end

	--持续怒气时间更新
	if self.angerInfo.hasAnger then
		self:updateAngerState(dt)
	end

	--升级钱够了更新状态，也许以后有别的要更新。
	--由playScene做统一changeMoney控制。
	--self:updateUpgradeState()

	--子类不同
	self:updateSelf(dt)
end

function Fort:updateSelf(dt)
	self.interval = self.interval + dt
	if self.interval > 0.1 then
		self:updateInterval()
		self.interval = 0
	end

	if self.cityTarget == nil and self.thingTarget == nil and self.target == nil then 
		return 
	end
	if self.fortType == 2 then
		if self.thingTarget ~= nil then
			if self.rotate.time > 0 then
				self.rotate.time = self.rotate.time - dt
				local rt = self:getRotation() + dt * self.rotate.speed
				self:setRotation(rt)
				if self.rotate.time <= 0 then
					self:setRotation(self.rotate.rt)
				end
				--旋转时不发子弹。
				self.passTime = self.passTime + dt
				return
			else
				self.rotate.time = 0
			end
		elseif self.cityTarget ~= nil then
			local speed = self.rotateSpeed * dt
			local rtNow = self:getRotation()
			local rt = kkFunc.getLineRotation(cc.p(self:getPosition()), cc.p(self.cityTarget:getPositionCenter()))
			local rtDiff = rt - rtNow
			if rtDiff < 0 then rtDiff = rtDiff + 360 end
			if rtDiff <= speed or rtDiff >= 360 - speed then
				self:setRotation(rt)
			else
				if rtDiff <= 180 then
					self:setRotation(rtNow+speed)
				else
					self:setRotation(rtNow-speed)
				end
				--旋转时不发子弹。
				self.passTime = self.passTime + dt
				return
			end
		elseif self.target ~= nil then
			local speed = self.rotateSpeed * dt
			local rtNow = self:getRotation()
			local rt = kkFunc.getLineRotation(cc.p(self:getPosition()), cc.p(self.target:getPositionCenter()))
			local rtDiff = rt - rtNow
			if rtDiff < 0 then rtDiff = rtDiff + 360 end
			if rtDiff <= speed or rtDiff >= 360 - speed then
				self:setRotation(rt)
			else
				if rtDiff <= 180 then
					self:setRotation(rtNow+speed)
				else
					self:setRotation(rtNow-speed)
				end
				--旋转时不发子弹。
				self.passTime = self.passTime + dt
				return
			end
		end
	end
	self.passTime = self.passTime + dt
	if self.passTime >= self.speed then
		--有攻击动画，非等待时有持续动画。
		if self.hasAttackAnimation and not self.data.waitAnimation then
			if self.fortType == 1 or self.fortType == 2 then
				self:playAnimationOnce(self.animation, false, function()
					self:setSpriteFrame(display.newSpriteFrame(self.localLevelData.animationAttack.frame))
				end)
			elseif self.fortType == 3 then
				if self.orient == "frontLeft" or self.orient == "frontRight" then
					self:playAnimationOnce(self.frontAttack, false, function()
						self:setSpriteFrame(display.newSpriteFrame(self.localLevelData.front.frame))
					end)
				else
					self:playAnimationOnce(self.backAttack, false, function()
						self:setSpriteFrame(display.newSpriteFrame(self.localLevelData.back.frame))
					end)
				end
			end
		end
		self.isShooting = true
		--怒气子弹不一样。
		if self.angerInfo.hasAnger and self.angerInfo.angerState == 3 then
			self:performWithDelay(function()
				--转正时间清零
				self.backTime = 0
				self:createAngerBullet()
				--怒气就一发
				if self.angerInfo.oneTime then
					self:angerEnd()
				end
			end, self.attackDelay)
		else
			self:performWithDelay(function()
				--转正时间清零
				self.backTime = 0
				self:createBullet()
			end, self.attackDelay)
		end
		self.passTime = 0 --旋转时会蓄力不发。所以时间还是清零好。
	end
end

function Fort:playAttack1Sound()
	SoundManager.playTowerAttack1Sound(self.idx)
end

function Fort:playSkill1Sound()
	SoundManager.playTowerSkill1Sound(self.idx)
end

function Fort:createAngerBullet()
	if self.cityTarget == nil and self.target == nil and self.thingTarget == nil then 
		self.isShooting = false
		return 
	end

	self:playSkill1Sound()
	--local deltaPos = self:getAttackDeltaPos()
	local bullet = Bullets[self.bulletIdx].new({
			idx = self.bulletIdx,
			--fortPos = deltaPos,
			fortPos = cc.p(self:getPosition()),
			target = self.target,
			thingTarget = self.thingTarget,
			cityTarget = self.cityTarget,
			level = 4,
			range = self.range,
			towerIdx = self.idx
		});
	g_fightLayer:addBullet(bullet)
	self.isShooting = false	
end

function Fort:createBullet()
	if self.cityTarget == nil and self.target == nil and self.thingTarget == nil then 
		self.isShooting = false
		return 
	end

	self:playAttack1Sound()
	--local deltaPos = self:getAttackDeltaPos()
	local bullet = Bullets[self.bulletIdx].new({
			idx = self.bulletIdx,
			--fortPos = deltaPos,
			fortPos = cc.p(self:getPosition()),
			target = self.target,
			thingTarget = self.thingTarget,
			cityTarget = self.cityTarget,
			level = self.level,
			range = self.range,
			towerIdx = self.idx
		})
	g_fightLayer:addBullet(bullet)
	self.isShooting = false
end

function Fort:getAttackDeltaPos()
	local px,py = self:getPosition()
	if self.data.attackDx ~= nil and self.data.attackDy ~= nil then
		if self.orient == "frontLeft" then
			px = px - self.data.attackDx
			py = py - self.data.attackDy
		elseif self.orient == "frontRight" then
			px = px + self.data.attackDx
			py = py - self.data.attackDy
		elseif self.orient == "backLeft" then
			px = px - self.data.attackDx
			py = py + self.data.attackDy
		elseif self.orient == "backRight" then
			px = px + self.data.attackDx
			py = py + self.data.attackDy
		end
	end
	return cc.p(px,py)
end

function Fort:callBeforRemove()
	self.upgradeTip:removeFromParent()
	self.hpBar:removeFromParent()
	if self.angerInfo.hasAnger then
		self.angerBar:removeFromParent()
	end
	if self.seat ~= nil then
		self.seat:removeFromParent()
		self.seat = nil
	end
end

return Fort