--
-- Author: KK
-- Date: 2014-08-25 13:10:53
--
--警灯兔

--基础转速,几秒一圈
local ROTATE_SPEED = 3

local MapInfo = import("..views.MapInfo")
local Bullets = import(".Bullets")

local Fort = import(".Fort")

local Fort16 = class("Fort16", Fort)

function Fort16:ctor(params)
    -- 如果继承类覆盖了 ctor() 构造函数，那么必须手动调用父类构造函数
    -- 类名.super 可以访问指定类的父类
    Fort16.super.ctor(self, params)
    self.pos = cc.p(self:getPosition())
    self.lightTime = 0
    --self.hitTimeForAnger = 0
end

function Fort16:initFan()
	if self.fanNode ~= nil then
		self.fanNode:removeFromParent()
	end
	self.fanNode = display.newNode()
	self.deltaRotation = 360 / self.level
	for i = 1, self.level do
		local fan = display.newSprite("#policeLight.png")
		fan:setAnchorPoint(cc.p(0,0.5))
		fan:setRotation((i-1)*self.deltaRotation)
		self.fanNode:addChild(fan)
	end
	local size = self:getContentSize()
	self.fanNode:setPosition(cc.p(self:getPosition()))
	g_fightLayer:addChild(self.fanNode,300)

	self.fanNode:setVisible(false)
    self.angle = 0
    self.lighting = false
    self.calcTime = 0
	self.attack = self.levelData.attack

    --直接把fanTime写到目标上，但时间更新在这个兔子里做。
    self.fanRecords = {}
end

function Fort16:getLevelInfo()
	Fort16.super.getLevelInfo(self)
	self:initFan()
end

function Fort16:isTargetInFan(pt, angle)
	local pt1 = cc.p(pt.x,pt.y)
	local pt2 = cc.p(100*math.cos(angle/180*3.1416),100*math.sin(angle/180*3.1416))

	pt1.x = pt1.x - self.pos.x
	pt1.y = pt1.y - self.pos.y

	--print("Angle:",math.cos(20/180*3.1416),angle,pt2.x,pt2.y,pt1.x,pt1.y)

	local cosValue = (pt1.x*pt2.x+pt1.y*pt2.y) /
					 (math.sqrt(pt1.x*pt1.x+pt1.y*pt1.y) * 
					 	math.sqrt(pt2.x*pt2.x+pt2.y*pt2.y))
	--print(cosValue)
	if cosValue >= 0.9 then
		return true
	end
	return false					 
end

function Fort16:hitSomething()
	self.hasHit = false --打过就要重计时延迟停闪。
	for fanNum = 1, self.level do
		local angle = self.angle+(fanNum-1)*self.deltaRotation
		local enemies = g_fightLayer:getEnimiesAroundPos(self.pos, self.rangeSq)
		for i = 1, #enemies do
			local en = enemies[i]
			if en ~= nil and not en.underWater then
				self:fanTarget(angle, en)
			end
		end
		local things = g_fightLayer:getThingsAroundPos(self.pos, self.rangeSq)
		for i = 1, #things do
			if things[i].part == false then
				self:fanTarget(angle, things[i], true)
			end
		end
	end
	if self.hasHit then
		self.lightTime = 0
		self.backTime = 0
	end
end

function Fort16:fanTarget(angle, target, isThing)
	--判断有没有被这片叶子扇过
	local faned = false
	for i = 1, #self.fanRecords do
		if self.fanRecords[i] == target then
			faned = true
			--print("faned")
			break
		end
	end
	if faned then return end
	
	if not isThing then
		local en = target
		if self:isTargetInFan(cc.p(en:getPosition()), angle) then
			self.hasHit = true
			en.hp = en.hp - self.attack
			en:updateHp()
			if en ~= nil then
				self.fanRecords[#self.fanRecords+1] = en
				en.fanTime = 0.3
			end
		end
	else
		local thing = target
		if self:isTargetInFan(thing.pos, angle) then
			self.hasHit = true
			g_fightLayer:hitThing(thing, self.attack)
			if thing ~= nil then
				self.fanRecords[#self.fanRecords+1] = thing
				thing.fanTime = ROTATE_SPEED*0.8
			end
		end
	end
end

--更新被扇过的目标时间。
function Fort16:updateFanTimeForTargets(dt)
	local ts = {}
	for i = 1, #self.fanRecords do
		local target = self.fanRecords[i]
		if target == nil then
			ts[#ts+1] = i
		elseif target.fanTime ~= nil then
			target.fanTime = target.fanTime - dt
			if target.fanTime < 0 then
				ts[#ts+1] = i
			end
		end
	end
	for i = #ts, 1, -1 do
		table.remove(self.fanRecords, i)
	end
end

function Fort16:updateLight(dt)
	if not self.lighting then return end
	self.angle = self.angle + (360/ROTATE_SPEED)*dt
	if self.angle >= 360 then self.angle = self.angle - 360 end
	self.fanNode:setRotation(-self.angle)

	self.calcTime = self.calcTime + dt
	if self.calcTime > 0.05 then
		self:hitSomething()
		self.calcTime = self.calcTime - 0.05
	end

	--更新被扇过的目标时间。
	self:updateFanTimeForTargets(dt)

	--最少时间1秒
	self.lightTime = self.lightTime + dt

end

function Fort16:angerStart()
	Fort16.super.angerStart(self)

	self.fanNode:setScale(4)
	self.rangeSq = self.rangeSq*16 --结束时会自动取3级的数值
	self.attack = self.data.level.anger.attack
end

function Fort16:angerEnd()
	Fort16.super.angerEnd(self)

	self.fanNode:setScale(1)
	self.attack = self.levelData.attack
end

function Fort16:updateSelf(dt)
	self.interval = self.interval + dt
	if self.interval > 0.1 then
		self:updateInterval()
		self.interval = 0
	end

	local hasTarget = self.thingTarget ~= nil or self.target ~= nil
	if self.lighting then
		if self.lightTime > 1 and not hasTarget then
			self:stopAttack()
			return
		end
		self:updateLight(dt)
	end

	if hasTarget then
		if not self.lighting then 
			if self.orient == "frontLeft" or self.orient == "frontRight" then
				self:playAnimationForever(self.frontAttack)
			else
				self:playAnimationForever(self.backAttack)
			end
			self:shining()
		else
			if (self.lastOrient == "backLeft" or self.lastOrient == "backRight")
					and (self.orient == "frontLeft" or self.orient == "frontRight") then
				self:stopAllActions()
				self:playAnimationForever(self.frontAttack)
			elseif (self.lastOrient == "frontLeft" or self.lastOrient == "frontRight")
					and (self.orient == "backLeft" or self.orient == "backRight") then
				self:stopAllActions()
				self:playAnimationForever(self.backAttack)
			end
		end
		self.lastOrient = self.orient
	end
end

function Fort16:stopAttack()
	self:stopAllActions()
	self.fanNode:stopAllActions()
	self.fanNode:setVisible(false)
	self.lighting = false
end

function Fort16:shining()
	self.fanNode:setVisible(true)
	self.lighting = true
	self.lightTime = 0
end

function Fort16:callBeforRemove()
	Fort16.super.callBeforRemove(self)
	if self.fanNode ~= nil then
		self.fanNode:removeFromParent()
	end
end

return Fort16