--
-- Author: KK
-- Date: 2014-11-04 09:29:10
--

local HpBar = import("..sprites.HpBar")
local MapInfo = import("..views.MapInfo")

local Hero = class("Hero", function(params)
	return display.newNode()
end)

function Hero:ctor(params)
	self.shadow = display.newSprite("#heroShadow.png",0,0):addTo(self)
	self.heroPic = display.newSprite("#"..HeroData[params.idx].pic):addTo(self,2)
	self.idx = params.idx
	self.data = HeroData[self.idx]
	self.level = GameData.heros["h"..self.idx].level
	self:setPosition(params.pos)
	self:setAnchor(cc.p(unpack(self.data.anchor)))
	--触摸框大小
	self.rect = self.data.touchRect
	--等待时间，回指定位置
	self.waitInfo = {
		pos = cc.p(params.pos),
		waiting = true, --在指定位置等待。
		waitingTime = 0, --不在指定位置，需计时回到指定位置
		hpTime = 0, --在指定位置回血计时
		smallActTime = math.random(4,8), --小动作时间，如眨眼
		focusTime = 0, --蓄力时间，等待时蓄完力可以瞬移，2秒开始蓄3秒。
		focusAct = false, --开始蓄力动作
		focusEffect = nil, --蓄力时光效
		focus = false, --蓄力完成
		focusBuff = nil, --完成时光效
	}
	--锁定城堡
	self.lockCity = false
	--触摸状态
	self.selected = false
	--正在打的怪
	self.attackEnemy = nil
	--减少计算
	self.interval = 0
	--攻击间隔
	self.attackInterval = 0
	--动画
	self.attackAni = display.getAnimationCache(self.data.actAttack)
	self.deadAni = display.getAnimationCache(self.data.actDead)
	self.hurtAni = display.getAnimationCache(self.data.actHurt)
	self.moveAni = display.getAnimationCache(self.data.actMove)
	self.magicAni = display.getAnimationCache(self.data.actMagic)
	self.waitAni = display.getAnimationCache(self.data.actWait)
	self.focusPreAni = display.getAnimationCache(self.data.actFocusPre)
	self.focusAni = display.getAnimationCache(self.data.actFocus)
	if self.data.actMagic2 ~= nil then
		self.magic2Ani = display.getAnimationCache(self.data.actMagic2)
	end
	if self.data.actWait2 ~= nil then
		self.wait2Ani = display.getAnimationCache(self.data.actWait2)
	end
	--英雄状态
	self:changeState("wait") --有self.state有wait,move,attack,dead,skilling。
	--正在放技能
	self.skilling = nil
	--正在攻击时不放受击动画
	self.isAttacking = false

	--数值
	self.maxHp = ExcelHeroData[self.idx][self.level].hp
	self.hp = self.maxHp
	self.speed = ExcelHeroData[self.idx][self.level].speed * MapInfo.tileSize
	self.attack = ExcelHeroData[self.idx][self.level].attack
	if self.idx == 1 then
		self.excelSpeed = self.speed
		self.excelAttack = self.attack
	end
	self.attackSpeed = self.data.attackSpeed
	self.rangeSq = math.pow(MapInfo.tileSize*self.data.range, 2)

	--头上血条
	self.hpBarHead = HpBar.new(HPBAR_TYPE_HERO_HEAD, cc.size(0,115), self.maxHp)
	self:addChild(self.hpBarHead)
	self.hpBarHead:setVisible(false)

	--血条，加在技能栏上
	local hpBarPos
	if VERSION_MONSTER then
		hpBarPos = cc.p(display.cx + 254, 23.5)
	else
		hpBarPos = cc.p(display.cx + 283, 15)
	end
	self.hpBar = HpBar.new(HPBAR_TYPE_HERO, cc.size(90,0), self.maxHp, hpBarPos)
	g_playScene.skillLayer:addChild(self.hpBar,10)
	--新手引导
	if NEW_GUIDE and GameData.guide.level1 == false and g_playScene.level == 101 then
		self.hpBar:setVisible(false)
	end
end

function Hero:clearFocusStatus()
	self.waitInfo.focusTime = 0
	self.waitInfo.focusAct = false
	self.waitInfo.focus = false
	if self.waitInfo.focusBuff ~= nil then
		self.waitInfo.focusBuff:removeFromParent()
		self.waitInfo.focusBuff = nil
	end
	if self.waitInfo.focusEffect ~= nil then
		self.waitInfo.focusEffect:removeFromParent()
		self.waitInfo.focusEffect = nil
	end
end

function Hero:setAnchor(pt)
	self.anchorPt = pt
	self:setFlip(self.heroPic:isFlippedX(), true)
end

--左右翻时设锚点，第二个参数是强制设锚点。
function Hero:setFlip(b, setAnchor)
	self.heroPic:setFlippedX(b)
	if setAnchor or self.anchorPt.x ~= 0.5 then
		if b then
			self.heroPic:setAnchorPoint(1-self.anchorPt.x, self.anchorPt.y)
		else
			self.heroPic:setAnchorPoint(self.anchorPt.x, self.anchorPt.y)
		end
	end
end

--放技能前摇
function Hero:showBeforeSkill(skill)
	local layer = display.newColorLayer(cc.c4b(0,0,0,200))
	g_fightLayer:addChild(layer, BaseZOrder.hero-1)
	self.heroPic:runAction(transition.sequence({
		cc.CallFunc:create(function()
			g_playScene:suspend(true)
			g_playScene:setCanTouch(false)
		end),
		cc.Animate:create(self.magicAni),
		cc.CallFunc:create(function()
			self.heroPic:setSpriteFrame(display.newSpriteFrame(self.data.pic))
			layer:removeFromParent()
			g_playScene:suspend(true)
			g_playScene:setCanTouch(true)
			skill:execute()
		end),
	}))
end

function Hero:changeState(state, notChangeValue)
	--print("changeState:",state, notChangeValue)
	if not notChangeValue then
		--改变值需注意
		if self.state == "attack" and self.isAttacking == true then
			self.isAttacking = false --正在打怪的时候放技能，或改打一个Boss怪。
		end
		self.state = state
	end
	self.heroPic:stopAllActions()
	if state == "wait" then
		self:changeAnchor(state)
		self.heroPic:playAnimationForever(self.waitAni)
	elseif state == "focus" then
		self:startFocus()
	elseif state == "focusAnger" then
		self:startFocusAnger()
	elseif state == "move" then
		self:changeAnchor(state)
		self.heroPic:playAnimationForever(self.moveAni)
	elseif state == "skilling" then
	elseif state == "attack" then
		self.heroPic:setSpriteFrame(display.newSpriteFrame(self.data.pic))
	elseif state == "dead" then
		self:changeAnchor(state)
		self.hpBarHead:setVisible(false)
	end
end

function Hero:changeAnchor(state)
	if state == "attack" and self.attackAnchor ~= nil then
		self:setAnchor(self.attackAnchor)
	elseif state == "wait" and self.waitAnchor ~= nil then
		self:setAnchor(self.waitAnchor)
	elseif state == "wait2" and self.wait2Anchor ~= nil then
		self:setAnchor(self.wait2Anchor)
	elseif state == "move" and self.moveAnchor ~= nil then
		self:setAnchor(self.moveAnchor)
	elseif state == "dead" and self.deadAnchor ~= nil then
		self:setAnchor(self.deadAnchor)
	end
	--print("anchor for",state,self.heroPic:getAnchorPoint().x, self.heroPic:getAnchorPoint().y)
end

function Hero:startSkill(skill)
	self.skilling = skill
	if self.moveDestPic ~= nil then
		self.moveDestPic:removeFromParent()
		self.moveDestPic = nil
	end
	if self.attackEnemy ~= nil then --在打英雄的怪停止，不然可能会延时stop英雄。
		self.attackEnemy.enemyPic:stopAllActions()
		self.attackEnemy.enemyPic:playAnimationForever(self.attackEnemy.moveAct)
	end
	self:changeState("skilling")
	self:clearFocusStatus()
end

function Hero:endSkill()
	--放技能时不让之前打的怪跑走，放完技能再解锁。
	if self.attackEnemy ~= nil then
		self.attackEnemy:unlockByHero()
		self.attackEnemy:unlockByHero()
		self.attackEnemy = nil
	end
	self.skilling = nil
	self:changeState("wait")
end

function Hero:redrawLine(p1,p2)
    MapInfo:adjustPosInMapRange(p2)
	if self.moveDestPic == nil then
		self.moveDestPic = display.newSprite("#hero_dest.png")
		self.moveDestPic:setAnchorPoint(cc.p(0.5,0.5))
		self.moveDestPic:setPosition(p2)
		self:getParent():addChild(self.moveDestPic, BaseZOrder.hero-1)
		local destsign = display.newSprite("#hero_destsign.png")
		destsign:setPosition(44,50)
		self.moveDestPic:addChild(destsign)
		destsign:runAction(cc.RepeatForever:create(cc.Sequence:create(cc.MoveBy:create(0.5, cc.p(0,15)), cc.MoveBy:create(0.5, cc.p(0,-15)))))
	else
		self.moveDestPic:setPosition(p2)
	end
	if self.moveLine then
		self.moveLine:removeFromParent()
	end
	self.moveLine = display.newSprite("#hero_moveline.png")
	if self.maxLine == nil then
		self.maxLine = self.moveLine:getContentSize().width
	end
	self.moveLine:setPosition(p1)
	self.moveLine:setAnchorPoint(cc.p(0,0.5))
	local dest = cc.p(p1.x - p2.x,
			 	p1.y - p2.y)
	local z = math.sqrt(dest.x*dest.x +dest.y*dest.y)
	local rt = kkFunc.getLineRotation(p1, p2)
	self.moveLine:setRotation(-90+rt)
	self.moveLine:setScaleX(z/self.maxLine)	
	self:getParent():addChild(self.moveLine, BaseZOrder.hero-1)
end

function Hero:moveToEn(en)
	if self.lockCity and g_fightLayer.selectedCity ~= nil then
	else
		en:lockByHero(self)
	end
	local enX,enY = en:getPosition()
	local px, py = self:getPosition()

	if self.waitInfo.focus then --蓄力后自动找怪也瞬移
		if px > enX then
			if self.heroPic:isFlippedX() then
				self:setFlip(false)
			end
			self:setPosition(enX+70,enY)
		else
			if not self.heroPic:isFlippedX() then
				self:setFlip(true)
			end
			self:setPosition(enX-70,enY)
		end
		self:changeState("attack")
	else
		if px > enX then
			self:moveTo(enX+70,enY)
		else
			self:moveTo(enX-70,enY)
		end
	end
end

--TEST
--[[function Hero:HeroMoveDestPic(p)
	if self.moveDestPic == nil then
		self.moveDestPic = display.newSprite("#hero_dest.png")
		self.moveDestPic:setAnchorPoint(cc.p(0.5,0.5))
		self.moveDestPic:setPosition(p)
		self:getParent():addChild(self.moveDestPic, BaseZOrder.hero-1)
		local destsign = display.newSprite("#hero_destsign.png")
		destsign:setPosition(44,50)
		self.moveDestPic:addChild(destsign)
		destsign:runAction(cc.RepeatForever:create(cc.Sequence:create(cc.MoveBy:create(0.5, cc.p(0,15)), cc.MoveBy:create(0.5, cc.p(0,-15)))))
	else
		self.moveDestPic:setPosition(p)
	end
end
]]
function Hero:moveTo(x,y)
	local px,py = self:getPosition()
	local dist = cc.pGetDistance(cc.p(px,py), cc.p(x,y))
	if dist == 0 then
		self.moveTime = 0
		self.moveSpeedX = 0
		self.moveSpeedY = 0
	else
		self.moveTime = dist/self.speed
		self.moveSpeedX = self.speed * (x-px) / dist
		self.moveSpeedY = self.speed * (y-py) / dist
	end
	if x - px > 0 and not self.heroPic:isFlippedX() then
		self:setFlip(true)
	elseif x - px < 0 and self.heroPic:isFlippedX() then
		self:setFlip(false)
	end
	self:changeState("move")
end

--蓄力
function Hero:startFocus()
    self.heroPic:playAnimationOnce(self.focusPreAni, false, function()
	    self.heroPic:playAnimationForever(self.focusAni)
    end)
    self.waitInfo.focusEffect = display.newSprite("#heroFocusEffect1.png",0,65)
    self:addChild(self.waitInfo.focusEffect,-1)
    local frames = display.newFrames("heroFocusEffect%d.png", 1, 6)
    local ani = display.newAnimation(frames, 0.1)
	self.waitInfo.focusEffect:playAnimationForever(ani)
end
--狂暴蓄力
function Hero:startFocusAnger()
    self.heroPic:playAnimationOnce(self.focusPreAni, false, function()
	    self.heroPic:playAnimationForever(self.focusAni)
    end)
    self.startAngerEffect = display.newSprite("#heroFocusEffect1.png",0,65)
    self:addChild(self.startAngerEffect,-1)
    local frames = display.newFrames("heroFocusEffect%d.png", 1, 6)
    local ani = display.newAnimation(frames, 0.1)
	self.startAngerEffect:playAnimationForever(ani)
end
--瞬移
function Hero:focusMove(x,y)
	if self.attackEnemy ~= nil then
		self.attackEnemy:unlockByHero()
		self.attackEnemy = nil
	end
	local px,py = self:getPosition()
	if x > px and not self.heroPic:isFlippedX() then
		self:setFlip(true)
	elseif x < px and self.heroPic:isFlippedX() then
		self:setFlip(false)
	end
	self:setPosition(x,y)
	if self.moveDestPic ~= nil then
		self.moveDestPic:removeFromParent()
		self.moveDestPic = nil
	end
	self:changeState("wait")
end

--[[
function Hero:instantMove(x,y)
	if self.attackEnemy ~= nil then
		self.attackEnemy:unlockByHero()
		self.attackEnemy = nil
	end
	self:changeState("skilling")
	self.heroPic:playAnimationOnce(self.magicAni,false,function()
		self:setPosition(x,y)
		if self.moveDestPic ~= nil then
			self.moveDestPic:removeFromParent()
			self.moveDestPic = nil
		end
		self:changeState("wait")
	end)
end
--]]

--TEST
--[[function Hero:onTouch(event, x, y)
	--死后、放技能时不能操作
	if self.state == "dead" or self.state == "skilling" then return end
	if event == "began" then
		self.moved = false
		local px,py = self:getPosition()
		local rect = cc.rect(px+self.rect.x,py+self.rect.y,self.rect.w,self.rect.h)
		if cc.rectContainsPoint(rect, cc.p(x,y)) then	
			if self.selected then
				self.selected = false
				if self.selectHeroSprite ~= nil then
					self.selectHeroSprite:removeFromParent()
					self.selectHeroSprite = nil
				end
			else
				self.selected = true
				self.selectHeroSprite = display.newSprite("#guideArrow.png",0,110)
				self:addChild(self.selectHeroSprite,5)
			end
			self.movedPoint = cc.p(x,y)
			return true
		end
	elseif event == "moved" then		
	elseif event == "ended" then
	end
end]]

function Hero:onTouch(event, x, y)
	--死后、放技能时不能操作
	if self.state == "dead" or self.state == "skilling" then return end
	if event == "began" then
		self.moved = false
		local px,py = self:getPosition()
		local rect = cc.rect(px+self.rect.x,py+self.rect.y,self.rect.w,self.rect.h)
		if cc.rectContainsPoint(rect, cc.p(x,y)) then
			self.selected = true
			self.movedPoint = cc.p(x,y)
			return true
		end
	elseif event == "moved" then
		self.moved = true
		self.movedPoint = cc.p(x,y)
		self:redrawLine(cc.p(self:getPosition()),cc.p(x,y))
	elseif event == "ended" then
		if not self.moved then return end
		--攻城模式
		if g_fightLayer.selectedCity ~= nil then
			local rect = g_fightLayer.selectedCity:getCascadeBoundingBox()
			if self.selected and cc.rectContainsPoint(rect, cc.p(x,y)) then
				self.lockCity = true
			else
				self.lockCity = false
			end
		end
		self.moved = false
		if self.moveLine ~= nil then
			self.moveLine:removeFromParent()
			self.moveLine = nil
		end
		self.selected = false
		local pt = cc.p(x,y)
		MapInfo:adjustPosInMapRange(pt)
		if self.waitInfo.focus then --瞬移
			self:focusMove(pt.x,pt.y)
		--elseif self.idx == 3 then	--希维尔被动,不用
		--	self:instantMove(pt.x,pt.y)
		else
			if self.lockCity and g_fightLayer.selectedCity ~= nil then
				self:moveToEn(g_fightLayer.selectedCity)	
			else
				self:moveTo(pt.x, pt.y)	
			end	
		end
		self.waitInfo.pos = cc.p(x,y)
		self.waitInfo.waiting = true
		self:clearFocusStatus()
		if self.attackEnemy ~= nil then
			self.attackEnemy:unlockByHero()
			self.attackEnemy = nil
		end
	end
end


function Hero:updateSub(dt)
end
--英雄移动时锁定视角相当于地图移动
function Hero:updateMapPosition(dt)
	if self.state == "dead" or self.state == "skilling" then
		return
	elseif self.state == "move" then
		local px, py = self:getPosition()
		px = px + self.moveSpeedX * dt
		py = py + self.moveSpeedY * dt
		self:setPosition(px,py)
		--[[if self.selected and self.moved then
			self:redrawLine(cc.p(self:getPosition()),self.movedPoint)
		end]]
		self.moveTime = self.moveTime - dt
		if self.moveTime <= 0 then
			if not self.moved and self.moveDestPic ~= nil then
				self.moveDestPic:removeFromParent()
				self.moveDestPic = nil
			end
			self:changeState("wait")
		end
	end 
end
--英雄狂暴
function Hero:heroAnger()
	self:changeState("focusAnger")
	self.angerTime = 15
	self.attack = self.attack*1.5
	self.attackSpeed = self.attackSpeed/1.25
	self.heroPic:runAction(cc.ScaleTo:create(0.3,1.4))
end
function Hero:cancleHeroAnger()
	self.angerTime = nil
	self.attack = self.attack/1.5
	self.attackSpeed = self.attackSpeed*1.25
	self.heroPic:setScale(1)
end
function Hero:update(dt)
	--取消道具效果
	if self.angerTime ~= nil then
		self.angerTime = self.angerTime - dt
		if self.angerTime < 14 and self.state == "focusAnger" then
			self:changeState("wait")
			self.startAngerEffect:removeFromParent()
		elseif self.angerTime < 0 then
			self:cancleHeroAnger()
		end
	end
	--英雄各自的处理
	self:updateSub(dt)

	--死后不能操作
	--技能状态下由技能做处理
	if self.state == "dead" or self.state == "skilling" then
		return
	elseif self.state == "move" then
		--玩家移动、锁定怪移动单独处理
		local px, py = self:getPosition()
		px = px + self.moveSpeedX * dt
		py = py + self.moveSpeedY * dt
		self:setPosition(px,py)
		if self.selected and self.moved then
			self:redrawLine(cc.p(self:getPosition()),self.movedPoint)
		end
		self.moveTime = self.moveTime - dt
		if self.moveTime <= 0 then
			--攻城模式,英雄首先攻击选中的城堡
			if self.lockCity and g_fightLayer.selectedCity ~= nil then
				--解锁普通怪
				if self.attackEnemy ~= nil then
					self.attackEnemy:unlockByHero()
					self.attackEnemy = nil
				end
				if not self.moved and self.moveDestPic ~= nil then
					self.moveDestPic:removeFromParent()
					self.moveDestPic = nil
				end
				--self:changeState("wait")
				self.attackInterval = self.attackSpeed - 0.2 --一上去就开始打。
				local cityX,cityY = g_fightLayer.selectedCity:getPosition()
				if cityX - px > 0 and not self.heroPic:isFlippedX() then
					self:setFlip(true)
				elseif cityX - px < 0 and self.heroPic:isFlippedX() then
					self:setFlip(false)
				end
				self:changeState("attack")				
			else
				if self.attackEnemy then --锁定怪物走过去。
					self.attackInterval = self.attackSpeed - 0.2 --一上去就开始打。
					local enX,enY = self.attackEnemy:getPosition()
					if enX - px > 0 and not self.heroPic:isFlippedX() then
						self:setFlip(true)
					elseif enX - px < 0 and self.heroPic:isFlippedX() then
						self:setFlip(false)
					end
					self:changeState("attack")
				else --玩家移动。
					if not self.moved and self.moveDestPic ~= nil then
						self.moveDestPic:removeFromParent()
						self.moveDestPic = nil
					end
					self:changeState("wait")
				end
			end
		end
	elseif self.state == "wait" or self.state == "focus" then
		--自动选怪攻击。
		self.interval = self.interval + dt
		if self.interval > 0.1 then
			if self.lockCity and g_fightLayer.selectedCity ~= nil then
				--解锁普通怪
				if self.attackEnemy ~= nil then
					self.attackEnemy:unlockByHero()
					self.attackEnemy = nil
				end
				--锁定城堡
				self:moveToEn(g_fightLayer.selectedCity)
				self.waitInfo.waiting = false
				self.waitInfo.waitingTime = 0
				self:clearFocusStatus()
				return
			else
				self.attackEnemy = self:getEnemy()
				if self.attackEnemy then
					--锁定怪
					self:moveToEn(self.attackEnemy)
					self.waitInfo.waiting = false
					self.waitInfo.waitingTime = 0
					self:clearFocusStatus()
					return
				end
			end
			self.interval = 0
		end
		--不是在上次指定位置等待
		self.waitInfo.waitingTime = self.waitInfo.waitingTime + dt

		if not self.waitInfo.waiting then
			--等待一会就回到上次指定位置
			if self.waitInfo.waitingTime > 2 then
				self:moveTo(self.waitInfo.pos.x, self.waitInfo.pos.y)
				self.waitInfo.waiting = true
			end
		else --在指定点待着，会回血，会做小动作
			self.waitInfo.hpTime = self.waitInfo.hpTime + dt
			if self.waitInfo.hpTime > 1 and self.hp < self.maxHp then
				self:updateHp(self.maxHp*0.05)
				self.waitInfo.hpTime = 0
			end
			self.waitInfo.smallActTime = self.waitInfo.smallActTime - dt
			if self.waitInfo.smallActTime <= 0 then
				self:doSmallAction()
				self.waitInfo.smallActTime = math.random(4,8)
			end
			--蓄力后可瞬移
			if not self.waitInfo.focus then
				self.waitInfo.focusTime = self.waitInfo.focusTime + dt
				if self.waitInfo.focusTime >= 5 then
					self.waitInfo.focus = true
					self.waitInfo.focusTime = 0
					self.waitInfo.focusEffect:removeFromParent()
					self.waitInfo.focusEffect = nil
					--蓄力完成光效
					--[[self.waitInfo.focusBuff = display.newSprite("#heroFocusLight.png",0,45)
					self:addChild(self.waitInfo.focusBuff,3)
					self.waitInfo.focusBuff:runAction(cc.RepeatForever:create(transition.sequence({
						cc.ScaleTo:create(0.5, 1.1),
						cc.ScaleTo:create(0.5, 1),
					})))]]
					self.waitInfo.focusBuff = display.newSprite("#heroFocusLight1.png",0,18)
				    self:addChild(self.waitInfo.focusBuff,-1)
				    local frames = display.newFrames("heroFocusLight%d.png", 1, 8)
				    local ani = display.newAnimation(frames, 0.1)
					self.waitInfo.focusBuff:playAnimationForever(ani)
					self:changeState("wait")
				elseif self.waitInfo.focusTime >= 2 and not self.waitInfo.focusAct then
					--开始做蓄力动作。
					self.waitInfo.focusAct = true
					self:changeState("focus")
				end
			end
		end
	elseif self.state == "attack" then
		if self.lockCity and g_fightLayer.selectedCity ~= nil then
		else
			if self.attackEnemy ~= nil then --有可能是空，在等攻击完后转wait。
				self.interval = self.interval + dt
				if self.interval > 0.1 then
					if self.attackEnemy.idx <= 40 then
						local en = self:getEnemy(true)
						if en then
							--解锁普通怪
							self.attackEnemy:unlockByHero()
							--锁定boss怪
							self.attackEnemy = en
							self:moveToEn(self.attackEnemy)
						end
					end
					self.interval = 0
				end
			end
		end
		self:updateAttackEnemy(dt)
	end
end

function Hero:getEnemy(onlyBoss)
	local enemy = nil
	local minDistSq = self.rangeSq
	for i = 1, #g_fightLayer.enemies do
		local en = g_fightLayer.enemies[i]
		local tornado = en.tornado --龙卷风
		local underWater = en.idx == 48 and en.underWater --水下萝卜
		local underE45 = en.idx == 45 and not en.hasChanged --地下的竹笋怪
		--local filledE44 = en.idx == 44 --只要是黄瓜怪就不挡 and en.filled --填坑的黄瓜怪
		if en.canHit and not tornado and not underWater and not underE45 then
			if not onlyBoss or en.idx > 40 then
				local disSq = cc.pDistanceSQ(cc.p(self:getPosition()), cc.p(en:getPosition()))
				if disSq < self.rangeSq then
					if en.idx > 40 then
						return en --boss直接返回
					elseif disSq < minDistSq then
						minDistSq = disSq
						enemy = en --普通怪取个最近的反正要遍历完。
					end
				end
			end
		end
	end
	return enemy
end

function Hero:updateAttackEnemy(dt)
	self.attackInterval = self.attackInterval + dt
	if self.attackInterval > self.attackSpeed then
		if self.isAttacking then --攻击频率比攻击动画还快时会出现。
			if self.lockCity and g_fightLayer.selectedCity ~= nil then
				g_fightLayer.selectedCity:updateHp(-self.attack)
			else
				if self.attackEnemy ~= nil then
					self.attackEnemy:updateHp(-self.attack,nil,true)
				end
			end
		else
			--音效
			SoundManager.playSound(GAME_SOUND.Hero[self.idx].attack)
			self.isAttacking = true
			self.heroPic:stopAllActions()
			self.heroPic:playAnimationOnce(self.attackAni, false, function()
				self.isAttacking = false
				if self.lockCity and g_fightLayer.selectedCity ~= nil then
					g_fightLayer.selectedCity:updateHp(-self.attack)
					self:changeState("wait", true)
				else
					if self.attackEnemy ~= nil then
						self.attackEnemy:updateHp(-self.attack,nil,true)
						self:changeState("wait", true) --只是做等待动作，不变状态。
						--[[
						if self.idx == 1 then --盖伦被动技能,10%暴击
							local d = math.floor(math.random(11,20)/10)
							self.attackEnemy:updateHp(-self.attack*d,nil,true)
						else
							self.attackEnemy:updateHp(-self.attack,nil,true)
						end
						--]]
					else
						self:changeState("wait") --怪没了变等待状态。
					end
				end
			end)
			self:showLightEffect()
		end
		self.attackInterval = 0
	end
end

function Hero:showLightEffect()
end

function Hero:lostEnemy()
	self.attackEnemy = nil
	if self.state == "skilling" or
			(self.state == "attack" and self.isAttacking == true) then --由攻击状态自然转换。
		return
	else
		self:changeState("wait")
	end
end

function Hero:doSmallAction()
	if self.data.actWait2 == nil then return end
	self.heroPic:stopAllActions()
	self:changeAnchor("wait2")
	self.heroPic:playAnimationOnce(self.wait2Ani, false, function()
		self:changeAnchor("wait")
		self.heroPic:stopAllActions()
		self.heroPic:playAnimationForever(self.waitAni)
	end)
end

function Hero:updateHp(hpAdd)
	self.hp = self.hp + hpAdd
	if self.hp > self.maxHp then
		self.hp = self.maxHp
	end
	--挂了直接返回。
	if self.hp <= 0 then
		if self.lowHpLight ~= nil then
			self.lowHpLight:removeFromParent()
			self.lowHpLight = nil
		end
		self.hp = 0
		self:die()
		self.hpBar:updateHp(self.hp)
		self.hpBarHead:setVisible(false)
		return
	elseif self.hp < self.maxHp*1/3 then --闪红
		if self.lowHpLight == nil then
			self.lowHpLight = display.newSprite("#heroHpEffect.png")
			g_playScene.skillLayer.heroIcon:addChild(self.lowHpLight)
			self.lowHpLight:setPosition(g_playScene.skillLayer.heroIcon:getContentSize().width/2,g_playScene.skillLayer.heroIcon:getContentSize().height/2)
			self.lowHpLight:setOpacity(0)
			self.lowHpLight:runAction(cc.RepeatForever:create(transition.sequence({
					cc.FadeIn:create(0.3),
					cc.FadeOut:create(0.3),
				})))
		end
	else--if self.hp >= self.maxHp*1/3 then
		if self.lowHpLight ~= nil then
			self.lowHpLight:removeFromParent()
			self.lowHpLight = nil
		end
	end
	--受击
	if hpAdd < 0 and self.hp > 0 then
		if self.state == "attack" and not self.isAttacking then
			self.heroPic:stopAllActions()
			self:changeAnchor("hurt")
			self.heroPic:playAnimationOnce(self.hurtAni, false, function()
				self.heroPic:stopAllActions()
				self:changeAnchor("wait")
				self.heroPic:playAnimationForever(self.waitAni)
			end)
		end
	end
	--盖伦被动技能,不用
	--[[
	if self.idx == 1 then
		local hpLevel = math.ceil(self.hp/ self.maxHp * 10)
		local percent = math.pow(1.2, 10 - hpLevel)
		self.speed = self.excelSpeed*percent
		self.attack = self.excelAttack*percent
	end
	--]]
	self.hpBar:updateHp(self.hp)
	self.hpBarHead:updateHp(self.hp)
end

function Hero:die()
	if self.moveDestPic ~= nil then
		self.moveDestPic:removeFromParent()
		self.moveDestPic = nil
	end
	if self.moveLine ~= nil then
		self.moveLine:removeFromParent()
		self.moveLine = nil
	end
	if self.state == "skilling" and self.skilling ~= nil then
		self.skilling:endSkill()
	end
	self:changeState("dead")

	self.heroPic:playAnimationOnce(self.deadAni, false, function()
		self.heroPic:runAction(cc.TintTo:create(1.5,128,128,128))
		local sprite = display.newSprite("#herodead1.png",50,50)
		self:addChild(sprite)
		sprite:setOpacity(0)
		local frames = display.newFrames("herodead%d.png", 1, 5)
		local animation = display.newAnimation(frames, 0.1)
		sprite:playAnimationForever(animation)
		sprite:runAction(transition.sequence({
			cc.Spawn:create(
				cc.MoveBy:create(0.5,cc.p(0,40)),
				cc.FadeTo:create(0.5,200)
			),
			cc.MoveBy:create(0.5,cc.p(0,40)),
			cc.Spawn:create(
				cc.FadeTo:create(0.5,0),
				cc.MoveBy:create(0.5,cc.p(0,40))
			),
			cc.CallFunc:create(function()
				sprite:removeFromParent()
			end)
		}))
		self:setLocalZOrder(BaseZOrder.fort + 1)
	end)

	--设初值
	self.selected = false
	if self.attackEnemy ~= nil then
		self.attackEnemy:unlockByHero()
		self.attackEnemy = nil
	end

	--重生冷却
	g_playScene.skillLayer:heroDeadProgress()
end

function Hero:reborn()
	self:setLocalZOrder(BaseZOrder.hero)
	self.hp = self.maxHp
	self:updateHp(self.maxHp - self.hp)
	self.heroPic:setColor(cc.c3b(255,255,255))
	self:changeState("wait")
end

function Hero:rebornEffect()
	local RebornEffect = display.newSprite()
	RebornEffect:setPosition(self:getPositionX(),self:getPositionY()+50)
	self:getParent():addChild(RebornEffect,BaseZOrder.hero)
	local animation = display.getAnimationCache("HeroReborn")
	RebornEffect:playAnimationOnce(animation,true)
end

return Hero