--
-- Author: Wangzhihao
-- Date: 2014-12-04 13:57:34
--
local MapInfo = import("..views.MapInfo")
local Skill = import(".Skill")
local Skill12 = class("Skill12", Skill)

function Skill12:ctor(params)
    -- 如果继承类覆盖了 ctor() 构造函数，那么必须手动调用父类构造函数
    -- 类名.super 可以访问指定类的父类
    Skill12.super.ctor(self, params)
    self.lasting = ExcelHeroSkillData[self.idx][GameData.heros["h4"].skill["s1"]].time
    self.vx = 0
    self.vy = 0
    self.enemies = {}
end

function Skill12:execute()
    --提示语
    self.tip = g_playScene:getTip({type = "skill12"})
    g_fightLayer:addChild(self.tip,BaseZOrder.ufo+1)
    --提示手 首次显示
    self.tipFinger = g_playScene:getTipFinger({type = "skill12"})
    if self.tipFinger then
        g_fightLayer:addChild(self.tipFinger,BaseZOrder.ufo+1)

    end
    g_playScene:changeSpeedNormal()
    self.hero:setAnchor(cc.p(0.38,0.2))
    self.heroPic:playAnimationForever(self.hero.waitAni)
    local dx = -150
    if self.heroPic:isFlippedX() then dx = 150 end
    local pt = cc.p(self.hero:getPositionX()+dx,self.hero:getPositionY()+150)
    MapInfo:adjustPosInMapRange(pt)

    --黑洞
    local heavenPic = display.newSprite("#s12_heaven_1.png",pt.x,pt.y)
    g_fightLayer:addChild(heavenPic,BaseZOrder.ufo)
    heavenPic:setScale(0)
    local heavenAni = display.getAnimationCache("h04_s12_heaven")
    heavenPic:runAction(transition.sequence({
        cc.ScaleTo:create(0.3,1),
        cc.CallFunc:create(function()       
            heavenPic:playAnimationForever(heavenAni)        
        end),
        cc.DelayTime:create(0.8),
        cc.ScaleTo:create(0.3,0),
        cc.CallFunc:create(function()
            heavenPic:removeFromParent()
        end)
    }))
    --飞碟掉下
    self.UFOPic = display.newSprite("#s12_dropout_1.png",pt.x,pt.y-60)
    g_fightLayer:addChild(self.UFOPic,BaseZOrder.ufo)
    local dropoutAni = display.getAnimationCache("h04_s12_dropout")
    self.UFOPic:playAnimationOnce(dropoutAni, false, function()
        --飞碟运行
        self.UFOPic:setSpriteFrame(display.newSpriteFrame("s12_fly_1.png"))
        local flyAni = display.getAnimationCache("h04_s12_fly")
        self.UFOPic:playAnimationForever(flyAni)
        --光
        local UFOLight = display.newSprite("#ufolight1.png",70,-30)
        self.UFOPic:addChild(UFOLight,-1)
        local lightAni = display.getAnimationCache("h04_s12_light")
        UFOLight:playAnimationForever(lightAni)
        --光圈
        local lightCircle = display.newSprite("#lightCircle.png",UFOLight:getContentSize().width/2+2,150)
        UFOLight:addChild(lightCircle)
        lightCircle:setScale(0.88)
        lightCircle:runAction(cc.RepeatForever:create(transition.sequence({
                cc.Spawn:create(
                    cc.ScaleTo:create(1,1.5),
                    cc.MoveBy:create(1,cc.p(0,-130))
                ),
                cc.CallFunc:create(function()
                    lightCircle:setPosition(UFOLight:getContentSize().width/2+2,150)
                    lightCircle:setScale(0.88)
                end)
            })))
    end, 0.3)     
    g_fightLayer:addSkill(self)

	cc.Device:setAccelerometerEnabled(true)
    local function didAccelerate(event,x,y,z,timestamp)
        self.vx = x*600
        self.vy = y*600
    end
    local listener = cc.EventListenerAcceleration:create(didAccelerate)
    cc.Director:getInstance():getEventDispatcher():addEventListenerWithSceneGraphPriority(listener, self)

    self.cover = cc.LayerColor:create(cc.c4b(0,0,0,200))
    self.cover:runAction(cc.FadeTo:create(0.2,100))
    g_fightLayer:addChild(self.cover, BaseZOrder.bullet+1)
    --音效
    self.soundHandler = SoundManager.playSound(GAME_SOUND.Hero[4].skill1,true)
end

function Skill12:update(dt)
	self.lasting = self.lasting - dt
    if self.lasting <= 0 then
        self:unscheduleUpdate()
        self.heroPic:runAction(transition.sequence({
            --英雄恢复原大小
            cc.ScaleTo:create(0.2,1),
            cc.CallFunc:create(function()
                self:endSkill()
            end)
        }))
    end

	local x,y = self.UFOPic:getPosition()
    x = x + self.vx*dt
    y = y + self.vy*dt
    local pt = cc.p(x,y)
    MapInfo:adjustPosInMapRange(pt)
	self.UFOPic:setPosition(pt)

    self:hitEnemy(pt)
    for i = #self.enemies, 1, -1 do
        if self.enemies[i] == 0 then
            table.remove(self.enemies, i)
        elseif self.enemies[i].time then
            self.enemies[i].time = self.enemies[i].time + dt
            if self.enemies[i].time > 0.1 then
                table.remove(self.enemies, i)
            end
        end
    end
end

function Skill12:hitEnemy(p)
    local UFORect = cc.rect(self.UFOPic:getPositionX()-45,self.UFOPic:getPositionY()-170,90,130)
    for i = 1, #g_fightLayer.enemies do
        local en = g_fightLayer.enemies[i]
        if en ~= nil then
            local enRect = en:getCascadeBoundingBox()
            local hasAttacked = false
            if cc.rectIntersectsRect(UFORect, enRect) then
                for j = 1,#self.enemies do
                    if en == self.enemies[j] then
                        hasAttacked = true
                        break
                    end
                end
                if not hasAttacked then
                    en:updateHp(-5)
                    if en ~= nil then
                        self.enemies[#self.enemies+1] = en
                        local idx = #self.enemies 
                        self.enemies[idx].time = 0
                    end
                end
            end
        end
    end
end

function Skill12:endSkill()
    --恢复怪物移动
    for i = 1 , #g_fightLayer.enemies do
        local en = g_fightLayer.enemies[i]
        en.pause = false
    end
    self.cover:runAction(cc.FadeOut:create(0.2))
	cc.Device:setAccelerometerEnabled(false)
    g_fightLayer:removeSkill(self)
    self.UFOPic:setVisible(false)
    --移除提示
    self.tip:runAction(cc.FadeOut:create(0.5)) 
    --移除手
    if self.tipFinger then
        self.tipFinger:removeFromParent()
    end
	self.hero:endSkill()
    g_playScene:setCanTouch(true)
    g_playScene:changeSpeedUser()
    self:performWithDelay(function()
        self:removeFromParent()
        self.UFOPic:removeFromParent()
    end, 1)
    SoundManager.stopSound(self.soundHandler)
end

return Skill12