--
-- Author: Wangzhihao
-- Date: 2015-04-27 13:32:08
--
local Heros = import("..obj.Heros")
local MapInfo = import("..views.MapInfo")
local HeroSkillLayer = import("..views.HeroSkillLayer")
local LoadingLayer = import("..views.LoadingLayer")
local FightLayer = import("..views.FightLayer")
local PlayScene = class("PlayScene", function()
    return display.newScene("PlayScene")
end)
PlaySceneOrder = {
    towerLayer = 2,
    skillLayer = 3,
    fightLayer = 3,
    towerCreateLayer = 4,
    selectLayer = 4,
    prop = 210,
    ice = 202,
    cloud = 202,
    storm = 202,
    starfish = 202,
    touchLayer = 100,
    ui = 200,
	resultLayer = 203,
    setting = 200,
}

g_fightLayer = nil
g_playScene = nil
function PlayScene:ctor(params)
	g_playScene = self
	self.level = params.level
	self.items = params.items or {}
	--加载资源
    self.res = ResData.getResFromLevelData(LevelsData[self.level])
    if LOADING_ASYNC == false then
        for k,v in pairs(self.res) do
            ResData.loadRes(v)
        end
    end
    --背景音乐
    SoundManager.playMusic(GAME_MUSIC.BGM2)

    -- touchLayer 用于接收触摸事件
    self.touchLayer = display.newLayer()
    self:addChild(self.touchLayer, PlaySceneOrder.touchLayer)
    --战斗层，要先于地图创建。
    self.fightLayer = FightLayer.new()
    g_fightLayer = self.fightLayer
    self:addChild(self.fightLayer, PlaySceneOrder.fightLayer)
    --加载地图
    self:loadMap()
    --hero
    self:showHero()
end

function PlayScene:loadMap()
    self.mapBgLayer = display.newLayer() 
    self:addChild(self.mapBgLayer,-10)   
    local themeNum = LevelsData[self.level].theme
    local theme = "map/Theme"..themeNum.."/theme"..themeNum.."Map.png"
    for i = 1,4 do
        local px,py
        local mapBg = display.newSprite(theme)
        if i == 1 then
            px = display.cx - mapBg:getContentSize().width/2
            py = mapBg:getContentSize().height/2 + display.cy
        elseif  i == 2 then
            px = display.cx - mapBg:getContentSize().width/2
            py = display.cy - mapBg:getContentSize().height/2
        elseif  i == 3 then
            px = display.cx + mapBg:getContentSize().width/2
            py = mapBg:getContentSize().height/2 + display.cy
        else
            px = display.cx + mapBg:getContentSize().width/2
            py = display.cy - mapBg:getContentSize().height/2
        end
        mapBg:setPosition(px,py)
        self.mapBgLayer:addChild(mapBg,-10)    
        MapInfo.mapBg = mapBg
    end   
    --[[MapInfo.mapBg = display.newSprite(theme,px,py)
    self.mapBgLayer:addChild(MapInfo.mapBg,-10)]]
    MapInfo.map = cc.TMXTiledMap:create(LevelsData[self.level].map)
    MapInfo.map:setAnchorPoint(cc.p(0.5,0.5))
    MapInfo.map:setPosition(cc.p(display.cx,display.cy))
    MapInfo.map:setVisible(false)
    self.mapBgLayer:addChild(MapInfo.map,-1)
    self.map = MapInfo.map
    
    MapInfo.mapPos = cc.p(MapInfo.map:getContentSize().width*0.5 - display.cx,
                          MapInfo.map:getContentSize().height*0.5 - display.cy)
    MapInfo:init()

    --守护点
    local endPos = cc.p(display.cx,display.cy)
    self.treaPic = display.newSprite("#protectPoint.png", endPos.x, endPos.y+30)
    self.mapBgLayer:addChild(self.treaPic)
    --红心
    self.heartPic = display.newSprite("#blood-1.png", endPos.x+8, endPos.y+80)
    self.mapBgLayer:addChild(self.heartPic)
    self.heartNum = MAX_HEART
    self.heartLabel = cc.ui.UILabel.new({
        UILabelType = 1,
        text = ""..self.heartNum,
        font = "fontTitleNumber.fnt",
        x = 48,
        y = 17,
    }):align(display.CENTER)
    self.heartLabel:setScale(0.8)
    self.heartPic:addChild(self.heartLabel)
end

function PlayScene:showHero()
    self.skillLayer = HeroSkillLayer.new(GameData.selectHero,self.items)
    self:addChild(self.skillLayer, PlaySceneOrder.skillLayer)
    local pos = cc.p(display.cx, display.cy)
    self.hero = Heros[GameData.selectHero].new({
        idx = GameData.selectHero,
        pos = pos,
    })
    g_fightLayer:addChild(self.hero, BaseZOrder.hero)
end
function PlayScene:onTouch(event, x, y)
	if self.hero then
        if event == "began" and self.hero:onTouch("began", x,y) then 
            return 
        elseif event == "moved" and self.hero.selected then
            return self.hero:onTouch("moved", x,y)
        elseif event == "ended" and self.hero.selected then
            return self.hero:onTouch("ended", x,y)
        end
    end
    if event == "began" then
        self.beginPos = cc.p(x,y)
        if self.hero.selected then
            self.hero:HeroMoveDestPic(self.beginPos)
            self.hero:moveTo(self.beginPos.x, self.beginPos.y)
        end
        return true
    elseif event == "moved" then 
        local subPos = cc.pSub(self.beginPos,cc.p(x,y))
        self.mapBgLayer:setPosition(self.mapBgLayer:getPositionX()-subPos.x,self.mapBgLayer:getPositionY()-subPos.y)
        local pt = cc.p(self.mapBgLayer:getPosition())
        if MapInfo:adjustMapPosInSceneRange(pt) then
            self.mapBgLayer:setPosition(pt)
        else
            self.hero:setPosition(self.hero:getPositionX()-subPos.x,self.hero:getPositionY()-subPos.y)    
            if self.hero.moveDestPic ~= nil then
                self.hero.moveDestPic:setPosition(self.hero.moveDestPic:getPositionX()-subPos.x,self.hero.moveDestPic:getPositionY()-subPos.y)
            end
        end                   
        self.beginPos = cc.p(x,y)
    elseif event == "ended" then
    end
end

function PlayScene:update(dt)
	--英雄也添加在fightLayer层上但不写在里面刷新了。
    if self.hero then
        self.hero:updateMapPosition(dt)
    end
end

function PlayScene:onEnter()
    -- 注册touch事件处理函数
    self.touchLayer:addNodeEventListener(cc.NODE_TOUCH_EVENT,function(event)
        return self:onTouch(event.name, event.x, event.y)
    end)
    --self.playSceneCanTouch = true
    self.touchLayer:setTouchEnabled(true)
    self.touchLayer:setTouchSwallowEnabled(false)
    -- 注册帧事件处理函数
    --self.updateInterval = 1/60
    --self.callInterval = 0
    self:addNodeEventListener(cc.NODE_ENTER_FRAME_EVENT,function(dt) 
        self:update(dt) 
    end)
    self:scheduleUpdate()
end

function PlayScene:onExit()
    self:unscheduleUpdate()
    --self.touchLayer:removeAllEventListeners()
end
return PlayScene