--
-- Author: KK
-- Date: 2014-09-28 17:34:15
--
--支线玩法中的礼盒

local AUTO_COLLECT = true

local MapInfo = import("..views.MapInfo")
local Enemys = import("..obj.Enemys")

local Gift = class("Gift", function()
    return display.newSprite("#gift.png")
end)

function Gift:ctor(params)
	self:setAnchorPoint(cc.p(0.5,0.2))

	self.wayIdx = params.wayIdx
	self.wayPoint = 1
	self.moveDist = 0
	self.needUpdate = true

	--朝向随机
	self.orient = "left"
	if math.random(0,1) == 1 then
		self.orient = "right"
		self:setFlippedX(true)
	end

	--减少update消耗
	self.newPos = cc.p(0,0)
	self.interval = 0

	if string.sub(params.giftIdx,1,1) == "E" then
		self.giftType = "enemy"
		self.idx = tonumber(string.sub(params.giftIdx,2,-1))
	elseif string.sub(params.giftIdx,1,1) == "T" then
		self.giftType = "tower"
		self.idx = tonumber(string.sub(params.giftIdx,2,-1))
	else
		self.giftType = "random"		
	end

	self.speed = MapInfo.tileSize*params.speed
end

function Gift:init()
end

function Gift:playOpenAnimation()
end

function Gift:open()
	if self.giftType == "enemy" then
		self:openEnemy()
	elseif self.giftType == "tower" then
		self:openRabbit()
	elseif self.giftType == "random" then
		local r = math.random(1,10)
		if r > 5 then
			self.idx = math.random(1,7)
			self:openEnemy()
		else
			self.idx = math.random(1,18)
			self:openRabbit()
		end
	end
	g_fightLayer:removeGift(self)
end

function Gift:openEnemy()
	local pos = cc.p(self:getPosition())
    local en = Enemys[self.idx].new({
        enemyIdx = self.idx,
        wayIdx = self.wayIdx,
        pos = pos})
    --要重置怪的所在路段，行进距离。
    en:getMoveDist()
    g_fightLayer:addEnemy(en)
end

function Gift:openRabbit()
	local pos = cc.p(self:getPosition())
	local card = display.newSprite("#towerCardBg.png")
	card:setPosition(pos)
	local size = card:getContentSize()
	local sprite = display.newSprite("#"..TowersData[self.idx].showPic)
	sprite:setPosition(cc.p(size.width*0.5,size.height*0.5))
	card:addChild(sprite)
	card:runAction(transition.sequence({
		cc.JumpBy:create(0.5, cc.p(math.random(-20,20),math.random(-20,20)), MapInfo.tileSize*1.5, 1),
		cc.CallFunc:create(function()
			if AUTO_COLLECT then
				g_fightLayer:addCardToTowerLayer(card)
			end
		end)
	}))
	card.towerIdx = self.idx
	g_fightLayer:addRabbitCard(card)
end

function Gift:update(dt)
	if not self.needUpdate then return end

	--走到终点就返回false结束了。。
	if not self:updateForMove(dt) then return end
end

function Gift:updateForMove(dt)
	--根据不同buff取一个速度
	self.currentSpeed = self.speed

	if not self:setMovePosition(dt) then
		return false 
	end

	self:getMoveDist(dt)

	return true
end

function Gift:setMovePosition(dt)
	local x,y = self:getPosition()
	--兔女郎退回到起点时。
	if self.wayPoint == 0 then
		self.wayPoint = 1
	end
	local orient = MapInfo.ways[self.wayIdx].wayOrient[self.wayPoint]
	--local targetPt = MapInfo:getCenterOfTile(MapInfo.ways[self.wayIdx].way[self.wayPoint])
	local targetPt = MapInfo.wayPoints[self.wayIdx][self.wayPoint]
	self.newPos.x = x
	self.newPos.y = y
	if not RANDOM_ENEMY_DIST then
		self.randmDist = 0
	else
		self.randmDist = self.randmDist or math.random(-MapInfo.tileSize*0.25,MapInfo.tileSize*0.25)
	end
	if orient == "right" then
		if x > targetPt.x + self.randmDist then
			self.newPos.x = targetPt.x+self.randmDist
			self.randmDist = nil
			self.wayPoint = self.wayPoint + 1
		else
			self.newPos.x = self.newPos.x + self.currentSpeed*dt
		end
		if self.orient == "left" then
			self.orient = "right"
			self:setFlippedX(true)
		end
	elseif orient == "left" then
		if x < targetPt.x+self.randmDist then
			self.newPos.x = targetPt.x+self.randmDist
			self.randmDist = nil
			self.wayPoint = self.wayPoint + 1
		else
			self.newPos.x = self.newPos.x - self.currentSpeed*dt
		end
		if self.orient == "right" then
			self.orient = "left"
			self:setFlippedX(false)
		end
	elseif orient == "up" then
		if y > targetPt.y+self.randmDist then
			self.newPos.y = targetPt.y+self.randmDist
			self.randmDist = nil
			self.wayPoint = self.wayPoint + 1
		else
			self.newPos.y = self.newPos.y + self.currentSpeed*dt
		end
	elseif orient == "down" then
		if y < targetPt.y+self.randmDist then
			self.newPos.y = targetPt.y+self.randmDist
			self.randmDist = nil
			self.wayPoint = self.wayPoint + 1
		else
			self.newPos.y = self.newPos.y - self.currentSpeed*dt
		end
	end

	if self.wayPoint > #MapInfo.ways[self.wayIdx].wayOrient then
		if g_playScene ~= nil then
			g_playScene:setHeart(g_playScene.heartNum - 1)
		end
		g_fightLayer:removeGift(self)
		return false
	end
	
	self:setPosition(self.newPos)
	return true
end

function Gift:getMoveDist(dt)
	if dt ~= nil then
		self.moveDist = self.moveDist + self.currentSpeed*dt
	else
		---消耗太大 可以偶尔用。
		local parent = g_fightLayer
		local idx = 1
		self.moveDist = 0
		while idx < self.wayPoint do
			self.moveDist = self.moveDist + 
				cc.pGetDistance(MapInfo.ways[self.wayIdx].startPt, MapInfo.wayPoints[self.wayIdx][idx])
			idx = idx + 1
		end
		if idx == 1 then
			self.moveDist = self.moveDist 
				+ cc.pGetDistance(cc.p(self:getPosition()), MapInfo:getCenterOfTile(MapInfo.ways[self.wayIdx].startPt))
		else
			self.moveDist = self.moveDist
				+ cc.pGetDistance(cc.p(self:getPosition()), MapInfo:getCenterOfTile(MapInfo.wayPoints[self.wayIdx][idx-1]))
		end
	end
	if self.moveDist > MapInfo.giftLines[self.wayIdx].dist then
		self:open()
	end
end

return Gift
