--
-- Author: KK
-- Date: 2014-08-05 14:55:38
--
--龙卷风

local TEST_TORNADO = true
if device.platform == "ios" or device.platform == "android" then
	TEST_TORNADO = false
end

local MapInfo = import("..views.MapInfo")

local Tornado = class("Tornado", function()
    return display.newSprite("#tornado_1.png")
end)

function Tornado:ctor(params)
	self:setAnchorPoint(cc.p(0.5,0.2))
	self:setPosition(params.from)

    local frames = display.newFrames("tornado_%d.png", 1, 6)
    local animation = display.newAnimation(frames, 0.1)
    self:playAnimationForever(animation)

    self.vx = 0
    self.vy = 0

    self.tornadoTime = params.time or 10
    self.isReady = false

	local nearPos = MapInfo:getNearestWayTilePos(params.from)
    --local wayPos = MapInfo:getNearestWayPos(params.from)
    self:runAction(transition.sequence({
    	cc.EaseInOut:create(cc.MoveTo:create(0.3,nearPos),0.65),
    	cc.CallFunc:create(function()
    		self.isReady = true
    	end)
    }))

    self.interval = 0
    self.enemies = {}

    --test
    if TEST_TORNADO then
	    self.timeCount = 1
	    self.timeRecord = 0
	    self.timeStamp = {3,3,3,3,3,3}
	    self.speeds = {
	    	[1] = {
	    		x = 0,
	    		y = -100,
	    	},
	    	[2] = {
	    		x = -100,
	    		y = -100,
	    	},
	    	[3] = {
	    		x = 100,
	    		y = 100,
	    	},
	    	[4] = {
	    		x = 0,
	    		y = 100,
	    	},
	    	[5] = {
	    		x = 80,
	    		y = 100,
	    	},
	    	[6] = {
	    		x = -100,
	    		y = 100,
	    	},
		}
	    self.vx = self.speeds[self.timeCount].x
	    self.vy = self.speeds[self.timeCount].y
	end
end

function Tornado:update(dt)
	if not self.isReady then return end
	self.tornadoTime = self.tornadoTime - dt
	if self.tornadoTime <= 0 then
		g_fightLayer:removeTornado(self)
		return
	end

	--for test
    if TEST_TORNADO then
		if self.timeCount < #self.timeStamp then
			self.timeRecord = self.timeRecord + dt
			if self.timeRecord >= self.timeStamp[self.timeCount] then
				self.timeCount = self.timeCount + 1
			    self.vx = self.speeds[self.timeCount].x
			    self.vy = self.speeds[self.timeCount].y
			    self.timeRecord = 0
			end
		end
	else
		self.vx = g_fightLayer.vx
		self.vy = g_fightLayer.vy
	end

	local tilePt = MapInfo:getTilePositionInCCP(cc.p(self:getPosition()))
	local tileCenter = MapInfo:getCenterOfTile(tilePt)
	local px,py = self:getPosition()
	--print("pos:",px,py,"   center:",tileCenter.x,tileCenter.y, "   vx,vy:",self.vx,self.vy)
	
	local function checkNextWay(dx,dy)
		local hasRealTileWay = MapInfo.tileType[MapInfo:getTileIdx(cc.p(tilePt.x+dx,tilePt.y+dy))] ~= nil
			and MapInfo.tileType[MapInfo:getTileIdx(cc.p(tilePt.x+dx,tilePt.y+dy))].wayDist ~= nil
		if not hasRealTileWay then return false end
		local d1 = MapInfo.tileType[MapInfo:getTileIdx(cc.p(tilePt.x,tilePt.y))].wayDist
		local d2 = MapInfo.tileType[MapInfo:getTileIdx(cc.p(tilePt.x+dx,tilePt.y+dy))].wayDist
		for i = 1, #MapInfo.ways do
			if d1[i] ~= nil and d2[i] ~= nil then
				return math.abs(d1[i]-d2[i]) == 1
			end
		end
		return false
	end

	local closeToX = math.abs(px - tileCenter.x) < MapInfo.tileSize/5
	local closeToY = math.abs(py - tileCenter.y) < MapInfo.tileSize/5

	if closeToX and not closeToY then --上下移动
		--(not closeToY or math.abs(self.vx) <= math.abs(self.vy)) then 
		if self.vy > 0 then
			if py + self.vy*dt <= tileCenter.y or checkNextWay(0,-1) then
				py = py + self.vy*dt
			else
				py = tileCenter.y
			end
		else
			if py + self.vy*dt >= tileCenter.y or checkNextWay(0,1) then
				py = py + self.vy*dt
			else
				py = tileCenter.y
			end
		end
		px = tileCenter.x
	elseif closeToY and not closeToX then --左右移动
		--(not closeToX or math.abs(self.vx) >= math.abs(self.vy)) then
		if self.vx > 0 then
			if px + self.vx*dt <= tileCenter.x or checkNextWay(1,0) then
				px = px + self.vx*dt
			else
				px = tileCenter.x
			end
		else
			if px + self.vx*dt >= tileCenter.x or checkNextWay(-1,0) then
				px = px + self.vx*dt
			else
				px = tileCenter.x
			end
		end
		py = tileCenter.y
	elseif closeToX and closeToY then --在拐角处，哪边有力就往哪边，不分大小。
		local moved = false
		--先判断速度快的方向。如果没动，再判断再慢的。
		if math.abs(self.vx) >= math.abs(self.vy) then
			if (self.vx > 0 and checkNextWay(1,0))
					or (self.vx < 0 and checkNextWay(-1,0)) then
				px = px + self.vx*dt
				py = tileCenter.y
				moved = true
			end
			if not moved then
				if (self.vy > 0 and checkNextWay(0,-1))
						or (self.vy < 0 and checkNextWay(0,1)) then
					py = py + self.vy*dt
					px = tileCenter.x
					moved = true
				end
			end
		else
			if (self.vy > 0 and checkNextWay(0,-1))
					or (self.vy < 0 and checkNextWay(0,1)) then
				py = py + self.vy*dt
				px = tileCenter.x
				moved = true
			end
			if not moved then
				if (self.vx > 0 and checkNextWay(1,0))
						or (self.vx < 0 and checkNextWay(-1,0)) then
					px = px + self.vx*dt
					py = tileCenter.y
					moved = true			
				end
			end
		end
	end

	self:setPosition(cc.p(px,py))

	--print("pos:",px,py)

	self.interval = self.interval + dt
	if self.interval > 0.1 then
		self:updateForEnemy()
		self.interval = 0
	end
end

function Tornado:updateForEnemy()
	local px,py = self:getPosition()
	local enemies = g_fightLayer:getEnimiesAroundPos(
		cc.p(self:getPosition()), math.pow(MapInfo.tileSize,2), true)
	for i = 1, #enemies do
		if enemies[i].idx ~= 61 and enemies[i].idx ~= 62 and enemies[i].idx ~= 63 and enemies[i].idx ~= 64 then
			if not enemies[i].tornado and enemies[i].needUpdate then
				self.enemies[#self.enemies+1] = enemies[i]
				enemies[i].tornadoTime = 0
				enemies[i].tornado = self
				enemies[i]:clearAllState()
				--print("Tornado:updateForEnemy()",#self.enemies)
			end
		end
	end
end

function Tornado:clear()
	--print("Tornado:clear()",#self.enemies)
	local tilePt = MapInfo:getTilePositionInCCP(cc.p(self:getPosition()))
	local pos = MapInfo:getCenterOfTile(tilePt)

	--有可能way1上的怪被卷到way2上。所以要取一个当前的wayIdx。给怪重设路径。
	local nowWayIdx = 1
	for i = 1, #MapInfo.ways do
		local dist = MapInfo.tileType[MapInfo:getTileIdx(tilePt)].wayDist[i]
		if dist ~= nil then
			nowWayIdx = i
			--print("idx,dist",nowWayIdx,dist)
			break
		end
	end

	--往前均匀分布
	local wayPoint = MapInfo:getWayPointByPos(nowWayIdx, pos)
	local orient = MapInfo.ways[nowWayIdx].wayOrient[wayPoint]
	local dx,dy = 0,0
	if #self.enemies >= 2 then
		if orient == "up" then
			dy = -20
		elseif orient == "down" then
			dy = 20
		elseif orient == "right" then
			dx = -20
		elseif orient == "left" then
			dx = 20
		end
		if #self.enemies > 3 then
			dx = dx * 70/(#self.enemies-1) / 20
			dy = dy * 70/(#self.enemies-1) / 20
		end
	end
	local px,py = 0,0
	for k,v in pairs(self.enemies) do
		if v ~= 0 then
			v.tornado = nil
			v.tornadoTime = nil
			v:setPosition(pos.x+px,pos.y+py)
			--print(self.enemies[i].wayIdx,pos.x,pos.y)
			v.wayIdx = nowWayIdx
    		--要重置怪的所在路段，行进距离。
			v:getMoveDist()
			--print("v.wayPoint:",v.wayPoint)
			px = px + dx
			py = py + dy
		end
	end
	self:removeFromParent()
	self = nil
end

return Tornado