--
-- Author: KK
-- Date: 2014-06-11 10:56:25
--

local CloudLayer = class("CloudLayer", function()
    return display.newNode()
end)

function CloudLayer:ctor(pic)
    self.couldTouchLayer = display.newLayer()
    self:addChild(self.couldTouchLayer)
    self.couldTouchLayer:addNodeEventListener(cc.NODE_TOUCH_EVENT,function(event)
            return self:cloudTouch(event)
        end)
    self.couldTouchLayer:setTouchEnabled(true)
    self.couldTouchLayer:setTouchSwallowEnabled(true)
    g_playScene:changeSpeedNormal()

    if USE_PIC_PEN then
        self.finger = display.newSprite("pic/pen.png")
        self.finger:setScale(1.2)
        self.finger:retain()
        self.finger:setBlendFunc(gl.ZERO, gl.ONE_MINUS_SRC_ALPHA)
    else
        self.pen = cc.DrawNode:create()
        self.pen:drawDot(cc.p(0,0), 40, cc.c4f(1,0,0,1))
        self.pen = self.finger
        self.pen:retain()
        self.pen:setBlendFunc(gl.ONE_MINUS_SRC_COLOR, gl.ONE_MINUS_SRC_ALPHA)
    end

    local cloud = display.newSprite(pic)
    cloud:setScale(2)
    local size = cloud:getContentSize()
    size = cc.size(display.width,display.height)
    --擦除分块记录
    self.matrix = {}
    local unitSize = 120
    self.matrix.row = math.ceil(size.height/unitSize)
    self.matrix.col = math.ceil(size.width/unitSize)
    self.matrix.picSize = size
    self.matrix.size = self.matrix.row *self.matrix.col
    self.matrix.rowUnit = size.height/self.matrix.row
    self.matrix.colUnit = size.width/self.matrix.col
    --print("row,col:",self.matrix.row,self.matrix.col)
    for i = 1,self.matrix.size do 
        self.matrix[i] = 1
    	---[[
        local row = math.ceil(i / self.matrix.col)
        local col = i - (row-1)*self.matrix.col
        if row == 1 or row == self.matrix.row
            or col == 1 or col == self.matrix.col then
            self.matrix[i] = 0
        else
            self.matrix[i] = 1
        end
    	--]]
    end
    self.clearRecord = {}

    cloud:setPosition(size.width*0.5,size.height*0.5)
    self.render = cc.RenderTexture:create(
        size.width, size.height,
        cc.TEXTURE2_D_PIXEL_FORMAT_RGB_A8888)
    self.render:setPosition(display.cx,display.cy)
    self.render:begin()
    cloud:visit()
    self.render:endToLua()
    self:addChild(self.render,1000)

    --提示语
    local typeName = "cloud"
    if pic == "pic/stormFog.png" then typeName = "storm" end
    self.tip = g_playScene:getTip({type = typeName})
    self:addChild(self.tip,1001)

    --提示手 首次显示
    self.tipFinger = g_playScene:getTipFinger({type = "cloud"})
    if self.tipFinger then
        self:addChild(self.tipFinger,1001)
    end

    --定时销毁
    self:performWithDelay(function()
        self:cloudGone()
    end, 10)
end

function CloudLayer:clearPoint(x,y,midPoints)
	--[[全屏雾不用算区域
    local rect = self.render:getCascadeBoundingBox()
    local px = rect.origin.x
    local py = rect.origin.y
    local subPos = cc.pSub(cc.p(x,y),cc.p(px,py))
    if subPos.x < 0 or subPos.x > self.matrix.picSize.width then return end
    if subPos.y < 0 or subPos.y > self.matrix.picSize.height then return end
	--]]

    if not self.render:isVisible() then return end

    local subPos = cc.p(x,y)

    self.render:begin()
    if USE_PIC_PEN then
        self.finger:setPosition(x,y)
        self.finger:visit()
        if midPoints ~= nil then
            for i = 1, #midPoints do
                local finger = display.newSprite("pic/pen.png")
                finger:setScale(1.2)
                finger:setBlendFunc(gl.ZERO, gl.ONE_MINUS_SRC_ALPHA)
                finger:setPosition(midPoints[i])
                finger:visit()
            end
        end
    else
        self.pen:setPosition(subPos)
        self.pen:visit()
        if midPoints ~= nil then
            for i = 1, #midPoints do
                local pen = cc.DrawNode:create()
                pen:drawDot(cc.p(0,0), 30, cc.c4f(1,0,0,1))
                pen:setBlendFunc(gl.ONE_MINUS_SRC_COLOR, gl.ONE_MINUS_SRC_ALPHA)
                pen:setPosition(midPoints[i])
                pen:visit()
            end
        end
    end
    self.render:endToLua()

    local ry = math.ceil(subPos.x / self.matrix.colUnit)
    local rx = math.ceil(subPos.y / self.matrix.rowUnit)
    --print(rx,ry)
    local idx = (rx-1)*self.matrix.col+ry
    --print(idx)
    if self.matrix[idx] == 1 then
        self.matrix[idx] = 0
        --print("clear "..idx)
        local notclear = 0
        for i = 1, self.matrix.size do
            if self.matrix[i] == 1 then
                notclear = notclear + 1
            end
        end
        --print("not clear:",notclear)
        if notclear <= 5 then
            self:cloudGone()
        end
        return true
    end
    return false
end

function CloudLayer:updateLeftCloud(dt)
    self.colorNum = self.colorNum - dt*100
    if self.colorNum < 0 then --消失殆尽
        self:unscheduleUpdate()
        self:getParent():removeCloud()
    else --渐变
        self.render:getSprite():setColor(cc.c3b(self.colorNum,self.colorNum,self.colorNum))
        self.render:getSprite():setOpacity(self.colorNum)
        self.scale = self.scale + dt*(1/2.5)
        if self.scale < 1 then return end
        self.render:getSprite():setFlippedY(true)
        self.render:getSprite():setScale(self.scale)
    end
end

function CloudLayer:cloudGone()
    --self.couldTouchLayer:removeNodeEventListener(cc.NODE_TOUCH_EVENT)
    --self.couldTouchLayer:removeFromParent()
    if self.gone then return end
    self.gone = true
    self:stopAllActions() --取消10秒自毁。
    --移除提示
    self.tip:runAction(cc.FadeOut:create(1)) 
    --移除手
    if self.tipFinger then
        self.tipFinger:removeFromParent()
    end
    --文字提示
    --self:getParent():cloudHasGone()

    self.colorNum = 255
    self.scale = 0.9
    self:addNodeEventListener(cc.NODE_ENTER_FRAME_EVENT, function(dt)
        self:updateLeftCloud(dt)
    end)
    self:scheduleUpdate()
    --最后再取消触摸会闪退。所以提前取消。
    self.couldTouchLayer:removeNodeEventListenersByEvent(cc.NODE_TOUCH_EVENT)
    g_playScene:changeSpeedUser()
end

function CloudLayer:inCloud(x,y)
    local ry = math.ceil(x / self.matrix.colUnit)
    local rx = math.ceil(y / self.matrix.rowUnit)
    local idx = (rx-1)*self.matrix.col+ry
	for i = 1, #self.clearRecord do
		--print("incloud?:",self.clearRecord[i],idx)
		if self.clearRecord[i] == idx then
			return false
		end
	end
	return true
end

function CloudLayer:cloudTouch(event)
    if event.name == "began" then
    local rect = self.render:getCascadeBoundingBox()
        if not cc.rectContainsPoint(rect, cc.p(event.x,event.y)) then return end
        self:clearPoint(event.x,event.y)
        self.lastX = event.x
        self.lastY = event.y
        return true
    elseif event.name == "moved" then
        local midPoints = {}
        if self.lastX ~= nil and self.lastY ~= nil then
            local newPos = cc.p(event.x,event.y)
            local oldPos = cc.p(self.lastX,self.lastY)
            local dist = cc.pGetDistance(newPos, oldPos)
            local n = math.ceil(dist/30)
            local sub = cc.pSub(newPos,oldPos)
            local dx = sub.x/n
            local dy = sub.y/n
            for i = 1, n-1 do
                local midPos = cc.p(oldPos.x+dx*i,oldPos.y+dy*i)
                midPoints[#midPoints+1] = midPos
            end
        end
        self:clearPoint(event.x, event.y, midPoints)
        self.lastX = event.x
        self.lastY = event.y
    elseif event.name == "ended" then
    	self.clearRecord = {}
        for i = 1, self.matrix.size do
            if self.matrix[i] == 0 then
                self.clearRecord[#self.clearRecord+1]=i
            end
        end
        self.lastX = nil
        self.lastY = nil
    end
end

return CloudLayer