--
-- Author: KK
-- Date: 2014-09-15 11:30:07
--
--兔子预览

local PREVIEW_WIDTH = 300
local PREVIEW_HEIGHT = 440

local RabbitType = {
	[5] = {
		map = "map/Preview/PreviewWater.tmx",
		fortPos = cc.p(2,2),
		enemy = {1,2,3}
	},
	[6] = {
		map = "map/Preview/PreviewWater.tmx",
		fortPos = cc.p(2,2),
		enemy = {4,5,6}
	},
	[10] = {
		map = "map/Preview/PreviewE45.tmx",
		fortPos = cc.p(2,1),
		enemy = {},
		fortOnce = true
	},
	[12] = {
		map = "map/Preview/PreviewWater.tmx",
		fortPos = cc.p(1,2),
		enemy = {1}
	},
	[14] = {
		map = "map/Preview/PreviewWater.tmx",
		fortPos = cc.p(2,1),
		enemy = {1}
	},
	[17] = {
		map = "map/Preview/PreviewWater.tmx",
		fortPos = cc.p(1,2),
		enemy = {1,1,1}
	},
	[23] = {
		map = "map/Preview/PreviewWater.tmx",
		fortPos = cc.p(2,1),
		enemy = {1,2,3}
	},
	--默认
	[100] = {
		map = "map/Preview/Preview.tmx",
		fortPos = cc.p(1,2),
		enemy = {1,2,3}
	}
}
local RADIO_BUTTON_IMAGES = {
    off = "#RadioButtonOff.png",
    off_pressed = "#RadioButtonOffPressed.png",
    off_disabled = "#RadioButtonOffDisabled.png",
    on = "#RadioButtonOn.png",
    on_pressed = "#RadioButtonOnPressed.png",
    on_disabled = "#RadioButtonOnDisabled.png",
}

local Enemys = import("..obj.Enemys")
local Forts = import("..obj.Forts")
local FightLayer = import(".FightLayer")
local MapInfo = import(".MapInfo")

local RabbitPreview = class("RabbitPreview", function()
	return display.newLayer()
end)

function RabbitPreview:ctor()
	local bg = cc.LayerColor:create(cc.c4b(70,160,90,255),PREVIEW_WIDTH,PREVIEW_HEIGHT)
	self:addChild(bg,1)

	self.idx = nil
	self.level = nil
	self.speed = nil
	self.speedTime = 0
	self.fightLayer = nil
	self.map = nil
	self.needUpdate = false

	self:addNodeEventListener(cc.NODE_ENTER_FRAME_EVENT, function(dt)
		self:update(dt)
	end)
	self:scheduleUpdate()
end

function RabbitPreview:getButtonGroup()
	if self.buttonGroup ~= nil then
		self.buttonGroup:removeFromParent()
	end
    self.buttonGroup = cc.ui.UICheckBoxButtonGroup.new(display.LEFT_TO_RIGHT)
        :addButton(cc.ui.UICheckBoxButton.new(RADIO_BUTTON_IMAGES)
            :setButtonLabel(cc.ui.UILabel.new({text = "L1", color = display.COLOR_BLACK}))
            :setButtonLabelOffset(-12, -25)
            :align(display.CENTER)
        	:setButtonSelected(true))
        :addButton(cc.ui.UICheckBoxButton.new(RADIO_BUTTON_IMAGES)
            :setButtonLabel(cc.ui.UILabel.new({text = "L2", color = display.COLOR_BLACK}))
            :setButtonLabelOffset(-12, -25)
            :align(display.CENTER))
        :addButton(cc.ui.UICheckBoxButton.new(RADIO_BUTTON_IMAGES)
            :setButtonLabel(cc.ui.UILabel.new({text = "L3", color = display.COLOR_BLACK}))
            :setButtonLabelOffset(-12, -25)
            :align(display.CENTER))
        :setButtonsLayoutMargin(0, 30, 0, 30)
        :align(display.RIGHT_TOP, 20, 150)
        :addTo(self,4)
    self.buttonGroup:align(display.CENTER)
    --选中正要升的等级按钮
    local button = self.buttonGroup:getButtonAtIndex(self.level)
    button:setButtonSelected(true)
    --再注册事件
    self.buttonGroup:onButtonSelectChanged(function(event)
    	if event.selected > 0 then
    		self.level = event.selected
    		self:getRabbitData()
    		self:showLevel()
    	end
        --printf("Option %d selected, Option %d unselected", event.selected, event.last)
    end)
end

function RabbitPreview:showLevel()
	if self.content.fortOnce then return end
	self:createFort()
end

function RabbitPreview:setRabbit(idx)
	if self.idx == idx then return end

	self.needUpdate = false
	self.idx = idx
	self.level = GameData.myRabbit["r"..idx].level + 1
	self:getRabbitData()
	self:getButtonGroup()
	--获取类别内容
	if RabbitType[self.idx] == nil then
		self.content = RabbitType[100]
	else
		self.content = RabbitType[self.idx]
	end

	if self.map ~= nil then
		self.fightLayer:removeFromParent()
		self.map:removeFromParent()
		self.map = nil
	end
	self:loadMap()
	self.fightLayer = FightLayer.new()
	self:addChild(self.fightLayer,3)
    g_fightLayer = self.fightLayer
	self.needUpdate = true

	if self.content.fortOnce then --一次性的一开始不造塔，等update自动创建。
		--竹笋
		self:createE45Enemy()
		return
	end
	self:createEnemy()
	self:createFort()

--[[
	if self.rabbit ~= nil then
		self.rabbit:removeFromParent()
		self.rabbit = nil
	end
	self.rabbit = display.newSprite("#"..TowersData[idx].pic)
	self.rabbit:setFlippedX(true)
	self.rabbit:setPosition(cc.p(50,300))
	self:addChild(self.rabbit)]]

end

function RabbitPreview:loadMap()
    MapInfo.map = cc.TMXTiledMap:create(self.content.map)
    MapInfo.map:setAnchorPoint(cc.p(0.5,0.5))
    MapInfo.map:setPosition(cc.p(PREVIEW_WIDTH*0.5,PREVIEW_HEIGHT*0.5+100))
    --MapInfo.map:setPosition(cc.p(display.cx,display.cy))

    self:addChild(MapInfo.map,2)
    self.map = MapInfo.map
	MapInfo.mapPos = cc.p(self.map:getContentSize().width*0.5 - PREVIEW_WIDTH*0.5,
						  self.map:getContentSize().height*0.5 - PREVIEW_HEIGHT*0.5-100)
    MapInfo:init()
end

function RabbitPreview:getRabbitData()
	self.speed = ExcelTowersData[self.idx].level[self.level].speed
	--竹笋的第一次建要马上
	self.noFortTime = self.speed-1
end

function RabbitPreview:createE45Enemy()
    for i = 1, #MapInfo.E45 do
        local en = Enemys[45].new({
            enemyIdx = 45,
            enHp = nil,
            wayIdx = MapInfo.E45[i].wayIdx,
            tilePt = MapInfo.E45[i].tilePt,
            pos = MapInfo.E45[i].pos
        })
        self.fightLayer:addEnemy(en)
        MapInfo.E45[i].enemy = en
        MapInfo.E45[i].created = true
        en:init()
		self.noEnemyTime = nil
    end
end

function RabbitPreview:createEnemy()
	for i = 1, #self.content.enemy do
		self:performWithDelay(function()
		    local start = MapInfo:getCenterOfTile(MapInfo.ways[1].startPt)
		    local en = Enemys[self.content.enemy[i]].new({
		        enemyIdx = self.content.enemy[i],
		        enHp = nil,
		        wayIdx = 1,
		        pos = start
		    })
		    self.fightLayer:addEnemy(en)
		    self.noEnemyTime = nil
		end, (i-1)*0.7)
	end
end

function RabbitPreview:createFort()
	local p = self.content.fortPos
	if self.fort ~= nil then
		self.fightLayer:removeFort(self.fort)
	end
    self.fort = Forts[self.idx].new({
        towerIdx = self.idx,
        leftRight = nil,--leftRight,
        level = self.level,
        tilePt = p,
    })
    self.fightLayer:addFort(self.fort)
    self.fort:init()
    self.fort:setFlippedX(true)
    self.noFortTime = nil
end

function RabbitPreview:update(dt)
	if not self.needUpdate then return end

	self.fightLayer:update(dt)

	if #self.fightLayer.enemies == 0 then
		if self.noEnemyTime == nil then
			self.noEnemyTime = 0
		else
			self.noEnemyTime = self.noEnemyTime + dt
			if self.noEnemyTime > 1 then
				self.noEnemyTime = nil
				self:createEnemy()
				if self.content.fortOnce then
					self:createE45Enemy()
				end
			end
		end
	end

	if #self.fightLayer.forts > 0 then 
		--自动放怒气技能。
		if self.fort.angerInfo.hasAnger and self.fort.angerInfo.angerState == 2 then
			self.fort:angerStart()
		end
	else
		if self.noFortTime == nil then
			self.noFortTime = 0
		else
			self.noFortTime = self.noFortTime + dt
			if self.noFortTime > self.speed then
				self.noFortTime = nil
				self:createFort()				
			end
		end
	end
end

return RabbitPreview