--
-- Author: KK
-- Date: 2014-04-03 17:51:53
--

local SelectTowerLayer = class("SelectTowerLayer", function()
    return display.newLayer()
end)

function SelectTowerLayer:ctor(towerList)
    --dump(towerList)
    self.bg = cc.LayerColor:create(cc.c4b(200, 100, 100, 255),display.width*0.5,display.height*0.8)
	self.bg:setAnchorPoint(cc.p(0.5,0.5))
	self.bg:ignoreAnchorPointForPosition(false)
	self.bg:setPosition(cc.p(display.cx, display.cy))
	self:addChild(self.bg)
	--self:setOpacity(128)

	self.contentLayer = cc.LayerColor:create(cc.c4b(100, 100, 200, 255),display.width*0.4,display.height*1)
	self.contentLayer:setAnchorPoint(cc.p(0.5,0.5))
	self.contentLayer:ignoreAnchorPointForPosition(false)
	self.contentLayer:setPosition(cc.p(display.cx, display.cy))

    self.towerList = towerList
    local picSize = 100
    local startX,startY = display.width*0.2 - picSize*2.5, display.height*1.1
    local rowSize = 4
    local rowNum = math.ceil(#self.towerList / rowSize)
    self.towers = {}
    for row = 1, rowNum do
    	for col = 1, rowSize do
    		local idx = (row-1)*rowSize+col
    		if idx > #self.towerList then break end
            local x = startX + picSize*col
            local y = startY - picSize*row
    		self.towers[idx] = display.newSprite("#"..TowersData[self.towerList[idx]].pic,x,y)
    		self.towers[idx].towerIdx = self.towerList[idx]
            if DEBUG_INFO then
                local idLabel = cc.ui.UILabel.new({
                    text = "T"..self.towerList[idx],
                    size = 15,
                    color = display.COLOR_BLACK,
                    x = 0,
                    y = 0,
                })
                self.towers[idx]:addChild(idLabel)
            end
            self.contentLayer:addChild(self.towers[idx],2)
    	end
    end
    self:addScrollView(self.contentLayer)
    --self:addChild(self.contentLayer)

    self.menu = cc.ui.UIPushButton.new({
        normal = PIC_RES.GreenButton,
        pressed = PIC_RES.GreenButton,
        disabled = PIC_RES.GreenButton,
    })
        :onButtonClicked(function(event)
            self:startGame()
        end)
        :pos(display.cx, display.height*0.2)
        :addTo(self)
    self.menu:setButtonEnabled(false)

    self.touchLayer = display.newLayer()
    self:addChild(self.touchLayer, 5)
    self.touchLayer:addNodeEventListener(cc.NODE_TOUCH_EVENT, function(event)
        return self:onTouch(event.name,event.x,event.y)
    end)
    self.touchLayer:setTouchEnabled(true)
    self.touchLayer:setTouchSwallowEnabled(false)
end

function SelectTowerLayer:chooseAction(idx)
	local canChoose = 8
	local chose = 0
	for i = 1, #self.towers do
		if self.towers[i].selected then
			chose = chose + 1
		end
	end
	if not self.towers[idx].selected then
		if chose < canChoose then
			self.towers[idx].selected = true
			self.towers[idx]:setScale(1.2)
			if chose + 1 == canChoose then
				self:onCanStart(true)
			end
		end
	else
		self.towers[idx].selected = false
		self.towers[idx]:setScale(1)
		self:onCanStart(false)
	end
end

function SelectTowerLayer:addScrollView(contentLayer)
    self.scrollView = cc.ui.UIScrollView.new({})
    self.scrollView:setTouchSwallowEnabled(false)
    self.scrollView:setColor(cc.c3b(230,143,233))
    local screenSize = cc.Director:getInstance():getWinSize()
    local function scrollView1DidScroll(event)
        if event.name == "moved" then
            self.preChoose = nil
        end
        local px, py = self.scrollView:getScrollNode():getPosition()
        --print("scrollView1DidScroll,",px,py)
    end
    local function scrollView1DidZoom()
        --print("scrollView1DidZoom")
    end
    local csize = self.contentLayer:getContentSize()
    if nil ~= self.scrollView then
        local scrollSize = contentLayer:getBoundingBox()
        --scrollSize.width = 400
        scrollSize.height = 350
        self.scrollView:setViewRect(scrollSize)
        self.scrollView:setPosition(0,200)
        self.scrollView:setScale(1.0)
        self.scrollView:setAnchorPoint(cc.p(0,0))
        self.scrollView:ignoreAnchorPointForPosition(false)
        self.scrollView:addScrollNode(contentLayer)
        self.scrollView:scrollTo(cc.p(480,0))
        self.scrollView:setDirection(1)
        --self.scrollView:setClippingToBounds(true)
        self.scrollView:setBounceable(true)
        self.scrollView:onScroll(scrollView1DidScroll)

    end
    self:addChild(self.scrollView,2)
end

function SelectTowerLayer:startGame()
	self:getParent():startReady()
end
function SelectTowerLayer:onCanStart(enable)
    self.menu:setButtonEnabled(enable)
end

function SelectTowerLayer:getSelected()
	local selected = {}
	for i = 1, #self.towers do
		if self.towers[i].selected then
			selected[#selected+1] = self.towers[i].towerIdx
		end
	end
	return selected
end

function SelectTowerLayer:onTouch(event, x, y)
    --print(event, x, y)
    if event == "began" then
        self.preChoose = nil
    	--local csize = self.contentLayer:getContentSize()
    	--local ssize = self.scrollView:getViewSize()
    	--print("sizeY:",csize.height,ssize.height)
    	for i = 1, #self.towers do
            --[[
    		local px,py = self.towers[i]:getPosition()
    		local ppx = (display.width - csize.width)/2+px
    		local ppy = (display.height - ssize.height)/2+py + self.scrollView:getContentOffset().y
    		local towerSize = self.towers[i]:getContentSize()
    		local rect = cc.rect(ppx-towerSize.width*0.5,ppy-towerSize.height*0.5,
    			towerSize.width, towerSize.height)
            --]]
            local rect = self.towers[i]:getCascadeBoundingBox()
    		if cc.rectContainsPoint(rect, cc.p(x,y)) then
    			self.preChoose = i
    			break
    		end    		
    	end
    	return true
    elseif event == "moved" then
    elseif event == "ended" then
    	if self.preChoose ~= nil then
    		self:chooseAction(self.preChoose)
    	end
	end
end

return SelectTowerLayer