--
-- Author: KK
-- Date: 2014-11-07 12:53:35
--
--点击地面出造塔列表

local MapInfo = import("..views.MapInfo")

local TowerCreateLayer = class("TowerCreateLayer", function()
	return display.newLayer("TowerCreateLayer")
end)

function TowerCreateLayer:ctor(towers)
	self.showLayer = display.newLayer()
	self:addChild(self.showLayer,1)
	local towersNum
	if g_playScene.mode == "randomTowerMode" then 
		towersNum = 1
    else
		towersNum = #towers
    end	
	self.row, self.col = 1, 4
	if towersNum > 4 then
		self.row = 2
	else
		self.col = towersNum
	end
	self.towers = {}
	for i = 1, towersNum do
		local towerPic,towerMoney,realTowerIdx
		if g_playScene.mode == "randomTowerMode" then 
			towerPic = "#lock.png"
			towerMoney = 120
			--print("jj",towers[math.random(1,#towers)])
			realTowerIdx = towers[math.random(1,#towers)]
	    else
			towerPic = "#"..TowersData[towers[i]].showPic
			towerMoney = ""..ExcelTowersData[towers[i]].level[1].money
	    	realTowerIdx = towers[i]
	    end
		local sprite = display.newSprite(towerPic)
		local moneyBg = display.newSprite("#fortMoneyBg.png")
		local label = cc.ui.UILabel.new({
            UILabelType = 1,
            text = towerMoney,
            font = "fontFortMoney.fnt",
        }):align(display.CENTER)
		self.towers[#self.towers+1] = {
			idx = realTowerIdx,
			sprite = sprite,
			enable = true,
			label = label,
		}
		local row, col = 1, i
		if i > 4 then
			col = col - 4
			row = 2
		end
		local x = (col-self.col/2-0.5) * 75
		local y = (1+self.row-row) * 80
		sprite:setPosition(x,y)
		self.showLayer:addChild(sprite)
		moneyBg:setPosition(x, y-30)
		self.showLayer:addChild(moneyBg)
		label:setPosition(x+10, y-36)
	    self.showLayer:addChild(label)
	end
	self.showLayer:setVisible(false)
	self.createTilePt = nil

	self.blank = display.newSprite("#blankArea.png")
	self:addChild(self.blank)
	self.blank:setScale(0.8)
	self.blank:setVisible(false)
end

function TowerCreateLayer:checkGroundShowList(tilePt)
	local blank = MapInfo:checkTileBlank(tilePt)
    local water = MapInfo:checkWater(tilePt)
    if not blank then return false end
    self.createTilePt = cc.p(tilePt)
	local pt = MapInfo:getCenterOfTile(self.createTilePt)
	self.blank:setPosition(pt)
	self.blank:setVisible(true)
	self.blank:runAction(cc.RepeatForever:create(transition.sequence({
		cc.ScaleTo:create(0.5, 0.7),
		cc.ScaleTo:create(0.5, 0.8),
	})))
	self:showTowers(tilePt)
	return true
end

function TowerCreateLayer:showTowers(tilePt)
	local showPt = MapInfo:getCenterOfTile(self.createTilePt)
	if tilePt.y == 0 or 
			(self.row == 2 and tilePt.y == 1) then
		showPt.y = showPt.y - 70 - 80*self.row
	end
	if tilePt.x == 1 then
		showPt.x = showPt.x + 75 * (self.col/2-0.5)
	elseif tilePt.x == 11 then
		showPt.x = showPt.x - 75 * (self.col/2-0.5)
	elseif self.col == 4 then --有四列会影响到第二和倒数第二列。
		if tilePt.x == 2 then
			showPt.x = showPt.x + 75 * 0.5
		elseif tilePt.x == 10 then
			showPt.x = showPt.x - 75 * 0.5
		end
	end
	self.showLayer:setPosition(showPt)
	self.showLayer:setVisible(true)
	local towerMoney
	for i = 1, #self.towers do
		local towerScale = 0.8
		if g_playScene.mode == "randomTowerMode" then
			self.towers[1].idx = LevelsData[g_playScene.level].selectedTowers[math.random(1,#LevelsData[g_playScene.level].selectedTowers)]
			towerMoney = 120
			towerScale = 1
		else
			towerMoney = ExcelTowersData[self.towers[i].idx].level[1].money
		end
		if self.towers[i].enable and 
				towerMoney > g_playScene.money then
			self.towers[i].enable = false
			self.towers[i].sprite:setColor(cc.c3b(128,128,128))
		elseif not self.towers[i].enable and 
				towerMoney <= g_playScene.money then
			self.towers[i].enable = true
			self.towers[i].sprite:setColor(cc.c3b(255,255,255))
		end
		self.towers[i].sprite:setScale(0)
		self.towers[i].sprite:runAction(transition.sequence({
			cc.ScaleTo:create(0.1,towerScale)
		}))
	end
end

function TowerCreateLayer:onTouch(x, y)
	for i = 1, #self.towers do
		local rect = self.towers[i].sprite:getCascadeBoundingBox()
		if cc.rectContainsPoint(rect, cc.p(x,y)) then
			if not self.towers[i].enable then
				return
			end
			self:create(self.towers[i].idx)
			--新手引导
			if g_playScene.level == 100 or (g_playScene.newbieguidelayer ~= nil and g_playScene.newbieguidelayer.isTeach) then
				g_playScene.newbieguidelayer:createTower(2)
			elseif (g_playScene.newbieguidelayer ~= nil and g_playScene.newbieguidelayer.rect ~= nil and g_playScene.level == 107) then
				g_playScene.newbieguidelayer:goLevel7(4)
			elseif (g_playScene.newbieguidelayer ~= nil and g_playScene.newbieguidelayer.rect ~= nil and g_playScene.level == 109) then
				g_playScene.newbieguidelayer:goLevel14(4)
			end
			break
		end
	end
	if not (NEW_GUIDE and g_playScene.level == 109 and g_playScene.newbieguidelayer ~= nil and g_playScene.newbieguidelayer.isCanTouchGround) then
		self.createTilePt = nil
		self.blank:stopAllActions()
		self.blank:setVisible(false)
		self.showLayer:setVisible(false)
	end
end

function TowerCreateLayer:create(idx)
	--防止在建造塔的瞬间E64怪物技能造塔冲突
	local pt = MapInfo:getTilePositionInCCP(cc.p(self.blank:getPosition()))
	local fort = g_playScene:getTileFort(pt)
	if fort == nil then
		g_playScene:createNewFort(self.createTilePt,idx)
	end
end

function TowerCreateLayer:changeMoney(money)
	if self.createTilePt == nil then return end
	for i = 1, #self.towers do
		local towerMoney = ExcelTowersData[self.towers[i].idx].level[1].money
		if g_playScene.mode == "randomTowerMode" then
			towerMoney = 120
		end 
		if towerMoney <= money then
			if not self.towers[i].enable then
				self.towers[i].enable = true
				self.towers[i].sprite:setColor(cc.c3b(255,255,255))
			end
		else
			if self.towers[i].enable then
				self.towers[i].enable = false
				self.towers[i].sprite:setColor(cc.c3b(128,128,128))
			end
		end
	end
end

return TowerCreateLayer